@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23980 $
+ * $Revision: 25893 $
  * $Author: rjongbloed $
- * $Date: 2010-01-24 19:30:23 -0600 (Sun, 24 Jan 2010) $
+ * $Date: 2011-05-25 20:31:43 -0500 (Wed, 25 May 2011) $
  */
 
 #ifndef OPAL_OPAL_H
@@ -80,7 +80,7 @@ typedef struct OpalHandleStruct * OpalHandle;
 typedef struct OpalMessage OpalMessage;
 
 /// Current API version
-#define OPAL_C_API_VERSION 21
+#define OPAL_C_API_VERSION 24
 
 
 ///////////////////////////////////////
@@ -369,7 +369,7 @@ typedef enum OpalMessageType {
   OpalCmdHoldCall,              /**<Place call in a hold state. The OpalMessage m_callToken field is set to
                                     the token returned in OpalIndIncomingCall. */
   OpalCmdRetrieveCall,          /**<Retrieve call from hold state. The OpalMessage m_callToken field is set
-                                    to the token returned in OpalIndIncomingCall. */
+                                    to the token for the call. */
   OpalCmdTransferCall,          /**<Transfer a call to another party. This starts the outgoing call process
                                     for the other party. See the  OpalParamSetUpCall structure for more
                                     information.*/
@@ -401,6 +401,18 @@ typedef enum OpalMessageType {
                                     OpalParamSetUpCall structure for more information. */
   OpalCmdAlerting,              /**<Send an indication to the remote that we are "ringing". The OpalMessage
                                     m_callToken field indicates which call is alerting.  */
+  OpalIndOnHold,                /**<Indicate a call has been placed on hold by remote. This message is returned
+                                    in the OpalGetMessage() function. */
+  OpalIndOffHold,               /**<Indicate a call has been retrieved from hold by remote. This message is
+                                    returned in the OpalGetMessage() function. */
+  OpalIndTransferCall,          /**<Status of transfer operation that is under way. This message is returned in
+                                    the OpalGetMessage() function. See the OpalStatusTransferCall structure for
+                                    more information. */
+  OpalIndCompletedIVR,          /**<Indicates completion of the IVR (VXML) script. This message is returned in
+                                    the OpalGetMessage() function. See the OpalStatusIVR structure for
+                                    more information. */
+
+// Always add new messages to ethe end to maintain backward compatibility
   OpalMessageTypeCount
 } OpalMessageType;
 
@@ -545,9 +557,13 @@ typedef struct OpalParamGeneral {
                                            This list of names (e.g. "G.723.1") is separated by the '\n'
                                            character. */
   const char * m_autoRxMedia;         /**< List of media types (e.g. audio, video) separated by spaces
-                                           which may automatically be received automatically. */
+                                           which may automatically be received automatically. If NULL
+                                           no change is made, but if "" then all media is prevented from
+                                           auto-starting. */
   const char * m_autoTxMedia;         /**< List of media types (e.g. audio, video) separated by spaces
-                                           which may automatically be transmitted automatically. */
+                                           which may automatically be transmitted automatically. If NULL
+                                           no change is made, but if "" then all media is prevented from
+                                           auto-starting.  */
   const char * m_natRouter;           /**< The host name or IP address of the Network Address Translation
                                            router which may be between the endpoint and the Internet. */
   const char * m_stunServer;          /**< The host name or IP address of the STUN server which may be
@@ -659,6 +675,19 @@ typedef struct OpalProductDescription {
 } OpalProductDescription;
 
 
+/**Type code for controlling the mode in which user input (DTMF) is sent.
+   This is used by the OpalCmdSetProtocolParameters command in the OpalParamProtocol structure.
+  */
+typedef enum OpalUserInputModes {
+  OpalUserInputDefault,   ///< Default mode for protocol
+  OpalUserInputAsQ931,    ///< Use Q.931 Information Elements (H.323 only)
+  OpalUserInputAsString,  ///< Use arbitrary strings (H.245 string, or INFO dtmf)
+  OpalUserInputAsTone,    ///< Use DTMF specific names (H.245 signal, or INFO dtmf-relay)
+  OpalUserInputAsRFC2833, ///< Use RFC 2833 for DTMF only
+  OpalUserInputInBand,    ///< Use in-band generated audio tones for DTMF
+} OpalUserInputModes;
+
+
 /**Protocol parameters for the OpalCmdSetProtocolParameters command.
    This is only passed to and returned from the OpalSendMessage() function.
 
@@ -681,7 +710,7 @@ typedef struct OpalParamProtocol {
                                            where they maybe set. */
   const char * m_userName;            /**< User name to identify the endpoint. This is usually the protocol
                                            specific name and may interact with the OpalCmdRegistration
-                                           command. e.g. "robertj or 61295552148 */
+                                           command. e.g. "robertj" or 61295552148 */
   const char * m_displayName;         /**< Display name to be used. This is the human readable form of the
                                            users name, e.g. "Robert Jongbloed". */
   OpalProductDescription m_product;   /**< Product description data */
@@ -689,7 +718,15 @@ typedef struct OpalParamProtocol {
                                            This list is separated by the '\n' character. If NULL no
                                            listeners are started or stopped. If and empty string ("")
                                            then all listeners are stopped. If a "*" then listeners
-                                           are started for all interfaces in the system. */
+                                           are started for all interfaces in the system.
+
+                                           If the prefix is "ivr", then this is the default VXML script
+                                           or URL to execute on incoming calls.*/
+  OpalUserInputModes m_userInputMode; /**< The mode for user input transmission. Note this only applies if an
+                                           explicit protocol is indicated in m_prefix. See OpalUserInputModes
+                                           for more information. */
+  const char * m_defaultOptions;      /**< Default options for new calls using the specified protocol. This
+                                           string is of the form key=value\nkey=value */
 } OpalParamProtocol;
 
 
@@ -948,6 +985,8 @@ typedef struct OpalStatusIncomingCall {
                                    Q.931 SIGNAL (0x34) Information Element. */
   const char * m_protocolCallId;  /**< ID assigned by the underlying protocol for the call. 
                                        Only available in version 18 and above */
+  const char * m_referredByAddress; ///< This is the full address of the party doing transfer, if available.
+  const char * m_redirectingNumber; ///< This is the E.164 number of the party doing transfer, if available.
 } OpalStatusIncomingCall;
 
 
@@ -1091,7 +1130,7 @@ typedef enum OpalVideoRecordMixMode {
 /**Call recording information for the OpalCmdStartRecording command.
   */
 typedef struct OpalParamRecording {
-  const char * m_callToken;  ///< Call token for call being cleared.
+  const char * m_callToken;  ///< Call token for call being recorded.
   const char * m_file;       /**< File to record into. If NULL then a test is done
                                   for if recording is currently active. */
   unsigned     m_channels;   /**< Number of channels in WAV file, 1 for mono (default) or 2 for
@@ -1112,6 +1151,48 @@ typedef struct OpalParamRecording {
 } OpalParamRecording;
 
 
+/**Call transfer information for the OpalIndTransferCall indication.
+   This is only returned from the OpalGetMessage() function.
+  */
+typedef struct OpalStatusTransferCall {
+  const char * m_callToken;       ///< Call token for call being transferred.
+  const char * m_protocolCallId;  /**< ID assigned by the underlying protocol for the call. 
+                                       Only available in version 18 and above */
+  const char * m_result;          /**< Result of transfer operation. This is one of:
+                                            "progress"  transfer of this call is still in progress.
+                                            "success"   transfer of this call completed, call will
+                                                        be cleared.
+                                            "failed"    transfer initiated by this call did not
+                                                        complete, call remains active.
+                                            "started"   remote system has asked local connection
+                                                        to transfer to another target.
+                                            "completed" local connection has completed the transfer
+                                                        to other target.
+                                            "forwarded" remote has forwarded call local system has
+                                                        initiated to another address.
+                                            "incoming"  this call is the target of an incoming
+                                                        transfer, e.g. party C in a consultation
+                                                        transfer scenario. */
+  const char * m_info;    /**< Protocol dependent information in the form:
+                                           key=value\n
+                                           key=value\n
+                                           etc */
+} OpalStatusTransferCall;
+
+
+/**IVR information for the OpalIndCompletedIVR indication.
+   This is only returned from the OpalGetMessage() function.
+  */
+typedef struct OpalStatusIVR {
+  const char * m_callToken;   ///< Call token for call being cleared.
+  const char * m_variables;   /**< Final values for variables defined by the script.
+                                   These will be in the form:
+                                           varname=value\n
+                                           varname=value\n
+                                           etc */
+} OpalStatusIVR;
+
+
 /**Call clearance information for the OpalIndCallCleared indication.
    This is only returned from the OpalGetMessage() function.
   */
@@ -1195,6 +1276,8 @@ struct OpalMessage {
     OpalStatusMediaStream    m_mediaStream;        ///< Used by OpalIndMediaStream/OpalCmdMediaStream
     OpalParamSetUserData     m_setUserData;        ///< Used by OpalCmdSetUserData
     OpalParamRecording       m_recording;          ///< Used by OpalCmdStartRecording
+    OpalStatusTransferCall   m_transferStatus;     ///< Used by OpalIndTransferCall
+    OpalStatusIVR            m_ivrStatus;          ///< Used by OpalIndCompletedIVR
   } m_param;
 };
 
@@ -1234,6 +1317,7 @@ class OpalMessagePtr
     OpalStatusMediaStream    * GetMediaStream() const;        ///< Used by OpalIndMediaStream/OpalCmdMediaStream
     OpalParamSetUserData     * GetSetUserData() const;        ///< Used by OpalCmdSetUserData
     OpalParamRecording       * GetRecording() const;          ///< Used by OpalCmdStartRecording
+    OpalStatusTransferCall   * GetTransferStatus() const;     ///< Used by OpalIndTransferCall
 
   protected:
     OpalMessage * m_message;
