@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23008 $
+ * $Revision: 24787 $
  * $Author: rjongbloed $
- * $Date: 2009-06-28 23:24:23 -0500 (Sun, 28 Jun 2009) $
+ * $Date: 2010-10-11 17:52:01 -0500 (Mon, 11 Oct 2010) $
  */
 
 #ifndef OPAL_OPAL_LOCALEP_H
@@ -113,7 +113,7 @@ class OpalLocalEndPoint : public OpalEndPoint
       const PString & party,     ///<  Remote party to call
       void * userData = NULL,    ///<  Arbitrary data to pass to connection
       unsigned int options = 0,  ///<  options to pass to conneciton
-      OpalConnection::StringOptions * stringOptions  = NULL
+      OpalConnection::StringOptions * stringOptions  = NULL ///< Options to pass to connection
     );
   //@}
 
@@ -127,7 +127,7 @@ class OpalLocalEndPoint : public OpalEndPoint
       */
     PSafePtr<OpalLocalConnection> GetLocalConnectionWithLock(
       const PString & token,     ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite
+      PSafetyMode mode = PSafeReadWrite   ///< Lock mode
     ) { return GetConnectionWithLockAs<OpalLocalConnection>(token, mode); }
 
     /**Create a connection for the PCSS endpoint.
@@ -136,8 +136,8 @@ class OpalLocalEndPoint : public OpalEndPoint
     virtual OpalLocalConnection * CreateConnection(
       OpalCall & call,    ///<  Owner of connection
       void * userData,    ///<  Arbitrary data to pass to connection
-      unsigned options,
-      OpalConnection::StringOptions * stringOptions
+      unsigned options,   ///< Option bit mask to pass to connection
+      OpalConnection::StringOptions * stringOptions ///< Options to pass to connection
     );
 
     /**Call back to indicate that remote is ringing.
@@ -179,7 +179,8 @@ class OpalLocalEndPoint : public OpalEndPoint
        connection.
       */
     virtual bool RejectIncomingCall(
-      const PString & token ///<  Token of connection to accept call
+      const PString & token,                 ///<  Token of connection to accept call
+      const OpalConnection::CallEndReason & reason = OpalConnection::EndedByAnswerDenied ///<  Reason for rejecting the call
     );
 
     /**Call back to indicate that the remote user has indicated something.
@@ -189,7 +190,7 @@ class OpalLocalEndPoint : public OpalEndPoint
       */
     virtual bool OnUserInput(
       const OpalLocalConnection & connection, ///<  Connection having event
-      const PString & indication
+      const PString & indication    ///< User input received
     );
 
     /**Call back to get media data for transmission.
@@ -258,10 +259,23 @@ class OpalLocalEndPoint : public OpalEndPoint
     /**Indicate OnAlerting() is be deferred or immediate.
       */
     bool IsDeferredAlerting() const { return m_deferredAlerting; }
+
+    /**Indicate OnAlerting() is be deferred or immediate.
+      */
+    void SetDeferredAlerting(bool defer) { m_deferredAlerting = defer; }
+
+    /**Indicate AcceptIncomingCall() execution is be deferred or immediate on OnIncomingCall().
+      */
+    bool IsDeferredAnswer() const { return m_deferredAnswer; }
+
+    /**Indicate AcceptIncomingCall() execution is be deferred or immediate on OnIncomingCall().
+      */
+    void SetDeferredAnswer(bool defer) { m_deferredAnswer = defer; }
   //@}
 
   protected:
     bool m_deferredAlerting;
+    bool m_deferredAnswer;
 
   private:
     P_REMOVE_VIRTUAL(OpalLocalConnection *, CreateConnection(OpalCall &, void *), 0);
@@ -284,9 +298,9 @@ class OpalLocalConnection : public OpalConnection
       OpalCall & call,              ///<  Owner call for connection
       OpalLocalEndPoint & endpoint, ///<  Owner endpoint for connection
       void * userData,              ///<  Arbitrary data to pass to connection
-      unsigned options,
-      OpalConnection::StringOptions * stringOptions,
-      char tokenPrefix = 'L'
+      unsigned options,             ///< Option bit mask to pass to connection
+      OpalConnection::StringOptions * stringOptions, ///< Options to pass to connection
+      char tokenPrefix = 'L'        ///< Prefix for token generation
     );
 
     /**Destroy connection.
@@ -467,7 +481,7 @@ class OpalLocalMediaStream : public OpalMediaStream, public OpalMediaStreamPacin
     );
 
     /**Indicate if the media stream is synchronous.
-       Returns PTrue for LID streams.
+       Returns true for LID streams.
       */
     virtual PBoolean IsSynchronous() const;
   //@}
