@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
- * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Revision: 25055 $
+ * $Author: csoutheren $
+ * $Date: 2011-01-12 02:07:20 -0600 (Wed, 12 Jan 2011) $
  */
 
 #ifndef OPAL_OPAL_MANAGER_H
@@ -38,13 +38,14 @@
 
 #include <opal/buildopts.h>
 
+#include <opal/pres_ent.h>
 #include <opal/call.h>
 #include <opal/connection.h> //OpalConnection::AnswerCallResponse
 #include <opal/guid.h>
-#include <opal/audiorecord.h>
 #include <codec/silencedetect.h>
 #include <codec/echocancel.h>
 #include <ptclib/pstun.h>
+#include <ptclib/url.h>
 
 #if OPAL_VIDEO
 #include <ptlib/videoio.h>
@@ -53,6 +54,7 @@
 class OpalEndPoint;
 class OpalMediaPatch;
 
+
 /**This class is the central manager for OPAL.
    The OpalManager embodies the root of the tree of objects that constitute an
    OPAL system. It contains all of the endpoints that make up the system. Other
@@ -144,18 +146,22 @@ class OpalManager : public PObject
        application to gain access to the call at later time. This is necesary
        as any pointer being returned could become invalid (due to being
        deleted) at any time due to the multithreaded nature of the OPAL
-       system. While this function does return a pointer it is only safe to
-       use immediately after the function returns. At any other time the
-       FindCallWithLock() function should be used to gain a locked pointer
-       to the call.
+       system. 
      */
+    virtual PSafePtr<OpalCall> SetUpCall(
+      const PString & partyA,       ///<  The A party of call
+      const PString & partyB,       ///<  The B party of call
+      void * userData = NULL,       ///<  user data passed to Call and Connection
+      unsigned options = 0,         ///<  options passed to connection
+      OpalConnection::StringOptions * stringOptions = NULL ///< Options to pass to connection
+    );
     virtual PBoolean SetUpCall(
       const PString & partyA,       ///<  The A party of call
       const PString & partyB,       ///<  The B party of call
       PString & token,              ///<  Token for call
       void * userData = NULL,       ///<  user data passed to Call and Connection
       unsigned options = 0,         ///<  options passed to connection
-      OpalConnection::StringOptions * stringOptions = NULL   ///<  complex string options passed to call
+      OpalConnection::StringOptions * stringOptions = NULL ///< Options to pass to connection
     );
 
     /**A call back function whenever a call is completed.
@@ -171,9 +177,9 @@ class OpalManager : public PObject
     );
 
     /**Determine if a call is active.
-       Return PTrue if there is an active call with the specified token. Note
+       Return true if there is an active call with the specified token. Note
        that the call could clear any time (even milliseconds) after this
-       function returns PTrue.
+       function returns true.
       */
     virtual PBoolean HasCall(
       const PString & token  ///<  Token for identifying call
@@ -185,10 +191,10 @@ class OpalManager : public PObject
 
 
     /**Determine if a call is established.
-       Return PTrue if there is an active call with the specified token and
+       Return true if there is an active call with the specified token and
        that call has at least two parties with media flowing between them.
        Note that the call could clear any time (even milliseconds) after this
-       function returns PTrue.
+       function returns true.
       */
     virtual PBoolean IsCallEstablished(
       const PString & token  ///<  Token for identifying call
@@ -204,7 +210,7 @@ class OpalManager : public PObject
       */
     PSafePtr<OpalCall> FindCallWithLock(
       const PString & token,  ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite
+      PSafetyMode mode = PSafeReadWrite ///< Lock mode
     ) { return activeCalls.FindWithLock(token, mode); }
 
     /**Clear a call.
@@ -239,7 +245,7 @@ class OpalManager : public PObject
       */
     virtual void ClearAllCalls(
       OpalConnection::CallEndReason reason = OpalConnection::EndedByLocalUser, ///<  Reason for call clearing
-      PBoolean wait = PTrue   ///<  Flag to wait for calls to e cleared.
+      PBoolean wait = true   ///<  Flag to wait for calls to e cleared.
     );
 
     /**A call back function whenever a call is cleared.
@@ -270,7 +276,6 @@ class OpalManager : public PObject
     virtual OpalCall * CreateCall(
       void * userData            ///<  user data passed to SetUpCall
     );
-    virtual OpalCall * CreateCall();
     OpalCall * InternalCreateCall();
 
     /**Destroy a call object.
@@ -285,10 +290,68 @@ class OpalManager : public PObject
       OpalCall * call
     );
 
-    /**Get next unique token ID for calls.
-       This is an internal function called by the OpalCall constructor.
+    /**Get next unique token ID for calls or connections.
+       This is an internal function called by the OpalCall and other
+       constructors.
       */
-    PString GetNextCallToken();
+    virtual PString GetNextToken(char prefix);
+  //@}
+
+  /**@name Presence management */
+  //@{
+    /**Add a presentity.
+       If the presentity is already present, a new one is not added, and the
+       existing instance is returned.
+
+       Returns a Read/Write locked pointer to presentity.
+      */
+    virtual PSafePtr<OpalPresentity> AddPresentity(
+      const PString & presentity  ///< Presentity URI
+    );
+
+    /**Get a presentity.
+      */
+    virtual PSafePtr<OpalPresentity> GetPresentity(
+      const PString & presentity,         ///< Presentity URI
+      PSafetyMode mode = PSafeReference   ///< Safety mode for presentity
+    );
+
+    /**Get all presentities.
+      */
+    virtual PStringList GetPresentities() const;
+
+    /**Remove a presentity.
+      */
+    virtual bool RemovePresentity(
+      const PString & presentity  ///< Presentity URI
+    );
+  //@}
+
+  /**@name Instant Messaging management */
+  //@{
+    /**Send an IM to a remote party.
+     */
+    virtual PBoolean Message(
+      const PString & to, 
+      const PString & body
+    );
+    virtual PBoolean Message(
+      const PURL & to, 
+      const PString & type,
+      const PString & body,
+      PURL & from, 
+      PString & conversationId
+    );
+    virtual PBoolean Message(
+      OpalIM & message
+    );
+
+    /**Called when text message received
+     */
+    virtual void OnMessageReceived(
+      const OpalIM & message
+    );
+
   //@}
 
   /**@name Connection management */
@@ -319,7 +382,7 @@ class OpalManager : public PObject
        This function usually returns almost immediately with the connection
        continuing to occur in a new background thread.
 
-       If PFalse is returned then the connection could not be established. For
+       If false is returned then the connection could not be established. For
        example if a PSTN endpoint is used and the associated line is engaged
        then it may return immediately. Returning a non-NULL value does not
        mean that the connection will succeed, only that an attempt is being
@@ -327,19 +390,28 @@ class OpalManager : public PObject
 
        The default behaviour is pure.
      */
-    virtual PBoolean MakeConnection(
+    virtual PSafePtr<OpalConnection> MakeConnection(
       OpalCall & call,                   ///<  Owner of connection
       const PString & party,             ///<  Party to call
       void * userData = NULL,            ///<  user data to pass to connections
       unsigned int options = 0,          ///<  options to pass to conneciton
-      OpalConnection::StringOptions * stringOptions = NULL
+      OpalConnection::StringOptions * stringOptions = NULL ///< Options to pass to connection
+    );
+
+    /**Call back for a new connection has been constructed.
+       This is called after CreateConnection has returned a new connection.
+       It allows an application to make any custom adjustments to the
+       connection before it begins to process the protocol. behind it.
+      */
+    virtual void OnNewConnection(
+      OpalConnection & connection   ///< New connection just created
     );
 
     /**Call back for answering an incoming call.
        This function is used for an application to control the answering of
        incoming calls.
 
-       If PTrue is returned then the connection continues. If PFalse then the
+       If true is returned then the connection continues. If false then the
        connection is aborted.
 
        Note this function should not block for any length of time. If the
@@ -364,14 +436,7 @@ class OpalManager : public PObject
     virtual PBoolean OnIncomingConnection(
       OpalConnection & connection,   ///<  Connection that is calling
       unsigned options,              ///<  options for new connection (can't use default as overrides will fail)
-      OpalConnection::StringOptions * stringOptions
-    );
-    virtual PBoolean OnIncomingConnection(
-      OpalConnection & connection,   ///<  Connection that is calling
-      unsigned options               ///<  options for new connection (can't use default as overrides will fail)
-    );
-    virtual PBoolean OnIncomingConnection(
-      OpalConnection & connection   ///<  Connection that is calling
+      OpalConnection::StringOptions * stringOptions ///< Options to pass to connection
     );
 
     /**Route a connection to another connection from an endpoint.
@@ -381,11 +446,31 @@ class OpalManager : public PObject
        variable and uses MakeConnection() to start the B-party connection.
       */
     virtual bool OnRouteConnection(
+      PStringSet & routesTried,     ///< Set of routes already tried
       const PString & a_party,      ///< Source local address
       const PString & b_party,      ///< Destination indicated by source
       OpalCall & call,              ///< Call for new connection
       unsigned options,             ///< Options for new connection (can't use default as overrides will fail)
-      OpalConnection::StringOptions * stringOptions
+      OpalConnection::StringOptions * stringOptions ///< Options to pass to connection
+    );
+
+    /**Call back for remote party is now responsible for completing the call.
+       This function is called when the remote system has been contacted and it
+       has accepted responsibility for completing, or failing, the call. This
+       is distinct from OnAlerting() in that it is not known at this time if
+       anything is ringing. This indication may be used to distinguish between
+       "transport" level error, in which case another host may be tried, and
+       that finalising the call has moved "upstream" and the local system has
+       no more to do but await a result.
+
+       If an application overrides this function, it should generally call the
+       ancestor version for correct operation.
+
+       The default behaviour calls the OnProceeding() on the connection's
+       associated OpalCall object.
+     */
+    virtual void OnProceeding(
+      OpalConnection & connection   ///<  Connection that is proceeeding
     );
 
     /**Call back for remote party being alerted on outgoing call.
@@ -393,7 +478,7 @@ class OpalManager : public PObject
        remote endpoint is "ringing". This function is generally called
        some time after the MakeConnection() function was called.
 
-       If PFalse is returned the connection is aborted.
+       If false is returned the connection is aborted.
 
        If an application overrides this function, it should generally call the
        ancestor version for correct operation. An application would typically
@@ -408,9 +493,25 @@ class OpalManager : public PObject
       OpalConnection & connection   ///<  Connection that was established
     );
 
-    virtual OpalConnection::AnswerCallResponse
-       OnAnswerCall(OpalConnection & connection,
-                     const PString & caller
+    /**Call back for answering an incoming call.
+       This function is called after the connection has been acknowledged
+       but before the connection is established
+
+       This gives the application time to wait for some event before
+       signalling to the endpoint that the connection is to proceed. For
+       example the user pressing an "Answer call" button.
+
+       If AnswerCallDenied is returned the connection is aborted and the
+       connetion specific end call PDU is sent. If AnswerCallNow is returned 
+       then the connection proceeding, Finally if AnswerCallPending is returned then the
+       protocol negotiations are paused until the AnsweringCall() function is
+       called.
+
+       The default behaviour simply returns AnswerNow.
+     */
+    virtual OpalConnection::AnswerCallResponse OnAnswerCall(
+      OpalConnection & connection,    ///<  connection that is being answered
+       const PString & caller         ///<  caller
     );
 
     /**A call back function whenever a connection is "connected".
@@ -458,7 +559,7 @@ class OpalManager : public PObject
        The default behaviour calls OnReleased() on the connection's
        associated OpalCall object. This indicates to the call that the
        connection has been released so it can release the last remaining
-       connection and then returns PTrue.
+       connection and then returns true.
       */
     virtual void OnReleased(
       OpalConnection & connection   ///<  Connection that was established
@@ -485,10 +586,46 @@ class OpalManager : public PObject
       OpalConnection & connection,  ///<  Connection that was held
       const PString & remoteParty         ///<  The new remote party
     );
+
+    /**A call back function to monitor the progress of a transfer.
+       When a transfer operation is initiated, the Transfer() function will
+       generally return immediately and the transfer may take some time. This
+       call back can give indicateion to the application of the progress of
+       the transfer.
+
+       The exact format of the \p info parameter is dependent on the protocol
+       being used. It will always have a value info["result"] which will be
+       "success" or "failed". Other variables are protocol dependent.
+
+       For SIP, there is an additional info["state"] containing the NOTIFY
+       subscription state and an info["code"] entry containing the 3 digit
+       code returned in the NOTIFY body.
+
+       A return value of false will immediately disconnect the current call.
+
+       The default behaviour returns false if info["result"] == "success".
+      */
+    virtual bool OnTransferNotify(
+      OpalConnection & connection,  ///< Connection being transferred.
+      const PStringToString & info  ///< Information on the transfer
+    );
   //@}
 
+
   /**@name Media Streams management */
   //@{
+    /**Get common media formats.
+       This is called by various places to get common media formats for the
+       basic connection classes.
+
+       The default behaviour uses the mediaFormatOrder and mediaFormatMask
+       member variables to adjust the mediaFormats list.
+      */
+    virtual OpalMediaFormatList GetCommonMediaFormats(
+      bool transportable,  ///< Include transportable media formats
+      bool pcmAudio        ///< Include raw PCM audio media formats
+    ) const;
+
     /**Adjust media formats available on a connection.
        This is called by a connection after it has called
        OpalCall::GetMediaFormats() to get all media formats that it can use so
@@ -499,6 +636,7 @@ class OpalManager : public PObject
        member variables to adjust the mediaFormats list.
       */
     virtual void AdjustMediaFormats(
+      bool local,                         ///<  Media formats a local ones to be presented to remote
       const OpalConnection & connection,  ///<  Connection that is about to use formats
       OpalMediaFormatList & mediaFormats  ///<  Media formats to use
     ) const;
@@ -531,7 +669,17 @@ class OpalManager : public PObject
       OpalMediaStream & stream    ///<  New media stream being opened
     );
 
-    /**Callback from the RTP session for statistics monitoring.
+    /**Create an RTP session.
+       This function allows an application to have the system create descendant
+       class versions of the RTP_UDP class. The application could use this to 
+	   modify the default RTP transfer behaviour.
+	   This is called when a connection determines that RTP is required for 
+	   transporting media.
+       The default behaviour returns a new RTP_UDP.
+      */
+	virtual RTP_UDP * CreateRTPSession (const RTP_Session::Params & params);
+
+	/**Callback from the RTP session for statistics monitoring.
        This is called every so many packets on the transmitter and receiver
        threads of the RTP session indicating that the statistics have been
        updated.
@@ -543,6 +691,68 @@ class OpalManager : public PObject
       const RTP_Session & session         ///<  Session with statistics
     );
 
+    /**Indicate is a local RTP connection.
+       This is called when a new media stream has been created and it has been
+       detected that media will be flowing between two RTP sessions within the
+       same process. An application could take advantage of this by optimising
+       the transfer in some way, rather than the full media path of codecs and
+       sockets which might not be necessary.
+
+       Note this is the complement to SetMediaPassThrough() as this function stops
+       RTP data from being sent/received, while SetMediaPassThrough() transfers
+       RTP data between the two endpoints.
+
+       The default behaviour returns false.
+
+       @return true if the application is going to execute some form of
+               bypass, and the media patch threads should not be started.
+      */
+    virtual bool OnLocalRTP(
+      OpalConnection & connection1, ///< First connection
+      OpalConnection & connection2, ///< Second connection
+      unsigned sessionID,           ///< Session ID of RTP session
+      bool opened                   ///< Media streams are opened/closed
+    ) const;
+
+    /**Set pass though mode for media.
+
+       Bypass the internal media handling, passing RTP data directly from
+       one call/connection to another.
+
+       This can be useful for back to back calls that happen to be the same
+       media format and you wish to avoid double decoding and encoding of
+       media. Note this scenario is not the same as two OpalConnections within
+       the same OpalCall, but two completely independent OpalCall where one
+       connection is to be bypassed. For example, two OpalCall instances might
+       have two SIPConnection instances and two OpalMixerConnection instances
+       connected via a single OpalMixerNode. Now while there are ONLY two
+       calls in the node, it is a waste to decode the audio, add to mixer and
+       re-encode it again. In practice this is identical to just bypassing the
+       mixer node completely, until a third party is added, then we need to
+       switch back to normal (non-pass-through) operation.
+
+       Note this is the complement to OnLocalRTP() as this function transfers
+       RTP data directly between the two endpoints, while OnLocalRTP() stops
+       the RTP data from being sent/received.
+
+       @return true if pass through is started/stopped, false if there was no
+               such call/connection/stream, the streams are incompatible formats
+               or a conflicting bypass is already in place.
+      */
+    bool SetMediaPassThrough(
+      const PString & token1, ///< First calls token
+      const PString & token2, ///< Second calls token
+      bool bypass,            ///< Bypass the media
+      unsigned sessionID = 0, ///< Session ID of media stream, 0 indicates all
+      bool network  = true    ///< Pass through the network connections of the calls only
+    );
+    static bool SetMediaPassThrough(
+      OpalConnection & connection1, ///< First connection
+      OpalConnection & connection2, ///< Second connection
+      bool bypass,                  ///< Bypass the media
+      unsigned sessionID = 0        ///< Session ID of media stream, 0 indicates all
+    );
+
     /**Call back for closed a media stream.
 
        The default behaviour does nothing.
@@ -552,16 +762,6 @@ class OpalManager : public PObject
     );
 
 #if OPAL_VIDEO
-
-    /**Add video media formats available on a connection.
-
-       The default behaviour calls the OpalEndPoint function of the same name.
-      */
-    virtual void AddVideoMediaFormats(
-      OpalMediaFormatList & mediaFormats, ///<  Media formats to use
-      const OpalConnection * connection = NULL  ///<  Optional connection that is using formats
-    ) const;
-
     /**Create a PVideoInputDevice for a source media stream.
       */
     virtual PBoolean CreateVideoInputDevice(
@@ -592,7 +792,7 @@ class OpalManager : public PObject
       */
     virtual OpalMediaPatch * CreateMediaPatch(
       OpalMediaStream & source,         ///<  Source media stream
-      PBoolean requiresPatchThread = PTrue
+      PBoolean requiresPatchThread = true  ///< The patch requires a thread
     );
 
     /**Destroy a OpalMediaPatch instance.
@@ -605,13 +805,20 @@ class OpalManager : public PObject
 
     /**Call back for a media patch thread starting.
        This function is called within the context of the thread associated
-       with the media patch. It may be used to do any last checks on if the
-       patch should proceed.
+       with the media patch.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour does nothing
       */
-    virtual PBoolean OnStartMediaPatch(
-      const OpalMediaPatch & patch     ///<  Media patch being started
+    virtual void OnStartMediaPatch(
+      OpalConnection & connection,  ///< Connection patch is in
+      OpalMediaPatch & patch        ///< Media patch being started
+    );
+
+    /**Call back when media stream patch thread stops.
+      */
+    virtual void OnStopMediaPatch(
+      OpalConnection & connection,  ///< Connection patch is in
+      OpalMediaPatch & patch        ///< Media Patch being stopped
     );
   //@}
 
@@ -696,7 +903,7 @@ class OpalManager : public PObject
        The "pattern" string regex is compared against routing strings that are built 
        as follows:
        
-                 a_party '\t' b_party
+                 a_party '\\t' b_party
 
        where:
 
@@ -708,8 +915,8 @@ class OpalManager : public PObject
 
        Note that all "pattern" strings have an implied '^' at the beginning and
        a '$' at the end. This forces the "pattern" to match the entire source string. 
-       For convenience, the sub-expression ".*\t" is inserted immediately after
-       any ':' character if no '\t' is present.
+       For convenience, the sub-expression ".*\\t" is inserted immediately after
+       any ':' character if no '\\t' is present.
 
        Route entries are stored and searched in the route table in the order they are added. 
        
@@ -727,7 +934,7 @@ class OpalManager : public PObject
 
        will produce the same result as the single entry "pots:26=h323:10.0.1.1".
 
-       If the "destination" parameter is of the form @filename, then the file
+       If the "destination" parameter is of the form \@filename, then the file
        is read with each line consisting of a pattern=destination route
        specification. 
        
@@ -739,17 +946,17 @@ class OpalManager : public PObject
        1) A local H.323 endpoint with with name of "myname@myhost.com" that receives a 
           call with a destination h323Id of "boris" would generate:
           
-                          "h323:myname@myhost.com\tboris"
+                          "h323:myname@myhost.com\\tboris"
 
        2) A local SIP endpoint with with name of "fred@nurk.com" that receives a 
           call with a destination of "sip:fred@nurk.com" would generate:
           
-                          "sip:fred@nurk.com\tsip:fred@nurk.com"
+                          "sip:fred@nurk.com\\tsip:fred@nurk.com"
 
        3) Using line 0 of a PhoneJACK handset with a serial # of 01AB3F4 to dial
           the digits 2, 6 and # would generate:
 
-                          "pots:Quicknet:01AB3F4:0\t26"
+                          "pots:Quicknet:01AB3F4:0\\t26"
 
 
        Destination meta-strings:
@@ -758,7 +965,7 @@ class OpalManager : public PObject
        The available meta-strings are:
        
          <da>    Replaced by the "b_party" string. For example
-                 "pc:.*\t.* = sip:<da>" directs calls to the SIP protocol. In
+                 "pc:.*\\t.* = sip:<da>" directs calls to the SIP protocol. In
                  this case there is a special condition where if the original
                  destination had a valid protocol, eg h323:fred.com, then
                  the entire string is replaced not just the <da> part.
@@ -777,7 +984,7 @@ class OpalManager : public PObject
                  then the substitution is made so a legal URL can result.
 
          <dn>    Copy all valid consecutive E.164 digits from the "b_party" so
-                 pots:0061298765@vpb:1/2 becomes sip:0061298765@carrier.com
+                 pots:0061298765\@vpb:1/2 becomes sip:0061298765@carrier.com
 
          <dnX>   As above but skip X digits, eg <dn2> skips 2 digits, so
                  pots:00612198765 becomes sip:61298765@carrier.com
@@ -786,8 +993,8 @@ class OpalManager : public PObject
 
          <dn2ip> Translate digits separated by '*' characters to an IP
                  address. e.g. 10*0*1*1 becomes 10.0.1.1, also
-                 1234*10*0*1*1 becomes 1234@10.0.1.1 and
-                 1234*10*0*1*1*1722 becomes 1234@10.0.1.1:1722.
+                 1234*10*0*1*1 becomes 1234\@10.0.1.1 and
+                 1234*10*0*1*1*1722 becomes 1234\@10.0.1.1:1722.
 
 
        Returns true if an entry was added.
@@ -800,7 +1007,7 @@ class OpalManager : public PObject
        This removes the current routeTable and calls AddRouteEntry for every
        string in the array.
 
-       Returns PTrue if at least one entry was added.
+       Returns true if at least one entry was added.
       */
     PBoolean SetRouteTable(
       const PStringArray & specs  ///<  Array of specification strings.
@@ -816,22 +1023,56 @@ class OpalManager : public PObject
 
     /**Get the active route table for the manager.
       */
-    const RouteTable & GetRouteTable() const { return routeTable; }
+    const RouteTable & GetRouteTable() const { return m_routeTable; }
 
     /**Route the source address to a destination using the route table.
        The source parameter may be something like pots:vpb:1/2 or
-       sip:fred@nurk.com.
+       sip:fred\@nurk.com.
 
        The destination parameter is a partial URL, it does not include the
-       protocol, but may be of the form user@host, or simply digits.
+       protocol, but may be of the form user\@host, or simply digits.
       */
     virtual PString ApplyRouteTable(
-      const PString & source,      /// Source address, including endpoint protocol
-      const PString & destination, /// Destination address read from source protocol
-      PINDEX & entry
+      const PString & source,      ///< Source address, including endpoint protocol
+      const PString & destination, ///< Destination address read from source protocol
+      PINDEX & entry               ///< Index into table to start search
     );
   //@}
 
+#if OPAL_HAS_MIXER
+  /**@name Call recording */
+  //@{
+    /**Start recording a call.
+       Current version saves to a WAV file. It may either mix the receive and
+       transmit audio stream to a single mono file, or the streams are placed
+       into the left and right channels of a stereo WAV file.
+
+       Returns true if the call exists and there is no recording in progress
+               for the call.
+      */
+    virtual PBoolean StartRecording(
+      const PString & callToken,  ///< Call token for call to record
+      const PFilePath & filename, ///< File into which to record
+      const OpalRecordManager::Options & options = false ///< Record mixing options
+    );
+
+    /**Indicate if recording is currently active on call.
+      */
+    virtual bool IsRecording(
+      const PString & callToken   ///< Call token for call to check if recording
+    );
+
+    /** Stop a recording.
+        Returns true if the call does exists, that recording is active is
+                not indicated.
+      */
+    virtual bool StopRecording(
+      const PString & callToken   ///< Call token for call to stop recording
+    );
+
+  //@}
+#endif
+
   /**@name Member variable access */
   //@{
     /**Get the product info for all endpoints.
@@ -869,21 +1110,27 @@ class OpalManager : public PObject
 
 #if OPAL_VIDEO
 
+    //
+    // these functions are deprecated and used only for backwards compatibility
+    // applications should use OpalConnection::GetAutoStart() to check whether
+    // a specific media type can be auto-started
+    //
+
     /**See if should auto-start receive video channels on connection.
      */
-    PBoolean CanAutoStartReceiveVideo() const { return autoStartReceiveVideo; }
+    bool CanAutoStartReceiveVideo() const { return (OpalMediaType::Video().GetAutoStart()&OpalMediaType::Receive) != 0; }
 
     /**Set if should auto-start receive video channels on connection.
      */
-    void SetAutoStartReceiveVideo(PBoolean can) { autoStartReceiveVideo = can; }
+    void SetAutoStartReceiveVideo(bool can) { OpalMediaType::Video().GetDefinition()->SetAutoStart(OpalMediaType::Receive, can); }
 
     /**See if should auto-start transmit video channels on connection.
      */
-    PBoolean CanAutoStartTransmitVideo() const { return autoStartTransmitVideo; }
+    bool CanAutoStartTransmitVideo() const { return (OpalMediaType::Video().GetAutoStart()&OpalMediaType::Transmit) != 0; }
 
     /**Set if should auto-start transmit video channels on connection.
      */
-    void SetAutoStartTransmitVideo(PBoolean can) { autoStartTransmitVideo = can; }
+    void SetAutoStartTransmitVideo(bool can) { OpalMediaType::Video().GetDefinition()->SetAutoStart(OpalMediaType::Transmit, can); }
 
 #endif
 
@@ -1028,19 +1275,31 @@ class OpalManager : public PObject
 
     /**Get the IP Type Of Service byte for media (eg RTP) channels.
      */
-    BYTE GetRtpIpTypeofService() const { return rtpIpTypeofService; }
+    BYTE GetMediaTypeOfService() const { return m_defaultMediaTypeOfService; }
 
     /**Set the IP Type Of Service byte for media (eg RTP) channels.
      */
-    void SetRtpIpTypeofService(unsigned tos) { rtpIpTypeofService = (BYTE)tos; }
+    void SetMediaTypeOfService(unsigned tos) { m_defaultMediaTypeOfService = (BYTE)tos; }
 
-    /**Get the maximum RTP payload size.
+    // For backward compatibility
+    BYTE P_DEPRECATED GetRtpIpTypeofService() const { return m_defaultMediaTypeOfService; }
+    void P_DEPRECATED SetRtpIpTypeofService(unsigned tos) { m_defaultMediaTypeOfService = (BYTE)tos; }
+
+    /**Get the IP Type Of Service byte for media (eg RTP) channels.
+     */
+    BYTE GetMediaTypeOfService(const OpalMediaType & type) const;
+
+    /**Set the IP Type Of Service byte for media (eg RTP) channels.
+     */
+    void SetMediaTypeOfService(const OpalMediaType & type, unsigned tos);
+
+    /**Get the maximum transmitted RTP payload size.
        Defaults to maximum safe MTU size (576 bytes as per RFC879) minus the
        typical size of the IP, UDP an RTP headers.
       */
     PINDEX GetMaxRtpPayloadSize() const { return rtpPayloadSizeMax; }
 
-    /**Get the maximum RTP payload size.
+    /**Get the maximum transmitted RTP payload size.
        Defaults to maximum safe MTU size (576 bytes as per RFC879) minus the
        typical size of the IP, UDP an RTP headers.
       */
@@ -1049,6 +1308,18 @@ class OpalManager : public PObject
       bool mtu = false
     ) { rtpPayloadSizeMax = size - (mtu ? (20+16+12) : 0); }
 
+    /**Get the maximum received RTP packet size.
+       Defaults to 2048.
+      */
+    PINDEX GetMaxRtpPacketSize() const { return rtpPacketSizeMax; }
+
+    /**Get the maximum transmitted RTP payload size.
+       Defaults to 2048.
+      */
+    void SetMaxRtpPacketSize(
+      PINDEX size
+    ) { rtpPacketSizeMax = size; }
+
     /**Get the default maximum audio jitter delay parameter.
        Defaults to 50ms
      */
@@ -1084,10 +1355,16 @@ class OpalManager : public PObject
     );
 
     /**Get the default media format mask.
+       The is the default list of media format names to be removed from media
+       format lists bfeore use by a connection.
+       See OpalMediaFormatList::Remove() for more information.
      */
     const PStringArray & GetMediaFormatMask() const { return mediaFormatMask; }
 
     /**Set the default media format mask.
+       The is the default list of media format names to be removed from media
+       format lists bfeore use by a connection.
+       See OpalMediaFormatList::Remove() for more information.
      */
     void SetMediaFormatMask(
       const PStringArray & mask   //< New mask
@@ -1103,6 +1380,7 @@ class OpalManager : public PObject
      */
     const OpalSilenceDetector::Params & GetSilenceDetectParams() const { return silenceDetectParams; }
     
+#if OPAL_AEC
     /**Set the default parameters for the echo cancelation.
      */
     virtual void SetEchoCancelParams(
@@ -1112,12 +1390,13 @@ class OpalManager : public PObject
     /**Get the default parameters for the silence detector.
      */
     const OpalEchoCanceler::Params & GetEchoCancelParams() const { return echoCancelParams; }
+#endif
 
 #if OPAL_VIDEO
 
     /**Set the parameters for the video device to be used for input.
        If the name is not suitable for use with the PVideoInputDevice class
-       then the function will return PFalse and not change the device.
+       then the function will return false and not change the device.
 
        This defaults to the value of the PVideoInputDevice::GetInputDeviceNames()
        function.
@@ -1133,7 +1412,7 @@ class OpalManager : public PObject
 
     /**Set the parameters for the video device to be used to preview input.
        If the name is not suitable for use with the PVideoOutputDevice class
-       then the function will return PFalse and not change the device.
+       then the function will return false and not change the device.
 
        This defaults to the value of the PVideoInputDevice::GetOutputDeviceNames()
        function.
@@ -1149,7 +1428,7 @@ class OpalManager : public PObject
 
     /**Set the parameters for the video device to be used for output.
        If the name is not suitable for use with the PVideoOutputDevice class
-       then the function will return PFalse and not change the device.
+       then the function will return false and not change the device.
 
        This defaults to the value of the PVideoInputDevice::GetOutputDeviceNames()
        function.
@@ -1198,30 +1477,15 @@ class OpalManager : public PObject
     // needs to be public for gcc 3.4
     void GarbageCollection();
 
-    /**Call back for a new connection has been constructed.
-       This is called after CreateConnection has returned a new connection.
-       It allows an application to make any custom adjustments to the
-       connection before it begins to process the protocol. behind it.
-      */
-    virtual void OnNewConnection(
-      OpalConnection & connection   ///< New connection just created
+#ifdef OPAL_ZRTP
+    virtual bool GetZRTPEnabled() const;
+#endif
+
+    virtual void OnApplyStringOptions(
+      OpalConnection & conn,
+      OpalConnection::StringOptions & stringOptions
     );
 
-    virtual void SetDefaultSecurityMode(const PString & v)
-    { defaultSecurityMode = v; }
-
-    virtual PString GetDefaultSecurityMode() const 
-    { return defaultSecurityMode; }
-
-    virtual PBoolean UseRTPAggregation() const;
-
-    OpalRecordManager & GetRecordManager()
-    { return recordManager; }
-
-    virtual PBoolean StartRecording(const PString & callToken, const PFilePath & fn);
-    virtual bool IsRecording(const PString & callToken);
-    virtual void StopRecording(const PString & callToken);
-
   protected:
     // Configuration variables
     OpalProductInfo productInfo;
@@ -1229,13 +1493,11 @@ class OpalManager : public PObject
     PString       defaultUserName;
     PString       defaultDisplayName;
 
-#if OPAL_VIDEO
-    PBoolean          autoStartReceiveVideo;
-    PBoolean          autoStartTransmitVideo;
-#endif
+    BYTE                     m_defaultMediaTypeOfService;
+    map<OpalMediaType, BYTE> m_mediaTypeOfService;
 
-    BYTE          rtpIpTypeofService;
     PINDEX        rtpPayloadSizeMax;
+    PINDEX        rtpPacketSizeMax;
     unsigned      minAudioJitterDelay;
     unsigned      maxAudioJitterDelay;
     PStringArray  mediaFormatOrder;
@@ -1245,7 +1507,9 @@ class OpalManager : public PObject
     PString       ilsServer;
 
     OpalSilenceDetector::Params silenceDetectParams;
+#if OPAL_AEC
     OpalEchoCanceler::Params echoCancelParams;
+#endif
 
 #if OPAL_VIDEO
     PVideoDevice::OpenArgs videoInputDevice;
@@ -1293,8 +1557,8 @@ class OpalManager : public PObject
     PSTUNClient      * stun;
     InterfaceMonitor * interfaceMonitor;
 
-    RouteTable routeTable;
-    PMutex     routeTableMutex;
+    RouteTable m_routeTable;
+    PMutex     m_routeMutex;
 
     // Dynamic variables
     PReadWriteMutex     endpointsMutex;
@@ -1311,22 +1575,40 @@ class OpalManager : public PObject
         OpalManager & manager;
     } activeCalls;
 
-    PBoolean	 clearingAllCalls;
-    PSyncPoint   allCallsCleared;
+    PSafeDictionary<PString, OpalPresentity> m_presentities;
+
+    PAtomicInteger m_clearingAllCallsCount;
+    PMutex         m_clearingAllCallsMutex;
+    PSyncPoint     m_allCallsCleared;
+    void InternalClearAllCalls(OpalConnection::CallEndReason reason, bool wait, bool first);
+
     PThread    * garbageCollector;
     PSyncPoint   garbageCollectExit;
     PDECLARE_NOTIFIER(PThread, OpalManager, GarbageMain);
 
-    PString defaultSecurityMode;
-
-#if OPAL_RTP_AGGREGATE
-    PBoolean useRTPAggregation; 
+#ifdef OPAL_ZRTP
+    bool zrtpEnabled;
 #endif
 
-    OpalRecordManager recordManager;
-
     friend OpalCall::OpalCall(OpalManager & mgr);
-    friend void OpalCall::OnReleased(OpalConnection & connection);
+    friend void OpalCall::InternalOnClear();
+
+  private:
+    P_REMOVE_VIRTUAL(OpalCall *,CreateCall(), 0);
+    P_REMOVE_VIRTUAL(PBoolean, OnIncomingConnection(OpalConnection &, unsigned), false);
+    P_REMOVE_VIRTUAL(PBoolean, OnIncomingConnection(OpalConnection &), false);
+    P_REMOVE_VIRTUAL(PBoolean, OnStartMediaPatch(const OpalMediaPatch &), false);
+    P_REMOVE_VIRTUAL_VOID(AdjustMediaFormats(const OpalConnection &, OpalMediaFormatList &) const);
+    P_REMOVE_VIRTUAL_VOID(OnMessageReceived(const PURL&,const PString&,const PURL&,const PString&,const PString&,const PString&));
+
+
+#ifdef OPAL_HAS_IM
+  public:
+    OpalIMManager & GetIMManager() { return *m_imManager; }
+
+  protected:
+    OpalIMManager * m_imManager;
+#endif
 };
 
 
