@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24504 $
+ * $Revision: 25002 $
  * $Author: rjongbloed $
- * $Date: 2010-06-17 17:49:01 -0500 (Thu, 17 Jun 2010) $
+ * $Date: 2011-01-04 22:39:39 -0600 (Tue, 04 Jan 2011) $
  */
 
 #ifndef OPAL_OPAL_MEDIAFMT_H
@@ -140,7 +140,7 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
        The '*' character indicates substrings, for example: "G.711*" would
        match "G.711-uLaw-64k" and "G.711-ALaw-64k".
 
-       The '@' character indicates a type of media format, so say "@video
+       The '@' character indicates a type of media format, so say "\@video"
        would find the first video codec.
 
        The '!' character indicates a negative test. That is the first entry
@@ -151,7 +151,7 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
       */
     const_iterator FindFormat(
       const PString & wildcard,    ///<  Wildcard string name.
-      const_iterator start = const_iterator()
+      const_iterator start = const_iterator() ///< Starting position iterator for search
     ) const;
 
     /**Determine if a format matching the payload type is in the list.
@@ -176,7 +176,7 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
        The '*' character indicates substrings, for example: "G.711*" would
        match "G.711-uLaw-64k" and "G.711-ALaw-64k".
 
-       The '@' character indicates a type of media format, so say "@video
+       The '@' character indicates a type of media format, so say "\@video"
        would find the first video codec.
 
        The '!' character indicates a negative test. That is the first entry
@@ -201,7 +201,7 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
        The '*' character indicates substrings, for example: "G.711*" would
        match "G.711-uLaw-64k" and "G.711-ALaw-64k".
 
-       The '@' character indicates a type of media format, so say "@video
+       The '@' character indicates a type of media format, so say "\@video"
        would sort by video codec.
 
         The '!' character is not supported.
@@ -222,7 +222,7 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
     virtual PINDEX Append(PObject *) { return P_MAX_INDEX; }
     virtual PINDEX Insert(const PObject &, PObject *) { return P_MAX_INDEX; }
     virtual PINDEX InsertAt(PINDEX, PObject *) { return P_MAX_INDEX; }
-    virtual PBoolean SetAt(PINDEX, PObject *) { return PFalse; }
+    virtual PBoolean SetAt(PINDEX, PObject *) { return false; }
 };
 
 
@@ -374,10 +374,10 @@ class OpalMediaOptionValue : public OpalMediaOption
         strm.setstate(ios::badbit);
     }
 
-    static __inline bool Intersect(bool a, bool b)         { return a && b; }
-    static __inline bool Intersect(int a, int b)           { return a & b; }
-    static __inline bool Intersect(unsigned a, unsigned b) { return a & b; }
-    static __inline bool Intersect(double a, double b)     { return std::min(a, b); }
+    static __inline bool     Intersect(bool     a, bool     b) { return a && b; }
+    static __inline int      Intersect(int      a, int      b) { return a & b; }
+    static __inline unsigned Intersect(unsigned a, unsigned b) { return a & b; }
+    static __inline double   Intersect(double   a, double   b) { return std::min(a, b); }
 
     virtual bool Merge(const OpalMediaOption & option)
     {
@@ -605,7 +605,7 @@ class OpalMediaFormatInternal : public PObject
     virtual bool GetOptionOctets(const PString & name, PBYTEArray & octets) const;
     virtual bool SetOptionOctets(const PString & name, const PBYTEArray & octets);
     virtual bool SetOptionOctets(const PString & name, const BYTE * data, PINDEX length);
-    virtual bool AddOption(OpalMediaOption * option, PBoolean overwrite = PFalse);
+    virtual bool AddOption(OpalMediaOption * option, PBoolean overwrite = false);
     virtual OpalMediaOption * FindOption(const PString & name) const;
 
     virtual bool ToNormalisedOptions();
@@ -625,6 +625,7 @@ class OpalMediaFormatInternal : public PObject
     PSortedList<OpalMediaOption> options;
     time_t                       codecVersionTime;
     bool                         forceIsTransportable;
+    int                          m_channels;
 
   friend bool operator==(const char * other, const OpalMediaFormat & fmt);
   friend bool operator!=(const char * other, const OpalMediaFormat & fmt);
@@ -802,7 +803,7 @@ class OpalMediaFormat : public PContainer
        object has a tx number of frames of 3, but the parameter has a value
        of 1, then the current object will be set to 1.
 
-       Returns PFalse if the media formats are incompatible and cannot be
+       Returns false if the media formats are incompatible and cannot be
        merged.
       */
     bool Merge(
@@ -818,13 +819,13 @@ class OpalMediaFormat : public PContainer
       */
     PString GetName() const { PWaitAndSignal m(m_mutex); return m_info == NULL ? "" : m_info->formatName; }
 
-    /**Return PTrue if media format info is valid. This may be used if the
+    /**Return true if media format info is valid. This may be used if the
        single string constructor is used to check that it matched something
        in the registered media formats database.
       */
     PBoolean IsValid() const { PWaitAndSignal m(m_mutex); return m_info != NULL && m_info->IsValid(); }
 
-    /**Return PTrue if media format info may be sent via RTP. Some formats are internal
+    /**Return true if media format info may be sent via RTP. Some formats are internal
        use only and are never transported "over the wire".
       */
     PBoolean IsTransportable() const { PWaitAndSignal m(m_mutex); return m_info != NULL && m_info->IsTransportable(); }
@@ -848,7 +849,7 @@ class OpalMediaFormat : public PContainer
     /**Determine if the media format requires a jitter buffer. As a rule an
        audio codec needs a jitter buffer and all others do not.
       */
-    bool NeedsJitterBuffer() const { PWaitAndSignal m(m_mutex); return m_info != NULL && m_info->GetOptionBoolean(NeedsJitterOption(), PFalse); }
+    bool NeedsJitterBuffer() const { PWaitAndSignal m(m_mutex); return m_info != NULL && m_info->GetOptionBoolean(NeedsJitterOption(), false); }
     static const PString & NeedsJitterOption();
 
     /**Get the average bandwidth used in bits/second.
@@ -933,7 +934,7 @@ class OpalMediaFormat : public PContainer
       */
     bool GetOptionBoolean(
       const PString & name,   ///<  Option name
-      bool dflt = PFalse       ///<  Default value if option not present
+      bool dflt = false       ///<  Default value if option not present
     ) const { PWaitAndSignal m(m_mutex); return m_info != NULL && m_info->GetOptionBoolean(name, dflt); }
 
     /**Set the option value of the specified name as a boolean.
@@ -1026,7 +1027,7 @@ class OpalMediaFormat : public PContainer
     ) { PWaitAndSignal m(m_mutex); MakeUnique(); return m_info != NULL && m_info->SetOptionString(name, value); }
 
     /**Get the option value of the specified name as an octet array.
-       Returns PFalse if not present.
+       Returns false if not present.
       */
     bool GetOptionOctets(
       const PString & name, ///<  Option name
@@ -1049,16 +1050,6 @@ class OpalMediaFormat : public PContainer
       PINDEX length               ///<  Number of octets
     ) { PWaitAndSignal m(m_mutex); MakeUnique(); return m_info != NULL && m_info->SetOptionOctets(name, data, length); }
 
-    /**Create a media format list containing all of the registered media 
-	   formats matching the encoding name and clock rate, or the payload type.
-      */
-    static OpalMediaFormatList GetMatchingRegisteredMediaFormats(
-      RTP_DataFrame::PayloadTypes rtpPayloadType, ///<  RTP payload type code
-      const unsigned clockRate,                   ///<  clock rate
-      const char * rtpEncodingName = NULL,        ///<  RTP payload type name
-      const char * protocol = NULL                ///<  protocol to be valid for (if NULL, then all)
-    );
-
     /**Get a copy of the list of media formats that have been registered.
       */
     static OpalMediaFormatList GetAllRegisteredMediaFormats();
@@ -1078,7 +1069,7 @@ class OpalMediaFormat : public PContainer
       */
     bool AddOption(
       OpalMediaOption * option,
-      PBoolean overwrite = PFalse
+      PBoolean overwrite = false
     ) { PWaitAndSignal m(m_mutex); MakeUnique(); return m_info != NULL && m_info->AddOption(option, overwrite); }
 
     /**
@@ -1094,7 +1085,7 @@ class OpalMediaFormat : public PContainer
       const PString & name
     ) const { PWaitAndSignal m(m_mutex); return m_info == NULL ? NULL : m_info->FindOption(name); }
 
-    /** Returns PTrue if the media format is valid for the protocol specified
+    /** Returns true if the media format is valid for the protocol specified
         This allow plugin codecs to customise which protocols they are valid for
         The default implementation returns true unless the protocol is H.323
         and the rtpEncodingName is NULL
@@ -1140,7 +1131,7 @@ class OpalMediaFormat : public PContainer
 #endif
 
   private:
-    PBoolean SetSize(PINDEX) { return PTrue; }
+    PBoolean SetSize(PINDEX) { return true; }
 
   protected:
     void Construct(OpalMediaFormatInternal * info);
@@ -1275,13 +1266,21 @@ class OpalVideoFormat : public OpalMediaFormat
 // List of known media formats
 
 #define OPAL_PCM16          "PCM-16"
+#define OPAL_PCM16S         "PCM-16S"
 #define OPAL_PCM16_16KHZ    "PCM-16-16kHz"
+#define OPAL_PCM16S_16KHZ   "PCM-16S-16kHz"
 #define OPAL_PCM16_32KHZ    "PCM-16-32kHz"
+#define OPAL_PCM16S_32KHZ   "PCM-16S-32kHz"
 #define OPAL_PCM16_48KHZ    "PCM-16-48kHz"
+#define OPAL_PCM16S_48KHZ   "PCM-16S-48kHz"
 #define OPAL_L16_MONO_8KHZ  "Linear-16-Mono-8kHz"
+#define OPAL_L16_STEREO_8KHZ "Linear-16-Stereo-8kHz"
 #define OPAL_L16_MONO_16KHZ "Linear-16-Mono-16kHz"
+#define OPAL_L16_STEREO_16KHZ "Linear-16-Stereo-16kHz"
 #define OPAL_L16_MONO_32KHZ "Linear-16-Mono-32kHz"
+#define OPAL_L16_STEREO_32KHZ "Linear-16-Stereo-32kHz"
 #define OPAL_L16_MONO_48KHZ "Linear-16-Mono-48kHz"
+#define OPAL_L16_STEREO_48KHZ "Linear-16-Stereo-48kHz"
 #define OPAL_G711_ULAW_64K  "G.711-uLaw-64k"
 #define OPAL_G711_ALAW_64K  "G.711-ALaw-64k"
 #define OPAL_G722           "G.722"
@@ -1314,12 +1313,19 @@ class OpalVideoFormat : public OpalMediaFormat
 #define OPAL_T38            "T.38"
 
 extern const OpalAudioFormat & GetOpalPCM16();
+extern const OpalAudioFormat & GetOpalPCM16S();
 extern const OpalAudioFormat & GetOpalPCM16_16KHZ();
+extern const OpalAudioFormat & GetOpalPCM16S_16KHZ();
 extern const OpalAudioFormat & GetOpalPCM16_32KHZ();
+extern const OpalAudioFormat & GetOpalPCM16S_32KHZ();
 extern const OpalAudioFormat & GetOpalPCM16_48KHZ();
+extern const OpalAudioFormat & GetOpalPCM16S_48KHZ();
 extern const OpalAudioFormat & GetOpalL16_MONO_8KHZ();
+extern const OpalAudioFormat & GetOpalL16_STEREO_8KHZ();
 extern const OpalAudioFormat & GetOpalL16_MONO_16KHZ();
+extern const OpalAudioFormat & GetOpalL16_STEREO_16KHZ();
 extern const OpalAudioFormat & GetOpalL16_MONO_32KHZ();
+extern const OpalAudioFormat & GetOpalL16_STEREO_32KHZ();
 extern const OpalAudioFormat & GetOpalL16_MONO_48KHZ();
 extern const OpalAudioFormat & GetOpalL16_STEREO_48KHZ();
 extern const OpalAudioFormat & GetOpalG711_ULAW_64K();
@@ -1353,12 +1359,19 @@ extern const OpalMediaFormat & GetOpalT38();
 
 
 #define OpalPCM16          GetOpalPCM16()
+#define OpalPCM16S         GetOpalPCM16S()
 #define OpalPCM16_16KHZ    GetOpalPCM16_16KHZ()
+#define OpalPCM16S_16KHZ   GetOpalPCM16S_16KHZ()
 #define OpalPCM16_32KHZ    GetOpalPCM16_32KHZ()
+#define OpalPCM16S_32KHZ   GetOpalPCM16S_32KHZ()
 #define OpalPCM16_48KHZ    GetOpalPCM16_48KHZ()
+#define OpalPCM16S_48KHZ   GetOpalPCM16S_48KHZ()
 #define OpalL16_MONO_8KHZ  GetOpalL16_MONO_8KHZ()
+#define OpalL16_STEREO_8KHZ GetOpalL16_STEREO_8KHZ()
 #define OpalL16_MONO_16KHZ GetOpalL16_MONO_16KHZ()
+#define OpalL16_STEREO_16KHZ GetOpalL16_STEREO_16KHZ()
 #define OpalL16_MONO_32KHZ GetOpalL16_MONO_32KHZ()
+#define OpalL16_STEREO_32KHZ GetOpalL16_STEREO_32KHZ()
 #define OpalL16_MONO_48KHZ GetOpalL16_MONO_48KHZ()
 #define OpalL16_STEREO_48KHZ GetOpalL16_STEREO_48KHZ()
 #define OpalG711_ULAW_64K  GetOpalG711_ULAW_64K()
