@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 22198 $
+ * $Revision: 25290 $
  * $Author: rjongbloed $
- * $Date: 2009-03-11 20:21:09 -0500 (Wed, 11 Mar 2009) $
+ * $Date: 2011-03-07 13:22:29 -0600 (Mon, 07 Mar 2011) $
  */
 
 #ifndef OPAL_OPAL_TRANSPORT_H
@@ -73,9 +73,9 @@ class OpalTransportAddress : public PCaselessString
       const char * proto = NULL  ///<  Default is "tcp"
     );
     OpalTransportAddress(
-      const PIPSocket::Address & ip,
-      WORD port,
-      const char * proto = NULL ///<  Default is "tcp"
+      const PIPSocket::Address & ip,  ///<  IP address for transport
+      WORD port,                      ///<  Default port number
+      const char * proto = NULL       ///<  Default is "tcp"
     );
   //@}
 
@@ -87,7 +87,7 @@ class OpalTransportAddress : public PCaselessString
        is the value 65535.
       */
     PBoolean IsEquivalent(
-      const OpalTransportAddress & address,
+      const OpalTransportAddress & address,   ///< Address to compare
       bool wildcards = false   ///< Allow wildcards
     ) const;
 
@@ -99,15 +99,15 @@ class OpalTransportAddress : public PCaselessString
 
     /**Return the underlying protocol for the transport address.
       */
-    PCaselessString GetProto() const { return Left(Find('$')); }
+    PCaselessString GetProto(bool withDollar = false) const { return Left(Find('$')+(withDollar?1:0)); }
 
     /**Extract the ip address from transport address.
-       Returns PFalse, if the address is not an IP transport address.
+       Returns false, if the address is not an IP transport address.
       */
     PBoolean GetIpAddress(PIPSocket::Address & ip) const;
 
     /**Extract the ip address and port number from transport address.
-       Returns PFalse, if the address is not an IP transport address.
+       Returns false, if the address is not an IP transport address.
       */
     PBoolean GetIpAndPort(PIPSocket::Address & ip, WORD & port) const;
     PBoolean GetIpAndPort(PIPSocketAddressAndPort & ipPort) const;
@@ -292,9 +292,9 @@ class OpalListener : public PObject
         OpalTransport instance created by the listener. If this is NULL then
         it indicates an error occurred during the accept.
 
-        If singleThread is PFalse the acceptHandler function is called in the
+        If singleThread is false the acceptHandler function is called in the
         context of a new thread and the continues to listen for more
-        connections. If PTrue, then the acceptHandler function is called from
+        connections. If true, then the acceptHandler function is called from
         within the listener threads context and no more connections are
         created. That is only a single connection is ever created by this
         listener.
@@ -382,7 +382,7 @@ class OpalListenerIP : public OpalListener
       OpalEndPoint & endpoint,                 ///<  Endpoint listener is used for
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to listen on
       WORD port = 0,                           ///<  TCP port to listen for connections
-      PBoolean exclusive = PTrue
+      PBoolean exclusive = true               ///< Exclusive listening mode, no other process can accept on the port
     );
     OpalListenerIP(
       OpalEndPoint & endpoint,                  ///<  Endpoint listener is used for
@@ -430,7 +430,7 @@ class OpalListenerTCP : public OpalListenerIP
       OpalEndPoint & endpoint,                 ///<  Endpoint listener is used for
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to listen on
       WORD port = 0,                           ///<  TCP port to listen for connections
-      PBoolean exclusive = PTrue
+      PBoolean exclusive = true               ///< Exclusive listening mode, no other process can accept on the port
     );
     OpalListenerTCP(
       OpalEndPoint & endpoint,                  ///<  Endpoint listener is used for
@@ -454,9 +454,9 @@ class OpalListenerTCP : public OpalListenerIP
         new connection is created. The INT parameter to the acceptHandler is
         a pointer to the new OpalTransport instance created by the listener.
 
-        If singleThread is PFalse the acceptHandler function is called in the
+        If singleThread is false the acceptHandler function is called in the
         context of a new thread and the continues to listen for more
-        connections. If PTrue, then the acceptHandler function is called from
+        connections. If true, then the acceptHandler function is called from
         within the listener threads context and no more connections are
         created. That is only a single connection is ever created by this
         listener.
@@ -512,7 +512,7 @@ class OpalListenerUDP : public OpalListenerIP
       OpalEndPoint & endpoint,  ///<  Endpoint listener is used for
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to listen on
       WORD port = 0,            ///<  TCP port to listen for connections
-      PBoolean exclusive = PTrue
+      PBoolean exclusive = true  ///< Exclusive listening mode, no other process can accept on the port
     );
     OpalListenerUDP(
       OpalEndPoint & endpoint,                  ///<  Endpoint listener is used for
@@ -536,9 +536,9 @@ class OpalListenerUDP : public OpalListenerIP
         new connection is created. The INT parameter to the acceptHandler is
         a pointer to the new OpalTransport instance created by the listener.
 
-        If singleThread is PFalse the acceptHandler function is called in the
+        If singleThread is false the acceptHandler function is called in the
         context of a new thread and the continues to listen for more
-        connections. If PTrue, then the acceptHandler function is called from
+        connections. If true, then the acceptHandler function is called from
         within the listener threads context and no more connections are
         created. That is only a single connection is ever created by this
         listener.
@@ -583,11 +583,21 @@ class OpalListenerUDP : public OpalListenerIP
     ) const;
   //@}
 
+  /**@name Member access */
+  //@{
+    /**Set the size of UDP packet reads.
+      */
+    void SetBufferSize(
+      PINDEX size
+    ) { m_bufferSize = size; }
+  //@}
+
 
   protected:
     virtual const char * GetProtoPrefix() const;
 
     PMonitoredSocketsPtr listenerBundle;
+    PINDEX               m_bufferSize;
 };
 
 
@@ -739,6 +749,10 @@ class OpalTransport : public PIndirectChannel
        This will read using the transports mechanism for PDU boundaries, for
        example UDP is a single Read() call, while for TCP there is a TPKT
        header that indicates the size of the PDU.
+
+       If false is returned but there is data returned in the \p packet
+       that indicates that the available buffer space was too small, e.g. an
+       EMSGSIZE error was returned by recvfrom.
       */
     virtual PBoolean ReadPDU(
       PBYTEArray & packet   ///<  Packet read from transport
@@ -868,7 +882,7 @@ class OpalTransportTCP : public OpalTransportIP
       OpalEndPoint & endpoint,    ///<  Endpoint object
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to use
       WORD port = 0,              ///<  Local port to bind to
-      PBoolean reuseAddr = PFalse      ///<  Flag for binding to already bound interface
+      PBoolean reuseAddr = false      ///<  Flag for binding to already bound interface
     );
     OpalTransportTCP(
       OpalEndPoint & endpoint,    ///<  Endpoint object
@@ -899,6 +913,10 @@ class OpalTransportTCP : public OpalTransportIP
        This will read using the transports mechanism for PDU boundaries, for
        example UDP is a single Read() call, while for TCP there is a TPKT
        header that indicates the size of the PDU.
+
+       If false is returned but there is data returned in the \p packet
+       that indicates that the available buffer space was too small, e.g. an
+       EMSGSIZE error was returned by recvfrom.
       */
     virtual PBoolean ReadPDU(
       PBYTEArray & pdu  ///<  PDU read from transport
@@ -924,10 +942,10 @@ class OpalTransportTCP : public OpalTransportIP
        open channels. It may be used by descendent channels to do any
        handshaking required by the protocol that channel embodies.
 
-       The default behaviour is to simply return PTrue.
+       The default behaviour is to simply return true.
 
        @return
-       Returns PTrue if the protocol handshaking is successful.
+       Returns true if the protocol handshaking is successful.
      */
     virtual PBoolean OnOpen();
 
@@ -956,11 +974,8 @@ class OpalTransportUDP : public OpalTransportIP
      */
     OpalTransportUDP(
       OpalEndPoint & endpoint,              ///<  Endpoint object
-      const PBYTEArray & preReadPacket,     ///<  Packet already read by OpalListenerUDP
       const PMonitoredSocketsPtr & sockets, ///<  Bundle of sockets from OpalListenerUDP
-      const PString & iface,                ///<  Local interface to use
-      PIPSocket::Address remoteAddress,     ///<  Remote address received PDU on
-      WORD remotePort                       ///<  Remote port received PDU on
+      const PString & iface                 ///<  Local interface to use
     );
 
     /// Destroy the UDP channel
@@ -1064,6 +1079,10 @@ class OpalTransportUDP : public OpalTransportIP
        This will read using the transports mechanism for PDU boundaries, for
        example UDP is a single Read() call, while for TCP there is a TPKT
        header that indicates the size of the PDU.
+
+       If false is returned but there is data returned in the \p packet
+       that indicates that the available buffer space was too small, e.g. an
+       EMSGSIZE error was returned by recvfrom.
       */
     virtual PBoolean ReadPDU(
       PBYTEArray & packet   ///<  Packet read from transport
@@ -1092,6 +1111,12 @@ class OpalTransportUDP : public OpalTransportIP
       WriteConnectCallback function,  ///<  Function for writing data
       void * userData                 ///<  User data to pass to write function
     );
+
+    /**Set the size of UDP packet reads.
+      */
+    void SetBufferSize(
+      PINDEX size
+    ) { m_bufferSize = size; }
   //@}
 
   protected:
@@ -1100,7 +1125,11 @@ class OpalTransportUDP : public OpalTransportIP
     virtual const char * GetProtoPrefix() const;
 
     OpalManager & manager;
-    PBYTEArray    preReadPacket;
+    PINDEX        m_bufferSize;
+    PBYTEArray    m_preReadPacket;
+    bool          m_preReadOK;
+
+  friend class OpalListenerUDP;
 };
 
 
@@ -1204,7 +1233,7 @@ class OpalListenerTCPS : public OpalListenerTCP
       OpalEndPoint & endpoint,                 ///<  Endpoint listener is used for
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to listen on
       WORD port = 0,                           ///<  TCP port to listen for connections
-      PBoolean exclusive = PTrue
+      PBoolean exclusive = true               ///< Exclusive listening mode, no other process can accept on the port
     );
     OpalListenerTCPS(
       OpalEndPoint & endpoint,                  ///<  Endpoint listener is used for
@@ -1233,7 +1262,7 @@ class OpalTransportTCPS : public OpalTransportTCP
         OpalEndPoint & endpoint,    ///<  Endpoint object
         PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to use
         WORD port = 0,              ///<  Local port to bind to
-        PBoolean reuseAddr = PFalse      ///<  Flag for binding to already bound interface
+        PBoolean reuseAddr = false      ///<  Flag for binding to already bound interface
       );
       OpalTransportTCPS(
         OpalEndPoint & endpoint,    ///<  Endpoint object
