@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24127 $
+ * $Revision: 25550 $
  * $Author: rjongbloed $
- * $Date: 2010-03-14 18:12:08 -0500 (Sun, 14 Mar 2010) $
+ * $Date: 2011-04-12 01:56:22 -0500 (Tue, 12 Apr 2011) $
  */
 
 #ifndef OPAL_RTP_RTP_H
@@ -43,6 +43,9 @@
 #include <ptlib/sockets.h>
 #include <ptlib/safecoll.h>
 
+#include <list>
+
+#include <rtp/metrics.h>
 
 class RTP_JitterBuffer;
 class PNatMethod;
@@ -71,7 +74,7 @@ class RTP_DataFrame : public PBYTEArray
 
   public:
     RTP_DataFrame(PINDEX payloadSize = 0, PINDEX bufferSize = 0);
-    RTP_DataFrame(const BYTE * data, PINDEX len, PBoolean dynamic = PTrue);
+    RTP_DataFrame(const BYTE * data, PINDEX len, PBoolean dynamic = true);
 
     enum {
       ProtocolVersion = 2,
@@ -129,8 +132,10 @@ class RTP_DataFrame : public PBYTEArray
 
     bool GetPadding() const { return (theArray[0]&0x20) != 0; }
     void SetPadding(bool v)  { if (v) theArray[0] |= 0x20; else theArray[0] &= 0xdf; }
+    BYTE * GetPaddingPtr() const { return (BYTE *)(theArray+m_headerSize+m_payloadSize); }
 
-    unsigned GetPaddingSize() const;
+    unsigned GetPaddingSize() const { return m_paddingSize; }
+    bool     SetPaddingSize(PINDEX sz);
 
     PayloadTypes GetPayloadType() const { return (PayloadTypes)(theArray[1]&0x7f); }
     void         SetPayloadType(PayloadTypes t);
@@ -148,7 +153,7 @@ class RTP_DataFrame : public PBYTEArray
     DWORD  GetContribSource(PINDEX idx) const;
     void   SetContribSource(PINDEX idx, DWORD src);
 
-    PINDEX GetHeaderSize() const;
+    PINDEX GetHeaderSize() const { return m_headerSize; }
 
     int GetExtensionType() const; // -1 is no extension
     void   SetExtensionType(int type);
@@ -156,14 +161,21 @@ class RTP_DataFrame : public PBYTEArray
     bool   SetExtensionSizeDWORDs(PINDEX sz);   // set the number of 32 bit words in the extension (excluding the header)
     BYTE * GetExtensionPtr() const;
 
-    PINDEX GetPayloadSize() const { return payloadSize - GetPaddingSize(); }
-    PBoolean   SetPayloadSize(PINDEX sz);
-    BYTE * GetPayloadPtr()     const { return (BYTE *)(theArray+GetHeaderSize()); }
+    PINDEX GetPayloadSize() const { return m_payloadSize; }
+    bool   SetPayloadSize(PINDEX sz);
+    BYTE * GetPayloadPtr()     const { return (BYTE *)(theArray+m_headerSize); }
 
+    virtual PObject * Clone() const { return new RTP_DataFrame(*this); }
     virtual void PrintOn(ostream & strm) const;
 
+    // Note this sets the whole packet length, and calculates the various
+    // sub-section sizes: header payload and padding.
+    bool SetPacketSize(PINDEX sz);
+
   protected:
-    PINDEX payloadSize;
+    PINDEX m_headerSize;
+    PINDEX m_payloadSize;
+    PINDEX m_paddingSize;
 
 #if PTRACING
     friend ostream & operator<<(ostream & o, PayloadTypes t);
@@ -188,12 +200,15 @@ class RTP_ControlFrame : public PBYTEArray
     void     SetCount(unsigned count);
 
     enum PayloadTypes {
-      e_IntraFrameRequest = 192,
-      e_SenderReport = 200,
-      e_ReceiverReport,
-      e_SourceDescription,
-      e_Goodbye,
-      e_ApplDefined
+      e_IntraFrameRequest       = 192,
+      e_SenderReport            = 200,
+      e_ReceiverReport          = 201,
+      e_SourceDescription       = 202,
+      e_Goodbye                 = 203,
+      e_ApplDefined             = 204,
+      e_TransportLayerFeedBack  = 205, // RFC4585
+      e_PayloadSpecificFeedBack = 206,
+      e_ExtendedReport          = 207  // RFC3611
     };
 
     unsigned GetPayloadType() const { return (BYTE)theArray[compoundOffset+1]; }
@@ -234,6 +249,35 @@ class RTP_ControlFrame : public PBYTEArray
       PUInt32b osent;     /* octets sent */ 
     };
 
+    struct ExtendedReport {
+      /* VoIP Metrics Report Block */
+      BYTE bt;                     /* block type */
+      BYTE type_specific;          /* determined by the block definition */
+      PUInt16b length;             /* length of the report block */
+      PUInt32b ssrc;               /* data source being reported */
+      BYTE loss_rate;              /* fraction of RTP data packets lost */ 
+      BYTE discard_rate;           /* fraction of RTP data packets discarded */
+      BYTE burst_density;          /* fraction of RTP data packets within burst periods */
+      BYTE gap_density;            /* fraction of RTP data packets within inter-burst gaps */
+      PUInt16b burst_duration;     /* the mean duration, in ms, of burst periods */
+      PUInt16b gap_duration;       /* the mean duration, in ms, of gap periods */
+      PUInt16b round_trip_delay;   /* the most recently calculated round trip time */    
+      PUInt16b end_system_delay;   /* the most recently estimates end system delay */
+      BYTE signal_level;           /* voice signal level related to 0 dBm */
+      BYTE noise_level;            /* ratio of the silent background level to 0 dBm */
+      BYTE rerl;                   /* residual echo return loss */
+      BYTE gmin;                   /* gap threshold */
+      BYTE r_factor;               /* voice quality metric of the call */
+      BYTE ext_r_factor;           /* external R factor */
+      BYTE mos_lq;                 /* MOS for listen quality */
+      BYTE mos_cq;                 /* MOS for conversational quality */
+      BYTE rx_config;              /* receiver configuration byte */
+      BYTE reserved;               /* reserved for future definition */
+      PUInt16b jb_nominal;         /* current nominal jitter buffer delay, in ms */ 
+      PUInt16b jb_maximum;         /* current maximum jitter buffer delay, in ms */
+      PUInt16b jb_absolute;        /* current absolute maximum jitter buffer delay, in ms */
+    };
+
     enum DescriptionTypes {
       e_END,
       e_CNAME,
@@ -272,6 +316,41 @@ class RTP_ControlFrame : public PBYTEArray
       unsigned type,            ///<  Description type
       const PString & data      ///<  Data for description
     );
+
+    // RFC4585 Feedback Message Type (FMT)
+    unsigned GetFbType() const { return (BYTE)theArray[compoundOffset]&0x1f; }
+    void     SetFbType(unsigned type, PINDEX fciSize);
+
+    enum PayloadSpecificFbTypes {
+      e_PictureLossIndication = 1,
+      e_SliceLostIndication,
+      e_ReferencePictureSelectionIndication,
+      e_FullIntraRequest,                     //RFC5104
+      e_TemporalSpatialTradeOffRequest,
+      e_TemporalSpatialTradeOffNotification,
+      e_VideoBackChannelMessage,
+      e_ApplicationLayerFbMessage = 15
+    };
+
+    struct FbFCI {
+      PUInt32b senderSSRC;  /* data source of sender of message */
+      PUInt32b mediaSSRC;   /* data source of media */
+    };
+
+    struct FbFIR {
+      FbFCI    fci;
+      PUInt32b requestSSRC;
+      BYTE     sequenceNUmber;
+    };
+
+    struct FbTSTO {
+      FbFCI    fci;
+      PUInt32b requestSSRC;
+      BYTE     sequenceNUmber;
+      BYTE     reserver[2];
+      BYTE     tradeOff;
+    };
+
 #pragma pack()
 
   protected:
@@ -308,6 +387,7 @@ class OpalMediaStatistics : public PObject
     // Audio
     unsigned m_averageJitter;
     unsigned m_maximumJitter;
+    unsigned m_jitterBufferDelay;
 
     // Video
     unsigned m_totalFrames;
@@ -315,6 +395,12 @@ class OpalMediaStatistics : public PObject
 
     // Fax
 #if OPAL_FAX
+    enum {
+      FaxNotStarted = -2,
+      FaxInProgress = -1,
+      FaxSuccessful = 0,
+      FaxErrorBase  = 1
+    };
     struct Fax {
       Fax();
 
@@ -435,7 +521,7 @@ class RTP_Session : public PObject
     );
 
     /**Delete a session.
-       This deletes the userData field if autoDeleteUserData is PTrue.
+       This deletes the userData field if autoDeleteUserData is true.
      */
     ~RTP_Session();
   //@}
@@ -462,6 +548,7 @@ class RTP_Session : public PObject
        the SetJitterBufferSize() function.
       */
     unsigned GetJitterBufferSize() const;
+    unsigned GetJitterBufferDelay() const { return GetJitterBufferSize()/GetJitterTimeUnits(); }
     
     /**Get current time units of the jitter buffer.
      */
@@ -469,7 +556,7 @@ class RTP_Session : public PObject
 
     /**Modifies the QOS specifications for this RTP session*/
     virtual PBoolean ModifyQOS(RTP_QOS * )
-    { return PFalse; }
+    { return false; }
 
     /**Read a data frame from the RTP channel.
        This function will conditionally read data from the jitter buffer or
@@ -486,8 +573,7 @@ class RTP_Session : public PObject
        available or an error occurs.
       */
     virtual PBoolean ReadData(
-      RTP_DataFrame & frame,  ///<  Frame read from the RTP session
-      PBoolean loop               ///<  If PTrue, loop as long as data is available, if PFalse, only process once
+      RTP_DataFrame & frame   ///<  Frame read from the RTP session
     ) = 0;
 
     /**Write a data frame from the RTP channel.
@@ -579,6 +665,7 @@ class RTP_Session : public PObject
         DWORD packetsSent;
         DWORD octetsSent;
     };
+
     virtual void OnRxSenderReport(const SenderReport & sender,
                                   const ReceiverReportArray & reports);
     virtual void OnRxReceiverReport(DWORD src,
@@ -602,6 +689,31 @@ class RTP_Session : public PObject
 
     virtual void OnRxApplDefined(const PString & type, unsigned subtype, DWORD src,
                                  const BYTE * data, PINDEX size);
+
+#if OPAL_RTCP_XR
+    class ExtendedReport : public PObject  {
+        PCLASSINFO(ExtendedReport, PObject);
+      public:
+        void PrintOn(ostream &) const;
+
+        DWORD sourceIdentifier;
+        DWORD lossRate;            /* fraction of RTP data packets lost */ 
+        DWORD discardRate;         /* fraction of RTP data packets discarded */
+        DWORD burstDensity;        /* fraction of RTP data packets within burst periods */
+        DWORD gapDensity;          /* fraction of RTP data packets within inter-burst gaps */
+        DWORD roundTripDelay;  /* the most recently calculated round trip time */    
+        DWORD RFactor;            /* voice quality metric of the call */
+        DWORD mosLQ;               /* MOS for listen quality */
+        DWORD mosCQ;               /* MOS for conversational quality */
+        DWORD jbNominal;      /* current nominal jitter buffer delay, in ms */ 
+        DWORD jbMaximum;      /* current maximum jitter buffer delay, in ms */
+        DWORD jbAbsolute;     /* current absolute maximum jitter buffer delay, in ms */
+    };
+    PARRAY(ExtendedReportArray, ExtendedReport);
+
+    virtual void OnRxExtendedReport(DWORD src,
+                                    const ExtendedReportArray & reports);                             
+#endif
   //@}
 
   /**@name Member variable access */
@@ -648,7 +760,7 @@ class RTP_Session : public PObject
       */
     void SetUserData(
       RTP_UserData * data,            ///<  New user data to be used
-      PBoolean autoDeleteUserData = PTrue  ///<  Delete optional data with session.
+      PBoolean autoDeleteUserData = true  ///<  Delete optional data with session.
     );
 
     /**Get the source output identifier.
@@ -665,16 +777,6 @@ class RTP_Session : public PObject
       bool allow    ///<  Flag for allow any SSRC values
     ) { allowAnySyncSource = allow; }
 
-    /**Indicate if will ignore out of order packets.
-      */
-    PBoolean WillIgnoreOutOfOrderPackets() const { return ignoreOutOfOrderPackets; }
-
-    /**Indicate if will ignore out of order packets.
-      */
-    void SetIgnoreOutOfOrderPackets(
-      PBoolean ignore   ///<  Flag for ignore out of order packets
-    ) { ignoreOutOfOrderPackets = ignore; }
-
     /**Indicate if will ignore rtp payload type changes in received packets.
      */
     void SetIgnorePayloadTypeChanges(
@@ -830,7 +932,13 @@ class RTP_Session : public PObject
         This is called when the media stream receives an OpalVideoUpdatePicture
         media command.
       */
-    virtual void SendIntraFrameRequest();
+    virtual void SendIntraFrameRequest(bool rfc2032, bool pictureLoss);
+
+    /** Tell the rtp session to send out an temporal spatial trade off request
+        control packet. This is called when the media stream receives an
+        OpalTemporalSpatialTradeOff media command.
+      */
+    virtual void SendTemporalSpatialTradeOff(unsigned tradeOff);
 
     void SetNextSentSequenceNumber(WORD num) { lastSentSequenceNumber = (WORD)(num-1); }
 
@@ -860,12 +968,27 @@ class RTP_Session : public PObject
     bool HasFailed() const
     { return failed; }
 
-    void AddFilter(const PNotifier & filter);
+    typedef PNotifierTemplate<SendReceiveStatus &> FilterNotifier;
+    #define PDECLARE_RTPFilterNotifier(cls, fn) PDECLARE_NOTIFIER2(RTP_DataFrame, cls, fn, RTP_Session::SendReceiveStatus &)
+    #define PCREATE_RTPFilterNotifier(fn) PCREATE_NOTIFIER2(fn, RTP_Session::SendReceiveStatus &)
+
+    void AddFilter(const FilterNotifier & filter);
+
+#if OPAL_RTCP_XR
+    const RTCP_XR_Metrics & GetMetrics() const { return m_metrics; }
+#endif
+
+    virtual void SendBYE();
 
   protected:
-    virtual void SendBYE();
     void AddReceiverReport(RTP_ControlFrame::ReceiverReport & receiver);
+
     PBoolean InsertReportPacket(RTP_ControlFrame & report);
+    
+#if OPAL_RTCP_XR
+    void InsertExtendedReportPacket(RTP_ControlFrame & report);
+    void OnRxSenderReportToMetrics(const RTP_ControlFrame & frame, PINDEX offset);    
+#endif
 
     PString             m_encoding;
     PMutex              m_encodingMutex;
@@ -882,7 +1005,6 @@ class RTP_Session : public PObject
     typedef PSafePtr<RTP_JitterBuffer, PSafePtrMultiThreaded> JitterBufferPtr;
     JitterBufferPtr m_jitterBuffer;
 
-    PBoolean      ignoreOutOfOrderPackets;
     DWORD         syncSourceOut;
     DWORD         syncSourceIn;
     DWORD         lastSentTimestamp;
@@ -897,12 +1019,20 @@ class RTP_Session : public PObject
     WORD          expectedSequenceNumber;
     PTimeInterval lastSentPacketTime;
     PTimeInterval lastReceivedPacketTime;
+    PTime         lastSRTimestamp;
+    PTime         lastSRReceiveTime;
+    PTimeInterval delaySinceLastSR;
     WORD          lastRRSequenceNumber;
-    PINDEX        consecutiveOutOfOrderPackets;
+    bool          resequenceOutOfOrderPackets;
+    unsigned      consecutiveOutOfOrderPackets;
+    PTimeInterval outOfOrderPacketTime;
+
+    std::list<RTP_DataFrame> m_outOfOrderPackets;
+    void SaveOutOfOrderPacket(RTP_DataFrame & frame);
 
     PMutex        dataMutex;
     DWORD         timeStampOffs;               // offset between incoming media timestamp and timeStampOut
-    PBoolean      oobTimeStampBaseEstablished; // PTrue if timeStampOffs has been established by media
+    PBoolean      oobTimeStampBaseEstablished; // true if timeStampOffs has been established by media
     DWORD         oobTimeStampOutBase;         // base timestamp value for oob data
     PTimeInterval oobTimeStampBase;            // base time for oob timestamp
 
@@ -911,6 +1041,7 @@ class RTP_Session : public PObject
     DWORD rtcpPacketsSent;
     DWORD octetsSent;
     DWORD packetsReceived;
+    DWORD srPacketsReceived;
     DWORD octetsReceived;
     DWORD packetsLost;
     DWORD packetsLostByRemote;
@@ -930,6 +1061,11 @@ class RTP_Session : public PObject
 
     unsigned txStatisticsCount;
     unsigned rxStatisticsCount;
+    
+#if OPAL_RTCP_XR
+    // Calculate the VoIP Metrics for RTCP-XR
+    RTCP_XR_Metrics m_metrics;
+#endif
 
     DWORD    averageSendTimeAccum;
     DWORD    maximumSendTimeAccum;
@@ -950,13 +1086,7 @@ class RTP_Session : public PObject
     PBoolean byeSent;
     bool                failed;      ///<  set to true if session has received too many ICMP destination unreachable
 
-    class Filter : public PObject {
-        PCLASSINFO(Filter, PObject);
-      public:
-        Filter(const PNotifier & n) : notifier(n) { }
-        PNotifier notifier;
-    };
-    PList<Filter> filters;
+    list<FilterNotifier> m_filters;
 };
 
 /**This class is for the IETF Real Time Protocol interface on UDP/IP.
@@ -985,8 +1115,8 @@ class RTP_UDP : public RTP_Session
        returned by this function. It will block until a data frame is
        available or an error occurs.
       */
-    virtual PBoolean ReadData(RTP_DataFrame & frame, PBoolean loop);
-    virtual PBoolean Internal_ReadData(RTP_DataFrame & frame, PBoolean loop);
+    virtual PBoolean ReadData(RTP_DataFrame & frame);
+    virtual PBoolean Internal_ReadData(RTP_DataFrame & frame);
 
     /** Write a data frame to the RTP channel.
       */
@@ -1114,7 +1244,7 @@ class RTP_UDP : public RTP_Session
       PINDEX frameSize,
       PBoolean fromDataChannel
     );
-    
+
     virtual bool WriteDataPDU(RTP_DataFrame & frame);
     virtual bool WriteDataOrControlPDU(
       const BYTE * framePtr,
@@ -1122,6 +1252,8 @@ class RTP_UDP : public RTP_Session
       bool toDataChannel
     );
 
+    virtual void SetEncoding(const PString & newEncoding);
+
 
   protected:
     PIPSocket::Address localAddress;
@@ -1142,9 +1274,13 @@ class RTP_UDP : public RTP_Session
     bool appliedQOS;
     bool remoteIsNAT;
     bool localHasNAT;
-    bool first;
+    bool m_firstData;
+    bool m_firstControl;
     int  badTransmitCounter;
     PTime badTransmitStart;
+
+    PTimer timerWriteDataIdle;
+    PDECLARE_NOTIFIER(PTimer,  RTP_UDP, OnWriteDataIdle);
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -1162,11 +1298,13 @@ class RTP_Encoding
     virtual RTP_Session::SendReceiveStatus OnSendData(RTP_DataFrame & frame);
     virtual PBoolean WriteData(RTP_DataFrame & frame, bool oob);
     virtual PBoolean WriteDataPDU(RTP_DataFrame & frame);
+    virtual void OnWriteDataIdle() {}
+    virtual void SetWriteDataIdleTimer(PTimer &) {}
     virtual RTP_Session::SendReceiveStatus OnSendControl(RTP_ControlFrame & frame, PINDEX & len);
     virtual RTP_Session::SendReceiveStatus ReadDataPDU(RTP_DataFrame & frame);
     virtual RTP_Session::SendReceiveStatus OnReceiveData(RTP_DataFrame & frame);
     virtual RTP_Session::SendReceiveStatus OnReadTimeout(RTP_DataFrame & frame);
-    virtual PBoolean ReadData(RTP_DataFrame & frame, PBoolean loop);
+    virtual PBoolean ReadData(RTP_DataFrame & frame);
     virtual int WaitForPDU(PUDPSocket & dataSocket, PUDPSocket & controlSocket, const PTimeInterval &);
 
     PMutex      mutex;
