@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24162 $
+ * $Revision: 25336 $
  * $Author: rjongbloed $
- * $Date: 2010-03-29 02:46:26 -0500 (Mon, 29 Mar 2010) $
+ * $Date: 2011-03-16 03:51:39 -0500 (Wed, 16 Mar 2011) $
  */
 
 #ifndef OPAL_T38_T38PROTO_H
@@ -44,7 +44,7 @@
 
 #include <opal/mediafmt.h>
 #include <opal/mediastrm.h>
-#include <opal/endpoint.h>
+#include <opal/localep.h>
 
 
 class OpalTransport;
@@ -53,6 +53,15 @@ class PASN_OctetString;
 class OpalFaxConnection;
 
 
+#define OPAL_OPT_STATION_ID  "Station-Id"      ///< String option for fax station ID string
+#define OPAL_NO_G111_FAX     "No-G711-Fax"     ///< Suppress G.711 fall back
+#define OPAL_SUPPRESS_CED    "Suppress-CED"    ///< Suppress transmission of CED tone
+#define OPAL_IGNORE_CED      "Ignore-CED"      ///< Ignore receipt of CED tone
+#define OPAL_T38_SWITCH_TIME "T38-Switch-Time" ///< Seconds for fail safe switch to T.38 mode
+
+#define OPAL_FAX_TIFF_FILE "TIFF-File"
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 class OpalFaxConnection;
@@ -68,9 +77,9 @@ class OpalFaxConnection;
 
     Relies on the presence of the spandsp plug in to do the hard work.
  */
-class OpalFaxEndPoint : public OpalEndPoint
+class OpalFaxEndPoint : public OpalLocalEndPoint
 {
-  PCLASSINFO(OpalFaxEndPoint, OpalEndPoint);
+  PCLASSINFO(OpalFaxEndPoint, OpalLocalEndPoint);
   public:
   /**@name Construction */
   //@{
@@ -87,23 +96,14 @@ class OpalFaxEndPoint : public OpalEndPoint
     ~OpalFaxEndPoint();
   //@}
 
+  /**@name Overrides from OpalEndPoint */
+  //@{
     virtual PSafePtr<OpalConnection> MakeConnection(
       OpalCall & call,          ///<  Owner of connection
       const PString & party,    ///<  Remote party to call
       void * userData = NULL,          ///<  Arbitrary data to pass to connection
       unsigned int options = 0,     ///<  options to pass to conneciton
-      OpalConnection::StringOptions * stringOptions = NULL
-    );
-
-    /**Create a connection for the fax endpoint.
-      */
-    virtual OpalFaxConnection * CreateConnection(
-      OpalCall & call,          ///< Owner of connection
-      void * userData,
-      OpalConnection::StringOptions * stringOptions,
-      const PString & filename, ///< filename to send/receive
-      bool receiving,
-      bool disableT38
+      OpalConnection::StringOptions * stringOptions = NULL  ///< Options to pass to connection
     );
 
     /**Get the data formats this endpoint is capable of operating.
@@ -114,12 +114,23 @@ class OpalFaxEndPoint : public OpalEndPoint
        media formats returned here, but should return no more.
       */
     virtual OpalMediaFormatList GetMediaFormats() const;
+  //@}
 
-  /**@name User Interface operations */
-    /**Accept the incoming connection.
+  /**@name Fax specific operations */
+  //@{
+    /**Determine if the fax plug in is available, that is fax system can be used.
       */
-    virtual void AcceptIncomingConnection(
-      const PString & connectionToken ///<  Token of connection to accept call
+    virtual bool IsAvailable() const;
+
+    /**Create a connection for the fax endpoint.
+      */
+    virtual OpalFaxConnection * CreateConnection(
+      OpalCall & call,          ///< Owner of connection
+      void * userData,          ///<  Arbitrary data to pass to connection
+      OpalConnection::StringOptions * stringOptions, ///< Options to pass to connection
+      const PString & filename, ///< filename to send/receive
+      bool receiving,           ///< Flag for receiving/sending fax
+      bool disableT38           ///< Flag to disable use of T.38
     );
 
     /**Fax transmission/receipt completed.
@@ -169,9 +180,9 @@ class OpalFaxEndPoint : public OpalEndPoint
     switched, to T.38 operation. If the switch fails, then the m_disableT38
     is set and we proceed in fall back mode.
  */
-class OpalFaxConnection : public OpalConnection
+class OpalFaxConnection : public OpalLocalConnection
 {
-  PCLASSINFO(OpalFaxConnection, OpalConnection);
+  PCLASSINFO(OpalFaxConnection, OpalLocalConnection);
   public:
   /**@name Construction */
   //@{
@@ -183,7 +194,7 @@ class OpalFaxConnection : public OpalConnection
       const PString & filename,        ///< TIFF file name to send/receive
       bool receiving,                  ///< True if receiving a fax
       bool disableT38,                 ///< True if want to force G.711
-      OpalConnection::StringOptions * stringOptions = NULL
+      OpalConnection::StringOptions * stringOptions = NULL  ///< Options to pass to connection
     );
 
     /**Destroy endpoint.
@@ -191,48 +202,14 @@ class OpalFaxConnection : public OpalConnection
     ~OpalFaxConnection();
   //@}
 
-  /**@name Overrides from OpalConnection */
+  /**@name Overrides from OpalLocalConnection */
   //@{
     virtual PString GetPrefixName() const;
 
-    /**Get indication of connection being to a "network".
-       This indicates the if the connection may be regarded as a "network"
-       connection. The distinction is about if there is a concept of a "remote"
-       party being connected to and is best described by example: sip, h323,
-       iax and pstn are all "network" connections as they connect to something
-       "remote". While pc, pots and ivr are not as the entity being connected
-       to is intrinsically local.
-      */
-    virtual bool IsNetworkConnection() const { return false; }
-
-    virtual void ApplyStringOptions(OpalConnection::StringOptions & stringOptions);
+    virtual void OnApplyStringOptions();
     virtual OpalMediaFormatList GetMediaFormats() const;
-    virtual void AdjustMediaFormats(bool local, OpalMediaFormatList & mediaFormats, OpalConnection * otherConnection) const;
-
-    /**Start an outgoing connection.
-       This function will initiate the connection to the remote entity, for
-       example in H.323 it sends a SETUP, in SIP it sends an INVITE etc.
-
-       The default behaviour does.
-      */
-    virtual PBoolean SetUpConnection();
-
-    /**Indicate to remote endpoint an alert is in progress.
-       If this is an incoming connection and the AnswerCallResponse is in a
-       AnswerCallDeferred or AnswerCallPending state, then this function is
-       used to indicate to that endpoint that an alert is in progress. This is
-       usually due to another connection which is in the call (the B party)
-       has received an OnAlerting() indicating that its remote endpoint is
-       "ringing".
-
-       The default behaviour does nothing.
-      */
-    virtual PBoolean SetAlerting(
-      const PString & calleeName,   ///<  Name of endpoint being alerted.
-      PBoolean withMedia                ///<  Open media with alerting
-    );
-
-    virtual PBoolean SetConnected();
+    virtual void AdjustMediaFormats(bool local, const OpalConnection * otherConnection, OpalMediaFormatList & mediaFormats) const;
+    virtual void AcceptIncoming();
     virtual void OnEstablished();
     virtual void OnReleased();
     virtual OpalMediaStream * CreateMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, PBoolean isSource);
@@ -242,13 +219,10 @@ class OpalFaxConnection : public OpalConnection
     virtual void OnUserInputTone(char tone, unsigned duration);
     virtual bool SwitchFaxMediaStreams(bool enableFax);
     virtual void OnSwitchedFaxMediaStreams(bool enabledFax);
+  //@}
 
   /**@name New operations */
   //@{
-    /**Accept the incoming connection.
-      */
-    virtual void AcceptIncoming();
-
     /**Fax transmission/receipt completed.
        Default behaviour calls equivalent function on OpalFaxEndPoint.
       */
@@ -276,7 +250,6 @@ class OpalFaxConnection : public OpalConnection
   protected:
     PDECLARE_NOTIFIER(PTimer,  OpalFaxConnection, OnSendCNGCED);
     PDECLARE_NOTIFIER(PThread, OpalFaxConnection, OpenFaxStreams);
-    PDECLARE_NOTIFIER(PThread, OpalFaxConnection, ReleaseConnection);
 
 
     OpalFaxEndPoint & m_endpoint;
@@ -287,8 +260,16 @@ class OpalFaxConnection : public OpalConnection
     PTimeInterval     m_releaseTimeout;
     PTimeInterval     m_switchTimeout;
     OpalMediaFormat   m_tiffFileFormat;
+#if OPAL_STATISTICS
+    void InternalGetStatistics(OpalMediaStatistics & statistics, bool terminate) const;
+    OpalMediaStatistics m_finalStatistics;
+#endif
 
-    bool     m_awaitingSwitchToT38;
+    enum {
+      e_AwaitingSwitchToT38,
+      e_SwitchingToT38,
+      e_CompletedSwitch
+    } m_state;
     PTimer   m_faxTimer;
 
   friend class OpalFaxMediaStream;
