@@ -29,73 +29,73 @@
 /* The Original Code was written by Matthias Schneider <ma30002000@yahoo.de> */
 /*****************************************************************************/
 
-#include "plugin-config.h"
-
 #include "h264frame.h"
-#ifdef _MSC_VER
- #include "../../common/trace.h"
-#else
- #include "trace.h"
-#endif
 #include <stdlib.h>
 #include <string.h>
 
-#ifndef LICENCE_MPL
+#ifdef GPL_HELPER_APP
+  #include "../gpl/enc-ctx.h"
   #ifndef _WIN32
-    #include "../gpl/enc-ctx.h"
     #include "../gpl/x264loader_unix.h"
     extern X264Library X264Lib;
   #else
     #define X264_NAL_ENCODE x264_nal_encode 
   #endif
+#else
+  #include <codec/opalplugin.hpp>
 #endif
+
+
 #define MAX_FRAME_SIZE 128 * 1024
 
+
 H264Frame::H264Frame ()
 {
-  _timestamp = 0;
-  _maxPayloadSize = 1400;
-  _encodedFrame = (uint8_t*)malloc(MAX_FRAME_SIZE);
-  _NALs = NULL;
-  _numberOfNALsReserved = 0;
+  m_timestamp = 0;
+  m_maxPayloadSize = 1400;
+  m_encodedFrame = (uint8_t*)malloc(MAX_FRAME_SIZE);
+  m_NALs = NULL;
+  m_numberOfNALsReserved = 0;
 
   BeginNewFrame();
 }
 
 void H264Frame::BeginNewFrame ()
 {
-  _encodedFrameLen = 0;
+  m_encodedFrameLen = 0;
 
-  _numberOfNALsInFrame = 0;
-  _currentNAL = 0; 
+  m_numberOfNALsInFrame = 0;
+  m_currentNAL = 0; 
 
-  _currentNALFURemainingLen = 0;
-  _currentNALFURemainingDataPtr = NULL;
-  _currentNALFUHeader0 = 0;
-  _currentNALFUHeader1 = 0;
+  m_currentNALFURemainingLen = 0;
+  m_currentNALFURemainingDataPtr = NULL;
+  m_currentNALFUHeader0 = 0;
+  m_currentNALFUHeader1 = 0;
 
-  _currentFU = 0;
+  m_currentFU = 0;
 
 }
 
 H264Frame::~H264Frame ()
 {
-  if (_encodedFrame) free (_encodedFrame);
-  if (_NALs) free(_NALs);
+  if (m_encodedFrame)
+    free(m_encodedFrame);
+  if (m_NALs)
+    free(m_NALs);
 }
 
-#ifndef LICENCE_MPL
+#ifdef GPL_HELPER_APP
 void H264Frame::SetFromFrame (x264_nal_t *NALs, int numberOfNALs) {
   int vopBufferLen;
   int currentNAL = 0;
 
-  uint8_t* currentPositionInFrame=(uint8_t*) _encodedFrame;
-  if (_NALs) free(_NALs);
-  _NALs = (h264_nal_t *)malloc(numberOfNALs * sizeof(h264_nal_t));
+  uint8_t* currentPositionInFrame=(uint8_t*) m_encodedFrame;
+  if (m_NALs) free(m_NALs);
+  m_NALs = (h264_nal_t *)malloc(numberOfNALs * sizeof(h264_nal_t));
 
-  _encodedFrameLen = 0;
-  _numberOfNALsInFrame = 0;
-  _currentNAL=0;
+  m_encodedFrameLen = 0;
+  m_numberOfNALsInFrame = 0;
+  m_currentNAL=0;
 
   // read the nals out of the encoder and create meta data about their location and size in the frame
   for (currentNAL = 0; currentNAL < numberOfNALs; currentNAL++) {
@@ -103,22 +103,21 @@ void H264Frame::SetFromFrame (x264_nal_t *NALs, int numberOfNALs) {
     currentNALLen = X264_NAL_ENCODE(currentPositionInFrame, &vopBufferLen, 1, &NALs[currentNAL]);
     if (currentNALLen > 0) 
     {
-      _NALs[_numberOfNALsInFrame].length = currentNALLen;
-      _NALs[_numberOfNALsInFrame].offset = _encodedFrameLen;
-      _NALs[_numberOfNALsInFrame].type = NALs[currentNAL].i_type;
+      m_NALs[m_numberOfNALsInFrame].length = currentNALLen;
+      m_NALs[m_numberOfNALsInFrame].offset = m_encodedFrameLen;
+      m_NALs[m_numberOfNALsInFrame].type = NALs[currentNAL].i_type;
       uint32_t header = 0;
       if (IsStartCode(currentPositionInFrame))
       {
 	header = currentPositionInFrame[2] == 1 ? 3 : 4;
       }
-      _NALs[_numberOfNALsInFrame].length -= header;
-      _NALs[_numberOfNALsInFrame].offset += header;
-      TRACE_UP(4, "H264\tEncap\tLoaded NAL unit #" << currentNAL << " - type " << NALs[currentNAL].i_type);
+      m_NALs[m_numberOfNALsInFrame].length -= header;
+      m_NALs[m_numberOfNALsInFrame].offset += header;
+      PTRACE(6, "x264", "Loaded NAL unit #" << currentNAL << " - type " << NALs[currentNAL].i_type);
 
       uint8_t* NALptr = NALs[currentNAL].p_payload;
-      if ( Trace::CanTraceUserPlane(4) && (NALs[currentNAL].i_type == H264_NAL_TYPE_SEQ_PARAM)) 
-      {
-      TRACE_UP(4,   "H264\tEncap\tProfile: " << (int)NALptr[0] << 
+      if (PTRACE_CHECK(4) && (NALs[currentNAL].i_type == H264_NAL_TYPE_SEQ_PARAM)) {
+        PTRACE(4, "x264", "Profile: " << (int)NALptr[0] << 
                                 " Level: "   << (int)NALptr[2] << 
 		   	        " Constraints: " << (NALptr[1] & 0x80 ? 1 : 0) 
 			                         << (NALptr[1] & 0x40 ? 1 : 0) 
@@ -126,16 +125,16 @@ void H264Frame::SetFromFrame (x264_nal_t *NALs, int numberOfNALs) {
 					         << (NALptr[1] & 0x10 ? 1 : 0));
       }
 
-      _numberOfNALsInFrame++;
-      _encodedFrameLen += currentNALLen;
+      m_numberOfNALsInFrame++;
+      m_encodedFrameLen += currentNALLen;
       currentPositionInFrame += currentNALLen;
     } 
     else
     {
-      printf("[enc] Need to increase vop buffer size by %d\n", -currentNALLen);
+      PTRACE(4, "x264", "[enc] Need to increase vop buffer size by  " << -currentNALLen);
     }
   }
-  TRACE_UP(4, "H264\tEncap\tLoaded an encoded frame of " << _encodedFrameLen << " bytes consisiting of " << numberOfNALs << " NAL units");
+  PTRACE(6, "x264", "Loaded an encoded frame of " << m_encodedFrameLen << " bytes consisiting of " << numberOfNALs << " NAL units");
 }
 #endif
 
@@ -143,10 +142,10 @@ bool H264Frame::GetRTPFrame(RTPFrame & frame, unsigned int & flags)
 {
   flags = 0;
   flags |= (IsSync()) ? isIFrame : 0;
-  if (_currentNAL < _numberOfNALsInFrame) 
+  if (m_currentNAL < m_numberOfNALsInFrame) 
   { 
-    uint32_t curNALLen = _NALs[_currentNAL].length;
-    const uint8_t *curNALPtr = _encodedFrame + _NALs[_currentNAL].offset;
+    uint32_t curNALLen = m_NALs[m_currentNAL].length;
+    const uint8_t *curNALPtr = m_encodedFrame + m_NALs[m_currentNAL].offset;
     /*
      * We have 3 types of packets we can send:
      * fragmentation units - if the NAL is > max_payload_size
@@ -156,7 +155,7 @@ bool H264Frame::GetRTPFrame(RTPFrame & frame, unsigned int & flags)
      * We don't send multiple time aggregation units
      */
 
-    if (curNALLen > _maxPayloadSize)
+    if (curNALLen > m_maxPayloadSize)
     {
       // fragmentation unit - break up into max_payload_size size chunks
       return EncapsulateFU(frame, flags);
@@ -165,20 +164,20 @@ bool H264Frame::GetRTPFrame(RTPFrame & frame, unsigned int & flags)
     {
       // it is the last NAL of that frame or doesnt fit into an STAP packet with next nal ?
 #ifdef SEND_STAP_PACKETS
-      if (((_currentNAL + 1) >= _numberOfNALsInFrame)  ||  
-	  ((curNALLen + _NALs[_currentNAL + 1].length + 5) > _maxPayloadSize)) 
+      if (((m_currentNAL + 1) >= m_numberOfNALsInFrame)  ||  
+	  ((curNALLen + m_NALs[m_currentNAL + 1].length + 5) > m_maxPayloadSize)) 
       { 
 #endif
         // single nal unit packet
 
         frame.SetPayloadSize(curNALLen);
         memcpy(frame.GetPayloadPtr(), curNALPtr, curNALLen);
-        frame.SetTimestamp(_timestamp);
-        frame.SetMarker((_currentNAL + 1) >= _numberOfNALsInFrame ? 1 : 0);
+        frame.SetTimestamp(m_timestamp);
+        frame.SetMarker((m_currentNAL + 1) >= m_numberOfNALsInFrame ? 1 : 0);
         flags |= frame.GetMarker() ? isLastFrame : 0;  // marker bit on last frame of video
 
-        TRACE_UP(4, "H264\tEncap\tEncapsulating NAL unit #" << _currentNAL << "/" << (_numberOfNALsInFrame-1) << " of " << curNALLen << " bytes as a regular NAL unit");
-        _currentNAL++;
+        PTRACE(6, "x264", "Encapsulating NAL unit #" << m_currentNAL << "/" << (m_numberOfNALsInFrame-1) << " of " << curNALLen << " bytes as a regular NAL unit");
+        m_currentNAL++;
         return true;
 #ifdef SEND_STAP_PACKETS
       } 
@@ -197,53 +196,53 @@ bool H264Frame::GetRTPFrame(RTPFrame & frame, unsigned int & flags)
 
 bool H264Frame::EncapsulateSTAP (RTPFrame & frame, unsigned int & flags) {
   uint32_t STAPLen = 1;
-  uint32_t highestNALNumberInSTAP = _currentNAL;
+  uint32_t highestNALNumberInSTAP = m_currentNAL;
   
   // first check how many nals we want to put into the packet
   do {
     STAPLen += 2;
-    STAPLen +=  _NALs[highestNALNumberInSTAP].length;
+    STAPLen +=  m_NALs[highestNALNumberInSTAP].length;
     highestNALNumberInSTAP++;
-  } while (highestNALNumberInSTAP < _numberOfNALsInFrame && STAPLen < _maxPayloadSize);
+  } while (highestNALNumberInSTAP < m_numberOfNALsInFrame && STAPLen < m_maxPayloadSize);
 
-  if (STAPLen > _maxPayloadSize)
+  if (STAPLen > m_maxPayloadSize)
   {
     STAPLen -= 2;
-    STAPLen -= _NALs[(highestNALNumberInSTAP-1)].length;
+    STAPLen -= m_NALs[(highestNALNumberInSTAP-1)].length;
     highestNALNumberInSTAP--;
   }
 
-  TRACE_UP(4, "H264\tEncap\tEncapsulating NAL units " << _currentNAL << "-"<< (highestNALNumberInSTAP-1) << "/" << (_numberOfNALsInFrame-1) << " as a STAP of " << STAPLen);
+  PTRACE(6, "x264", "Encapsulating NAL units " << m_currentNAL << "-"<< (highestNALNumberInSTAP-1) << "/" << (m_numberOfNALsInFrame-1) << " as a STAP of " << STAPLen);
 
   frame.SetPayloadSize(1); // for stap header
 
   uint32_t curNALLen;
   const uint8_t* curNALPtr;
   uint8_t  maxNRI = 0;
-  while (_currentNAL < highestNALNumberInSTAP) {
-    curNALLen = _NALs[_currentNAL].length;
-    curNALPtr = _encodedFrame + _NALs[_currentNAL].offset;
+  while (m_currentNAL < highestNALNumberInSTAP) {
+    curNALLen = m_NALs[m_currentNAL].length;
+    curNALPtr = m_encodedFrame + m_NALs[m_currentNAL].offset;
 
     // store the nal length information
     frame.SetPayloadSize(frame.GetPayloadSize() + 2);
-    *((uint8_t*)frame.GetPayloadPtr() + frame.GetPayloadSize() - 2) = curNALLen >> 8;
-    *((uint8_t*)frame.GetPayloadPtr() + frame.GetPayloadSize() - 1) = curNALLen & 0xff;
+    *((uint8_t*)frame.GetPayloadPtr() + frame.GetPayloadSize() - 2) = (uint8_t)(curNALLen >> 8);
+    *((uint8_t*)frame.GetPayloadPtr() + frame.GetPayloadSize() - 1) = (uint8_t) curNALLen;
 
     // store the nal
     frame.SetPayloadSize(frame.GetPayloadSize() + curNALLen);
     memcpy ((uint8_t*)frame.GetPayloadPtr() + frame.GetPayloadSize() - curNALLen, (void *)curNALPtr, curNALLen);
 
     if ((*curNALPtr & 0x60) > maxNRI) maxNRI = *curNALPtr & 0x60;
-    TRACE_UP(4, "H264\tEncap\tAdding NAL unit " << _currentNAL << "/" << (_numberOfNALsInFrame-1) << " of " << curNALLen << " bytes to STAP");
-    _currentNAL++;
+    PTRACE(6, "x264", "Adding NAL unit " << m_currentNAL << "/" << (m_numberOfNALsInFrame-1) << " of " << curNALLen << " bytes to STAP");
+    m_currentNAL++;
   }
 
   // set the nri value in the stap header
   //uint8_t stap = 24 | maxNRI;
   //memcpy (frame.GetPayloadPtr(),&stap,1);
   memset (frame.GetPayloadPtr(), 24 | maxNRI, 1);
-  frame.SetTimestamp(_timestamp);
-  frame.SetMarker(_currentNAL >= _numberOfNALsInFrame ? 1 : 0);
+  frame.SetTimestamp(m_timestamp);
+  frame.SetMarker(m_currentNAL >= m_numberOfNALsInFrame ? 1 : 0);
   flags |= frame.GetMarker() ? isLastFrame : 0;  // marker bit on last frame of video
 
   return true;
@@ -254,64 +253,68 @@ bool H264Frame::EncapsulateFU(RTPFrame & frame, unsigned int & flags) {
   uint8_t header[2];
   uint32_t curFULen;
 
-  if ((_currentNALFURemainingLen==0) || (_currentNALFURemainingDataPtr==NULL))
+  if ((m_currentNALFURemainingLen==0) || (m_currentNALFURemainingDataPtr==NULL))
   {
-    _currentNALFURemainingLen = _NALs[_currentNAL].length;
-    _currentNALFURemainingDataPtr = _encodedFrame + _NALs[_currentNAL].offset;
-    _currentNALFUHeader0 = (*_currentNALFURemainingDataPtr & 0x60) | 28;
-    _currentNALFUHeader1 = *_currentNALFURemainingDataPtr & 0x1f;
-    header[0] = _currentNALFUHeader0;
-    header[1] = 0x80 | _currentNALFUHeader1; // s indication
-    _currentNALFURemainingDataPtr++; // remove the first byte
-    _currentNALFURemainingLen--;
+    m_currentNALFURemainingLen = m_NALs[m_currentNAL].length;
+    m_currentNALFURemainingDataPtr = m_encodedFrame + m_NALs[m_currentNAL].offset;
+    m_currentNALFUHeader0 = (*m_currentNALFURemainingDataPtr & 0x60) | 28;
+    m_currentNALFUHeader1 = *m_currentNALFURemainingDataPtr & 0x1f;
+    header[0] = m_currentNALFUHeader0;
+    header[1] = 0x80 | m_currentNALFUHeader1; // s indication
+    m_currentNALFURemainingDataPtr++; // remove the first byte
+    m_currentNALFURemainingLen--;
   }
   else
   {
-    header[0] = _currentNALFUHeader0;
-    header[1] = _currentNALFUHeader1;
+    header[0] = m_currentNALFUHeader0;
+    header[1] = m_currentNALFUHeader1;
   }
 
-  if (_currentNALFURemainingLen > 0)
+  if (m_currentNALFURemainingLen > 0)
   {
     bool last = false;
-    if ((_currentNALFURemainingLen + 2) <= _maxPayloadSize)
+    if ((m_currentNALFURemainingLen + 2) <= m_maxPayloadSize)
     {
       header[1] |= 0x40;
-      curFULen = _currentNALFURemainingLen;
+      curFULen = m_currentNALFURemainingLen;
       last = true;
     }
     else
     {
-      curFULen = _maxPayloadSize - 2;
+      curFULen = m_maxPayloadSize - 2;
     }
 
     frame.SetPayloadSize(curFULen + 2);
     memcpy ((uint8_t*)frame.GetPayloadPtr(), header, 2);
-    memcpy ((uint8_t*)frame.GetPayloadPtr()+2, _currentNALFURemainingDataPtr, curFULen);
-    frame.SetTimestamp(_timestamp);
-    frame.SetMarker((last && ((_currentNAL+1) >= _numberOfNALsInFrame)) ? 1 : 0);
+    memcpy ((uint8_t*)frame.GetPayloadPtr()+2, m_currentNALFURemainingDataPtr, curFULen);
+    frame.SetTimestamp(m_timestamp);
+    frame.SetMarker((last && ((m_currentNAL+1) >= m_numberOfNALsInFrame)) ? 1 : 0);
     flags |= frame.GetMarker() ? isLastFrame : 0;  // marker bit on last frame of video
 
-    _currentNALFURemainingDataPtr += curFULen;
-    _currentNALFURemainingLen -= curFULen;
-    TRACE_UP(4, "H264\tEncap\tEncapsulating "<< curFULen << " bytes of NAL " << _currentNAL<< "/" << (_numberOfNALsInFrame-1) << " as a FU (" << _currentNALFURemainingLen << " bytes remaining)");
+    m_currentNALFURemainingDataPtr += curFULen;
+    m_currentNALFURemainingLen -= curFULen;
+    PTRACE(6, "x264", "Encapsulating "<< curFULen << " bytes of NAL " << m_currentNAL<< "/" << (m_numberOfNALsInFrame-1) << " as a FU (" << m_currentNALFURemainingLen << " bytes remaining)");
   } 
-  if (_currentNALFURemainingLen==0)
+  if (m_currentNALFURemainingLen==0)
   {
-    _currentNAL++;
-    _currentNALFURemainingDataPtr=NULL;
+    m_currentNAL++;
+    m_currentNALFURemainingDataPtr=NULL;
   }
   return true;
 }
 
-bool H264Frame::SetFromRTPFrame(RTPFrame & frame, unsigned int & flags) {
+bool H264Frame::SetFromRTPFrame(RTPFrame & frame, unsigned int & flags)
+{
+  if (frame.GetPayloadSize() == 0)
+    return true;
+
   uint8_t curNALType = *(frame.GetPayloadPtr()) & 0x1f;
 
   if (curNALType >= H264_NAL_TYPE_NON_IDR_SLICE &&
       curNALType <= H264_NAL_TYPE_FILLER_DATA)
   {
     // regular NAL - put in buffer, adding the header.
-    TRACE_UP(4, "H264\tDeencap\tDeencapsulating a regular NAL unit NAL of " << frame.GetPayloadSize() - 1 << " bytes (type " << (int) curNALType << ")");
+    PTRACE(6, "x264", "Deencapsulating a regular NAL unit NAL of " << frame.GetPayloadSize() - 1 << " bytes (type " << (int) curNALType << ")");
     AddDataToEncodedFrame(frame.GetPayloadPtr() + 1, frame.GetPayloadSize() - 1, *(frame.GetPayloadPtr()), 1);
   } 
   else if (curNALType == 24) 
@@ -326,7 +329,7 @@ bool H264Frame::SetFromRTPFrame(RTPFrame & frame, unsigned int & flags) {
   }
   else
   {
-    TRACE_UP(4, "H264\tDeencap\tSkipping unsupported NAL unit type " << curNALType);
+    PTRACE(2, "x264", "Skipping unsupported NAL unit type " << curNALType);
     return false;
   }
   return true;
@@ -334,11 +337,11 @@ bool H264Frame::SetFromRTPFrame(RTPFrame & frame, unsigned int & flags) {
 bool H264Frame::IsSync () {
   uint32_t i;
 
-  for (i=0; i<_numberOfNALsInFrame; i++)
+  for (i=0; i < m_numberOfNALsInFrame; i++)
   {
-    if ((_NALs[i].type == H264_NAL_TYPE_IDR_SLICE) ||
-        (_NALs[i].type == H264_NAL_TYPE_SEQ_PARAM) ||
-        (_NALs[i].type == H264_NAL_TYPE_PIC_PARAM))
+    if ((m_NALs[i].type == H264_NAL_TYPE_IDR_SLICE) ||
+        (m_NALs[i].type == H264_NAL_TYPE_SEQ_PARAM) ||
+        (m_NALs[i].type == H264_NAL_TYPE_PIC_PARAM))
     {
       return true;
     }
@@ -350,7 +353,7 @@ bool H264Frame::DeencapsulateSTAP (RTPFrame & frame, unsigned int & /*flags*/) {
   uint8_t* curSTAP = frame.GetPayloadPtr() + 1;
   uint32_t curSTAPLen = frame.GetPayloadSize() - 1; 
 
-  TRACE_UP(4, "H264\tDeencap\tDeencapsulating a STAP of " << curSTAPLen << " bytes");
+  PTRACE(6, "x264", "Deencapsulating a STAP of " << curSTAPLen << " bytes");
   while (curSTAPLen > 0)
   {
     // first, theres a 2 byte length field
@@ -358,13 +361,13 @@ bool H264Frame::DeencapsulateSTAP (RTPFrame & frame, unsigned int & /*flags*/) {
     curSTAP += 2;
     // then the header, followed by the body.  We'll add the header
     // in the AddDataToEncodedFrame - that's why the nal body is dptr + 1
-    TRACE_UP(4, "H264\tDeencap\tDeencapsulating an NAL unit of " << len << " bytes (type " << (int)(*curSTAP && 0x1f) << ") from STAP");
+    PTRACE(6, "x264", "Deencapsulating an NAL unit of " << len << " bytes (type " << (int)(*curSTAP && 0x1f) << ") from STAP");
     AddDataToEncodedFrame(curSTAP + 1,  len - 1, *curSTAP, 1);
     curSTAP += len;
     if ((len + 2) > curSTAPLen)
     {
       curSTAPLen = 0;
-      TRACE(1, "H264\tDeencap\tError deencapsulating STAP, STAP header says its " << len + 2 << " bytes long but there are only " << curSTAPLen << " bytes left of the packet");
+      PTRACE(2, "x264", "Error deencapsulating STAP, STAP header says its " << len + 2 << " bytes long but there are only " << curSTAPLen << " bytes left of the packet");
       return false;
     }
     else
@@ -382,51 +385,51 @@ bool H264Frame::DeencapsulateFU (RTPFrame & frame, unsigned int & /*flags*/) {
 
   if ((curFUPtr[1] & 0x80) && !(curFUPtr[1] & 0x40))
   {
-    TRACE_UP(4, "H264\tDeencap\tDeencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (_Startbit_, !Endbit)");
-    if (_currentFU) {
-      _currentFU=1;
+    PTRACE(6, "x264", "Deencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (Startbit, !Endbit)");
+    if (m_currentFU) {
+      m_currentFU=1;
     }
     else
     {
-      _currentFU++;
+      m_currentFU++;
       header = (curFUPtr[0] & 0xe0) | (curFUPtr[1] & 0x1f);
       AddDataToEncodedFrame(curFUPtr + 2, curFULen - 2, header,  1);
     }
   } 
   else if (!(curFUPtr[1] & 0x80) && !(curFUPtr[1] & 0x40))
   {
-    TRACE_UP(4, "H264\tDeencap\tDeencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (!Startbit, !Endbit)");
-    if (_currentFU)
+    PTRACE(6, "x264", "Deencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (!Startbit, !Endbit)");
+    if (m_currentFU)
     {
-      _currentFU++;
+      m_currentFU++;
       AddDataToEncodedFrame(curFUPtr + 2, curFULen - 2,  0, 0);
     }
     else
     {
-      _currentFU=0;
-      TRACE(1, "H264\tDeencap\tReceived an intermediate FU without getting the first - dropping!");
+      m_currentFU=0;
+      PTRACE(2, "x264", "Received an intermediate FU without getting the first - dropping!");
       return false;
     }
   } 
   else if (!(curFUPtr[1] & 0x80) && (curFUPtr[1] & 0x40))
   {
-    TRACE_UP(4, "H264\tDeencap\tDeencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (!Startbit, _Endbit_)");
-    if (_currentFU) {
-      _currentFU=0;
+    PTRACE(6, "x264", "Deencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (!Startbit, Endbit)");
+    if (m_currentFU) {
+      m_currentFU=0;
       AddDataToEncodedFrame( curFUPtr + 2, curFULen - 2, 0, 0);
     }
     else
     {
-      _currentFU=0;
-      TRACE(1, "H264\tDeencap\tReceived a last FU without getting the first - dropping!");
+      m_currentFU=0;
+      PTRACE(2, "x264", "Received a last FU without getting the first - dropping!");
       return false;
     }
   } 
   else if ((curFUPtr[1] & 0x80) && (curFUPtr[1] & 0x40))
   {
-    TRACE_UP(4, "H264\tDeencap\tDeencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (_Startbit_, _Endbit_)");
-    TRACE(1, "H264\tDeencap\tReceived a FU with both Starbit and Endbit set - This MUST NOT happen!");
-    _currentFU=0;
+    PTRACE(6, "x264", "Deencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (Startbit, Endbit)");
+    PTRACE(2, "x264", "Received a FU with both Starbit and Endbit set - This MUST NOT happen!");
+    m_currentFU=0;
     return false;
   } 
   return true;
@@ -434,15 +437,15 @@ bool H264Frame::DeencapsulateFU (RTPFrame & frame, unsigned int & /*flags*/) {
 
 void H264Frame::AddDataToEncodedFrame (uint8_t *data, uint32_t dataLen, uint8_t header, bool addHeader) {
   uint8_t headerLen= addHeader ? 5 : 0;
-  uint8_t* currentPositionInFrame = _encodedFrame + _encodedFrameLen;
+  uint8_t* currentPositionInFrame = m_encodedFrame + m_encodedFrameLen;
 
   if (addHeader) 
   {
-    TRACE_UP(4, "H264\tDeencap\tAdding a NAL unit of " << dataLen << " bytes to buffer (type " << (int)(header & 0x1f) << ")"); 
+    PTRACE(6, "x264", "Adding a NAL unit of " << dataLen << " bytes to buffer (type " << (int)(header & 0x1f) << ")"); 
     uint8_t* NALptr = data;
-    if ( Trace::CanTraceUserPlane(4) && ((header & 0x1f) == H264_NAL_TYPE_SEQ_PARAM) && (dataLen >= 3)) 
+    if (((header & 0x1f) == H264_NAL_TYPE_SEQ_PARAM) && (dataLen >= 3)) 
     {
-      TRACE_UP(4, "H264\tDeencap\tProfile: " << (int)NALptr[0] << 
+      PTRACE(4, "x264", "Profile: " << (int)NALptr[0] << 
                                 " Level: "   << (int)NALptr[2] << 
 			        " Constraints: " << (NALptr[1] & 0x80 ? 1 : 0) 
 			                         << (NALptr[1] & 0x40 ? 1 : 0) 
@@ -450,10 +453,11 @@ void H264Frame::AddDataToEncodedFrame (uint8_t *data, uint32_t dataLen, uint8_t
 					         << (NALptr[1] & 0x10 ? 1 : 0));
     }
   }
-  else TRACE_UP(4, "H264\tDeencap\tAdding a NAL unit of " << dataLen << " bytes to buffer");
+  else
+    PTRACE(6, "x264", "Adding a NAL unit of " << dataLen << " bytes to buffer");
 
-  if (_encodedFrameLen + dataLen + headerLen > MAX_FRAME_SIZE) {
-    TRACE(1, "H264\tDeencap\tFrame too big (" << _encodedFrameLen + dataLen + headerLen << ">" << MAX_FRAME_SIZE << ")");
+  if (m_encodedFrameLen + dataLen + headerLen > MAX_FRAME_SIZE) {
+    PTRACE(2, "x264", "Frame too big (" << m_encodedFrameLen + dataLen + headerLen << ">" << MAX_FRAME_SIZE << ")");
     return;
   }
 
@@ -465,33 +469,34 @@ void H264Frame::AddDataToEncodedFrame (uint8_t *data, uint32_t dataLen, uint8_t
     *currentPositionInFrame++ = 0;
     *currentPositionInFrame++ = 1;
 
-    if (_numberOfNALsInFrame + 1 >(_numberOfNALsReserved))
+    if (m_numberOfNALsInFrame + 1 >(m_numberOfNALsReserved))
     {
-      _NALs = (h264_nal_t *)realloc(_NALs, (_numberOfNALsReserved + 1) * sizeof(h264_nal_t));
-      _numberOfNALsReserved++;
+      m_NALs = (h264_nal_t *)realloc(m_NALs, (m_numberOfNALsReserved + 1) * sizeof(h264_nal_t));
+      m_numberOfNALsReserved++;
     }
-    if (_NALs)
+    if (m_NALs)
     {
-      _NALs[_numberOfNALsInFrame].offset = _encodedFrameLen + 4;
-      _NALs[_numberOfNALsInFrame].length = dataLen + 1;
-      _NALs[_numberOfNALsInFrame].type = header & 0x1f;
+      m_NALs[m_numberOfNALsInFrame].offset = m_encodedFrameLen + 4;
+      m_NALs[m_numberOfNALsInFrame].length = dataLen + 1;
+      m_NALs[m_numberOfNALsInFrame].type = header & 0x1f;
 
 
-      _numberOfNALsInFrame++;
+      m_numberOfNALsInFrame++;
     }
 
     *currentPositionInFrame++ = header;
   }
   else
   {
-    if (_NALs) _NALs[_numberOfNALsInFrame - 1].length += dataLen;
+    if (m_NALs)
+      m_NALs[m_numberOfNALsInFrame - 1].length += dataLen;
   }
 
-  TRACE_UP (4, "H264\tDeencap\tReserved memory for  " <<_numberOfNALsReserved <<" NALs, Inframe/current: "<< _numberOfNALsInFrame <<" Offset: "
-    <<_NALs[_numberOfNALsInFrame-1].offset << " Length: "<< _NALs[_numberOfNALsInFrame-1].length << " Type: "<< (int)(_NALs[_numberOfNALsInFrame-1].type));
+  PTRACE(6, "x264", "Reserved memory for  " << m_numberOfNALsReserved <<" NALs, Inframe/current: "<< m_numberOfNALsInFrame <<" Offset: "
+    << m_NALs[m_numberOfNALsInFrame-1].offset << " Length: "<< m_NALs[m_numberOfNALsInFrame-1].length << " Type: "<< (int)(m_NALs[m_numberOfNALsInFrame-1].type));
 
   memcpy(currentPositionInFrame, data, dataLen);
-  _encodedFrameLen += dataLen + headerLen;
+  m_encodedFrameLen += dataLen + headerLen;
 }
 
 bool H264Frame::IsStartCode (const uint8_t *positionInFrame)
