@@ -54,6 +54,8 @@
 
   HINSTANCE hDLL;
 
+  #define snprintf _snprintf
+
 #else // _WIN32
 
   #include <memory.h>
@@ -77,6 +79,7 @@ OpalHandle              hOPAL;
 
 char * CurrentCallToken;
 char * HeldCallToken;
+char * PlayScript;
 
 
 
@@ -188,7 +191,8 @@ int InitialiseOPAL()
                                   OPAL_PREFIX_H323  " "
                                   OPAL_PREFIX_SIP   " "
                                   OPAL_PREFIX_IAX2  " "
-                                  LOCAL_PREFIX
+                                  LOCAL_PREFIX      " "
+                                  OPAL_PREFIX_IVR
                                   " TraceLevel=4")) == NULL) {
     fputs("Could not initialise OPAL\n", stderr);
     return 0;
@@ -227,6 +231,17 @@ int InitialiseOPAL()
 
   FreeMessageFunction(response);
 
+  memset(&command, 0, sizeof(command));
+  command.m_type = OpalCmdSetProtocolParameters;
+
+  command.m_param.m_protocol.m_prefix = "sip";
+  command.m_param.m_protocol.m_defaultOptions = "PRACK-Mode=0\nInitial-Offer=false";
+
+  if ((response = MySendCommand(&command, "Could not set SIP options")) == NULL)
+    return 0;
+
+  FreeMessageFunction(response);
+
   return 1;
 }
 
@@ -324,6 +339,18 @@ static void HandleMessages(unsigned timeout)
 
       case OpalIndEstablished :
         puts("Established.\n");
+
+        if (PlayScript != NULL) {
+          printf("Playing %s\n", PlayScript);
+
+          memset(&command, 0, sizeof(command));
+          command.m_type = OpalCmdTransferCall;
+          command.m_param.m_callSetUp.m_callToken = CurrentCallToken;
+          command.m_param.m_callSetUp.m_partyA = "pc:";
+          command.m_param.m_callSetUp.m_partyB = PlayScript;
+          if ((response = MySendCommand(&command, "Could not start playing")) != NULL)
+            FreeMessageFunction(response);
+        }
         break;
 
       case OpalIndMediaStream :
@@ -530,6 +557,37 @@ int DoRecord(const char * to, const char * file)
 }
 
 
+int DoPlay(const char * to, const char * file)
+{
+  // Example cmd line: call 612@ekiga.net
+  OpalMessage command;
+  OpalMessage * response;
+
+
+  printf("Playing %s to %s\n", file, to);
+
+  PlayScript = (char *)malloc(1000);
+  snprintf(PlayScript, 999,
+           "ivr:<?xml version=\"1.0\"?>"
+           "<vxml version=\"1.0\">"
+             "<form id=\"PlayFile\">"
+               "<audio src=\"%s\"/>"
+             "</form>"
+           "</vxml>", file);
+
+  memset(&command, 0, sizeof(command));
+  command.m_type = OpalCmdSetUpCall;
+  command.m_param.m_callSetUp.m_partyB = to;
+  if ((response = MySendCommand(&command, "Could not make call")) == NULL)
+    return 0;
+
+  CurrentCallToken = strdup(response->m_param.m_callSetUp.m_callToken);
+  FreeMessageFunction(response);
+
+  return 1;
+}
+
+
 typedef enum
 {
   OpListen,
@@ -541,14 +599,15 @@ typedef enum
   OpRegister,
   OpSubscribe,
   OpRecord,
+  OpPlay,
   NumOperations
 } Operations;
 
 static const char * const OperationNames[NumOperations] =
-  { "listen", "call", "mute", "hold", "transfer", "consult", "register", "subscribe", "record" };
+  { "listen", "call", "mute", "hold", "transfer", "consult", "register", "subscribe", "record", "play" };
 
 static int const RequiredArgsForOperation[NumOperations] =
-  { 2, 3, 3, 3, 4, 4, 3, 3, 3 };
+  { 2, 3, 3, 3, 4, 4, 3, 3, 3, 3 };
 
 
 static Operations GetOperation(const char * name)
@@ -666,6 +725,12 @@ int main(int argc, char * argv[])
       HandleMessages(INT_MAX); // More or less forever
       break;
 
+    case OpPlay :
+      if (!DoPlay(argv[2], argv[3]))
+        break;
+      HandleMessages(INT_MAX); // More or less forever
+      break;
+
     default :
       break;
   }
