@@ -18,9 +18,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23831 $
+ * $Revision: 24244 $
  * $Author: rjongbloed $
- * $Date: 2009-12-03 04:21:07 -0600 (Thu, 03 Dec 2009) $
+ * $Date: 2010-04-14 02:17:13 -0500 (Wed, 14 Apr 2010) $
  */
 
 /*
@@ -68,8 +68,20 @@ class SDPH224MediaDescription : public SDPRTPAVPMediaDescription
 {
   PCLASSINFO(SDPH224MediaDescription, SDPRTPAVPMediaDescription);
   public:
-    SDPH224MediaDescription(const OpalTransportAddress & address);
-    virtual PString GetSDPMediaType() const;
+    SDPH224MediaDescription(const OpalTransportAddress & address)
+      : SDPRTPAVPMediaDescription(address, OpalH224MediaType::MediaType())
+    {
+    }
+
+    virtual SDPMediaDescription * CreateEmpty() const
+    {
+      return new SDPH224MediaDescription(OpalTransportAddress());
+    }
+
+    virtual PString GetSDPMediaType() const
+    {
+      return "application";
+    }
 };
 
 SDPMediaDescription * OpalH224MediaType::CreateSDPMediaDescription(const OpalTransportAddress & localAddress)
@@ -77,16 +89,6 @@ SDPMediaDescription * OpalH224MediaType::CreateSDPMediaDescription(const OpalTra
   return new SDPH224MediaDescription(localAddress);
 }
 
-SDPH224MediaDescription::SDPH224MediaDescription(const OpalTransportAddress & address)
-  : SDPRTPAVPMediaDescription(address, OpalH224MediaType::MediaType())
-{
-}
-
-PString SDPH224MediaDescription::GetSDPMediaType() const
-{
-  return "application";
-}
-
 #endif
 
 /////////////////////////////////////////////////////////////////////////
