@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23945 $
+ * $Revision: 25210 $
  * $Author: rjongbloed $
- * $Date: 2010-01-18 16:46:15 -0600 (Mon, 18 Jan 2010) $
+ * $Date: 2011-02-28 08:24:23 -0600 (Mon, 28 Feb 2011) $
  */
 
 #include <ptlib.h>
@@ -238,24 +238,40 @@ void H323Channel::OnMiscellaneousCommand(const H245_MiscellaneousCommand_type &
     return;
 
 #if OPAL_VIDEO
-  switch (type.GetTag()) {
-    case H245_MiscellaneousCommand_type::e_videoFastUpdatePicture :
+  switch (type.GetTag())
+  {
+    case H245_MiscellaneousCommand_type::e_videoFreezePicture :
+      mediaStream->ExecuteCommand(OpalVideoFreezePicture());
+      break;
+
+    case H245_MiscellaneousCommand_type::e_videoFastUpdatePicture:
       mediaStream->ExecuteCommand(OpalVideoUpdatePicture());
       break;
 
     case H245_MiscellaneousCommand_type::e_videoFastUpdateGOB :
-      {
-        const H245_MiscellaneousCommand_type_videoFastUpdateGOB & vfuGOB = type;
-        mediaStream->ExecuteCommand(OpalVideoUpdatePicture(vfuGOB.m_firstGOB, -1, vfuGOB.m_numberOfGOBs));
-      }
+    case H245_MiscellaneousCommand_type::e_videoFastUpdateMB :
+      mediaStream->ExecuteCommand(OpalVideoPictureLoss());
       break;
 
-    case H245_MiscellaneousCommand_type::e_videoFastUpdateMB :
-      {
-        const H245_MiscellaneousCommand_type_videoFastUpdateMB & vfuMB = type;
-        mediaStream->ExecuteCommand(OpalVideoUpdatePicture(vfuMB.m_firstGOB, vfuMB.m_firstMB, vfuMB.m_numberOfMBs));
-      }
+    case H245_MiscellaneousCommand_type::e_videoTemporalSpatialTradeOff :
+      mediaStream->ExecuteCommand(OpalTemporalSpatialTradeOff((const PASN_Integer &)type));
       break;
+
+    default:
+      break;
+  }
+#endif
+}
+
+
+void H323Channel::OnMediaCommand(OpalMediaCommand & command)
+{
+#if OPAL_VIDEO
+  if (PIsDescendant(&command, OpalVideoUpdatePicture)) {
+    H323ControlPDU pdu;
+    pdu.BuildMiscellaneousCommand(GetNumber(), H245_MiscellaneousCommand_type::e_videoFastUpdatePicture);
+    connection.WriteControlPDU(pdu);
+    return;
   }
 #endif
 }
@@ -397,86 +413,6 @@ void H323UnidirectionalChannel::InternalClose()
 }
 
 
-void H323UnidirectionalChannel::OnMiscellaneousCommand(const H245_MiscellaneousCommand_type & type)
-{
-  H323Channel::OnMiscellaneousCommand(type);
-
-  if (mediaStream == NULL)
-    return;
-
-#if OPAL_VIDEO
-  switch (type.GetTag())
-  {
-    case H245_MiscellaneousCommand_type::e_videoFreezePicture :
-      mediaStream->ExecuteCommand(OpalVideoFreezePicture());
-      break;
-
-    case H245_MiscellaneousCommand_type::e_videoFastUpdatePicture:
-      mediaStream->ExecuteCommand(OpalVideoUpdatePicture());
-      break;
-
-    case H245_MiscellaneousCommand_type::e_videoFastUpdateGOB :
-    {
-      const H245_MiscellaneousCommand_type_videoFastUpdateGOB & fuGOB = type;
-      mediaStream->ExecuteCommand(OpalVideoUpdatePicture(fuGOB.m_firstGOB, -1, fuGOB.m_numberOfGOBs));
-    }
-    break;
-
-    case H245_MiscellaneousCommand_type::e_videoFastUpdateMB :
-    {
-      const H245_MiscellaneousCommand_type_videoFastUpdateMB & vfuMB = type;
-      mediaStream->ExecuteCommand(OpalVideoUpdatePicture(vfuMB.HasOptionalField(H245_MiscellaneousCommand_type_videoFastUpdateMB::e_firstGOB) ? (int)vfuMB.m_firstGOB : -1,
-                                                         vfuMB.HasOptionalField(H245_MiscellaneousCommand_type_videoFastUpdateMB::e_firstMB)  ? (int)vfuMB.m_firstMB  : -1,
-                                                         vfuMB.m_numberOfMBs));
-    }
-    break;
-
-    case H245_MiscellaneousCommand_type::e_videoTemporalSpatialTradeOff :
-      mediaStream->ExecuteCommand(OpalTemporalSpatialTradeOff((const PASN_Integer &)type));
-      break;
-    default:
-      break;
-  }
-#endif
-}
-
-
-void H323UnidirectionalChannel::OnMediaCommand(OpalMediaCommand & command)
-{
-#if OPAL_VIDEO
-  if (PIsDescendant(&command, OpalVideoUpdatePicture)) {
-    H323ControlPDU pdu;
-    const OpalVideoUpdatePicture & updatePicture = (const OpalVideoUpdatePicture &)command;
-
-    if (updatePicture.GetNumBlocks() <= 0)
-      pdu.BuildMiscellaneousCommand(GetNumber(), H245_MiscellaneousCommand_type::e_videoFastUpdatePicture);
-    else if (updatePicture.GetFirstMB() < 0) {
-      H245_MiscellaneousCommand_type_videoFastUpdateGOB & fuGOB = 
-            pdu.BuildMiscellaneousCommand(GetNumber(), H245_MiscellaneousCommand_type::e_videoFastUpdateGOB).m_type;
-      fuGOB.m_firstGOB = updatePicture.GetFirstGOB();
-      fuGOB.m_numberOfGOBs = updatePicture.GetNumBlocks();
-    }
-    else {
-      H245_MiscellaneousCommand_type_videoFastUpdateMB & vfuMB =
-            pdu.BuildMiscellaneousCommand(GetNumber(), H245_MiscellaneousCommand_type::e_videoFastUpdateMB).m_type;
-      if (updatePicture.GetFirstGOB() >= 0) {
-        vfuMB.IncludeOptionalField(H245_MiscellaneousCommand_type_videoFastUpdateMB::e_firstGOB);
-        vfuMB.m_firstGOB = updatePicture.GetFirstGOB();
-      }
-      if (updatePicture.GetFirstMB() >= 0) {
-        vfuMB.IncludeOptionalField(H245_MiscellaneousCommand_type_videoFastUpdateMB::e_firstMB);
-        vfuMB.m_firstMB = updatePicture.GetFirstMB();
-      }
-      vfuMB.m_numberOfMBs = updatePicture.GetNumBlocks();
-    }
-
-    connection.WriteControlPDU(pdu);
-    return;
-  }
-#endif
-}
-
-
 OpalMediaStreamPtr H323UnidirectionalChannel::GetMediaStream() const
 {
   return mediaStream;
@@ -729,9 +665,6 @@ H323_RTPChannel::H323_RTPChannel(H323Connection & conn,
 
 H323_RTPChannel::~H323_RTPChannel()
 {
-  // Finished with the RTP session, this will delete the session if it is no
-  // longer referenced by any logical channels.
-  connection.ReleaseSession(GetSessionID());
 }
 
 
