@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24506 $
+ * $Revision: 25983 $
  * $Author: rjongbloed $
- * $Date: 2010-06-17 21:40:22 -0500 (Thu, 17 Jun 2010) $
+ * $Date: 2011-06-08 20:58:54 -0500 (Wed, 08 Jun 2011) $
  */
 
 #include <ptlib.h>
@@ -628,7 +628,7 @@ PBoolean H323GenericCapabilityInfo::OnReceivedGenericPDU(OpalMediaFormat & media
     if (PIsDescendant(&option, OpalMediaOptionBoolean)) {
       if (param->GetTag() == H245_ParameterValue::e_logical) {
         ((OpalMediaOptionBoolean &)option).SetValue(true);
-        break;
+        continue;
       }
     }
     else if (PIsDescendant(&option, OpalMediaOptionUnsigned)) {
@@ -650,7 +650,7 @@ PBoolean H323GenericCapabilityInfo::OnReceivedGenericPDU(OpalMediaFormat & media
 
       if (param->GetTag() == tag) {
         ((OpalMediaOptionUnsigned &)option).SetValue((const PASN_Integer &)*param);
-        break;
+        continue;
       }
     }
     else {
@@ -660,7 +660,7 @@ PBoolean H323GenericCapabilityInfo::OnReceivedGenericPDU(OpalMediaFormat & media
           ((OpalMediaOptionOctets &)option).SetValue(octetString);
         else
           ((OpalMediaOption &)option).FromString(octetString.AsString());
-        break;
+        continue;
       }
     }
 
@@ -2127,7 +2127,7 @@ H323Capabilities & H323Capabilities::operator=(const H323Capabilities & original
 
 void H323Capabilities::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision()-1;
+  std::streamsize indent = strm.precision()-1;
   strm << setw(indent) << " " << "Table:\n";
   for (PINDEX i = 0; i < table.GetSize(); i++)
     strm << setw(indent+2) << " " << table[i] << '\n';
