@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23831 $
- * $Author: rjongbloed $
- * $Date: 2009-12-03 04:21:07 -0600 (Thu, 03 Dec 2009) $
+ * $Revision: 25019 $
+ * $Author: willamowius $
+ * $Date: 2011-01-06 03:44:41 -0600 (Thu, 06 Jan 2011) $
  */
 
 #include <ptlib.h>
@@ -82,13 +82,24 @@ class SDPMSRPMediaDescription : public SDPMediaDescription
     SDPMSRPMediaDescription(const OpalTransportAddress & address, const PString & url);
 
     PCaselessString GetSDPTransportType() const
-    { return "tcp/msrp"; }
+    {
+      return "tcp/msrp";
+    }
+
+    virtual SDPMediaDescription * CreateEmpty() const
+    {
+      return new SDPMSRPMediaDescription(OpalTransportAddress());
+    }
 
     virtual PString GetSDPMediaType() const 
-    { return "message"; }
+    {
+      return "message";
+    }
 
     virtual PString GetSDPPortList() const
-    { return " *"; }
+    {
+      return " *";
+    }
 
     virtual void CreateSDPMediaFormats(const PStringArray &);
     virtual bool PrintOn(ostream & str, const PString & connectString) const;
@@ -426,8 +437,9 @@ void OpalMSRPMediaStream::OnReceiveMSRP(OpalMSRPManager &, OpalMSRPManager::Inco
     T140String t140(incomingMSRP.m_body);
     RTP_DataFrameList frames = m_rfc4103Context.ConvertToFrames(incomingMSRP.m_mime.GetString(PHTTP::ContentTypeTag, "text/plain"), t140);
     OpalMediaFormat fmt(m_rfc4103Context.m_mediaFormat);
-    for (PINDEX i = 0; i < frames.GetSize(); ++i)
-      connection.OnReceiveExternalIM(m_rfc4103Context.m_mediaFormat, (RTP_IMFrame &)frames[i]);
+    for (PINDEX i = 0; i < frames.GetSize(); ++i) {
+      //connection.OnReceiveExternalIM(m_rfc4103Context.m_mediaFormat, (RTP_IMFrame &)frames[i]);
+    }
   }
   else {
     PTRACE(3, "MSRP\tMediaStream " << *this << " receiving unknown MSRP message");
@@ -461,7 +473,8 @@ OpalMSRPManager::~OpalMSRPManager()
 
 std::string OpalMSRPManager::CreateSessionID()
 {
-  return PGloballyUniqueID().AsString();
+  PString str = PGloballyUniqueID().AsString();
+  return (const char *)str;
 }
 
 
@@ -509,7 +522,7 @@ void OpalMSRPManager::ListenerThread()
     {
       PWaitAndSignal m(m_connectionInfoMapAddMutex);
       connection.SetSafetyMode(PSafeReference);
-      m_connectionInfoMap.insert(ConnectionInfoMapType::value_type(remoteAddr.AsString(), connection));
+      m_connectionInfoMap.insert(ConnectionInfoMapType::value_type((const char *)remoteAddr.AsString(), connection));
       connection.SetSafetyMode(PSafeReadWrite);
     }
     connection->StartHandler();
@@ -539,10 +552,10 @@ PSafePtr<OpalMSRPManager::Connection> OpalMSRPManager::OpenConnection(const PURL
       } 
       else if (!PIPSocket::GetHostAddress(remoteURL.GetHostName(), ip)) {
         PTRACE(2, "MSRP\tUnable to resolve MSRP URL hostname '" << remoteURL << "' ");
-        return false;
+        return NULL;
       }
 #else
-      return false;
+      return NULL;
 #endif
     }
   }
