@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23978 $
+ * $Revision: 25893 $
  * $Author: rjongbloed $
- * $Date: 2010-01-24 19:28:06 -0600 (Sun, 24 Jan 2010) $
+ * $Date: 2011-05-25 20:31:43 -0500 (Wed, 25 May 2011) $
  */
 
 #include <ptlib.h>
@@ -61,6 +61,8 @@ OpalIVREndPoint::OpalIVREndPoint(OpalManager & mgr, const char * prefix)
                 "</vxml>\n")
 {
   defaultMediaFormats += OpalPCM16;
+  defaultMediaFormats += OpalPCM16_48KHZ;
+  defaultMediaFormats += OpalPCM16S_48KHZ;
   defaultMediaFormats += OpalRFC2833;
 #if OPAL_T38_CAPABILITY
   defaultMediaFormats += OpalCiscoNSE;
@@ -120,6 +122,12 @@ void OpalIVREndPoint::SetDefaultVXML(const PString & vxml)
 {
   inUseFlag.Wait();
   defaultVXML = vxml;
+
+  if (vxml.Find("<--G.722.1-->") != P_MAX_INDEX)
+    defaultMediaFormats += OPAL_G7231;
+  if (vxml.Find("<--G.729-->") != P_MAX_INDEX)
+    defaultMediaFormats += OPAL_G729;
+
   inUseFlag.Signal();
 }
 
@@ -131,12 +139,15 @@ void OpalIVREndPoint::SetDefaultMediaFormats(const OpalMediaFormatList & formats
   inUseFlag.Signal();
 }
 
-PBoolean OpalIVREndPoint::StartVXML()
+
+void OpalIVREndPoint::OnEndDialog(OpalIVRConnection & connection)
 {
-  return PFalse;
+  PTRACE(3, "IVR\tOnEndDialog for connection " << connection);
+  connection.Release();
 }
 
 
+
 /////////////////////////////////////////////////////////////////////////////
 
 OpalIVRConnection::OpalIVRConnection(OpalCall & call,
@@ -147,13 +158,13 @@ OpalIVRConnection::OpalIVRConnection(OpalCall & call,
                                      OpalConnection::StringOptions * stringOptions)
   : OpalLocalConnection(call, ep, userData, options, stringOptions, 'I'),
     endpoint(ep),
-    vxmlToLoad(vxml),
-    vxmlMediaFormats(ep.GetMediaFormats()),
+    m_vxmlScript(vxml),
+    m_vxmlMediaFormats(ep.GetMediaFormats()),
 #ifdef _MSC_VER
 #pragma warning(push)
 #pragma warning(disable:4355)
 #endif
-    vxmlSession(this, PFactory<PTextToSpeech>::CreateInstance(ep.GetDefaultTextToSpeech()), true)
+    m_vxmlSession(*this, PFactory<PTextToSpeech>::CreateInstance(ep.GetDefaultTextToSpeech()), true)
 #ifdef _MSC_VER
 #pragma warning(pop)
 #endif
@@ -170,67 +181,93 @@ OpalIVRConnection::~OpalIVRConnection()
 
 PString OpalIVRConnection::GetLocalPartyURL() const
 {
-  return GetPrefixName() + ':' + vxmlToLoad.Left(vxmlToLoad.FindOneOf("\r\n"));
+  return GetPrefixName() + ':' + m_vxmlScript.Left(m_vxmlScript.FindOneOf("\r\n"));
 }
 
 
 void OpalIVRConnection::OnEstablished()
 {
   OpalConnection::OnEstablished();
-  StartVXML();
+
+  if (!m_vxmlSession.IsLoaded())
+    StartVXML(m_vxmlScript);
 }
 
 
-PBoolean OpalIVRConnection::StartVXML()
+bool OpalIVRConnection::TransferConnection(const PString & remoteParty)
 {
-  if (vxmlSession.IsPlaying()) {
-    PTRACE(4, "IVR\tStartVXML, already playing");
-    return true;
-  }
+  // First strip off the prefix if present
+  PINDEX prefixLength = 0;
+  if (remoteParty.Find(GetPrefixName()+":") == 0)
+    prefixLength = GetPrefixName().GetLength()+1;
 
-  PStringToString & vars = vxmlSession.GetSessionVars();
+  return StartVXML(remoteParty.Mid(prefixLength));
+}
 
-  PString originator = m_connStringOptions(OPAL_OPT_ORIGINATOR_ADDRESS);
-  if (originator.IsEmpty())
-    originator = m_connStringOptions("Remote-Address");
-  if (!originator.IsEmpty()) {
-    PIPSocketAddressAndPort ap(originator);
-    vars.SetAt("Source-IP-Address", ap.GetAddress().AsString());
-  }
 
-  vars.SetAt("Time", PTime().AsString());
+PBoolean OpalIVRConnection::StartVXML(const PString & vxml)
+{
+  PSafeLockReadWrite mutex(*this);
+  if (!mutex.IsLocked())
+    return false;
 
-  if (vxmlToLoad.IsEmpty()) 
+  PString vxmlToLoad = vxml;
+
+  if (vxmlToLoad.IsEmpty() || vxmlToLoad == "*") {
     vxmlToLoad = endpoint.GetDefaultVXML();
+    if (vxmlToLoad.IsEmpty())
+      return false;
+  }
 
-  if (vxmlToLoad.IsEmpty())
-    return endpoint.StartVXML();
+  PURL remoteURL = GetRemotePartyURL();
+  m_vxmlSession.SetVar("session.connection.local.uri", GetLocalPartyURL());
+  m_vxmlSession.SetVar("session.connection.remote.ani", GetRemotePartyNumber());
+  m_vxmlSession.SetVar("session.connection.remote.uri", remoteURL);
+  m_vxmlSession.SetVar("session.connection.remote.ip", remoteURL.GetHostName());
+  m_vxmlSession.SetVar("session.connection.remote.port", remoteURL.GetPort());
+  m_vxmlSession.SetVar("session.time", PTime().AsString());
+
+  bool ok;
 
   PCaselessString vxmlHead = vxmlToLoad.LeftTrim().Left(5);
   if (vxmlHead == "<?xml" || vxmlHead == "<vxml") {
-    PTRACE(4, "IVR\tStartVXML:\n" << vxmlToLoad);
-    return vxmlSession.LoadVXML(vxmlToLoad);
-  }
-
-  PURL vxmlURL(vxmlToLoad, NULL);
-  if (vxmlURL.IsEmpty()) {
-    PFilePath vxmlFile = vxmlToLoad;
-    if (vxmlFile.GetType() *= ".vxml")
-      return vxmlSession.LoadFile(vxmlFile);
+    PTRACE(4, "IVR\tStarted using raw VXML:\n" << vxmlToLoad);
+    ok = m_vxmlSession.LoadVXML(vxmlToLoad);
   }
   else {
-    if (vxmlURL.GetScheme() != "file" || (vxmlURL.AsFilePath().GetType() *= ".vxml"))
-      return vxmlSession.LoadURL(vxmlURL);
+    PURL vxmlURL(vxmlToLoad, NULL);
+    if (vxmlURL.IsEmpty()) {
+      PFilePath vxmlFile = vxmlToLoad;
+      if (vxmlFile.GetType() != ".vxml")
+        ok = StartScript(vxmlToLoad);
+      else {
+        PTRACE(4, "IVR\tStarted using VXML file: " << vxmlFile);
+        ok = m_vxmlSession.LoadFile(vxmlFile);
+      }
+    }
+    else if (vxmlURL.GetScheme() == "file" && (vxmlURL.AsFilePath().GetType() *= ".vxml"))
+      ok = m_vxmlSession.LoadURL(vxmlURL);
+    else
+      ok = StartScript(vxmlToLoad);
   }
 
-  ////////////////////////////////
+  if (ok)
+    m_vxmlScript = vxmlToLoad;
 
+  return ok;
+}
+
+
+bool OpalIVRConnection::StartScript(const PString & script)
+{
   PINDEX repeat = 1;
   PINDEX delay = 0;
   PString voice;
 
+  PTRACE(4, "IVR\tStarted using simplified script: " << script);
+
   PINDEX i;
-  PStringArray tokens = vxmlToLoad.Tokenise(';', PFalse);
+  PStringArray tokens = script.Tokenise(';', false);
   for (i = 0; i < tokens.GetSize(); ++i) {
     PString str(tokens[i]);
 
@@ -251,7 +288,7 @@ PBoolean OpalIVRConnection::StartVXML()
         fn = fn.GetDirectory() + voice + PDIR_SEPARATOR + fn.GetFileName();
 
       PTRACE(3, "IVR\tPlaying file " << fn << ' ' << repeat << " times, " << delay << "ms");
-      vxmlSession.PlayFile(fn, repeat, delay);
+      m_vxmlSession.PlayFile(fn, repeat, delay);
       continue;
     }
 
@@ -274,7 +311,7 @@ PBoolean OpalIVRConnection::StartVXML()
     else if (key *= "voice") {
       if (!val.IsEmpty()) {
         voice = val;
-        PTextToSpeech * tts = vxmlSession.GetTextToSpeech();
+        PTextToSpeech * tts = m_vxmlSession.GetTextToSpeech();
         if (tts != NULL)
           tts->SetVoice(voice);
       }
@@ -282,37 +319,47 @@ PBoolean OpalIVRConnection::StartVXML()
 
     else if (key *= "tone") {
       PTRACE(3, "IVR\tPlaying tone " << val);
-      vxmlSession.PlayTone(val, repeat, delay);
+      m_vxmlSession.PlayTone(val, repeat, delay);
    }
 
     else if (key *= "speak") {
       if (!val.IsEmpty() && (val[0] == '$'))
-        val = vars(val.Mid(1));
+        val = m_vxmlSession.GetVar(val.Mid(1));
       PTRACE(3, "IVR\tSpeaking text '" << val << "'");
-      vxmlSession.PlayText(val, PTextToSpeech::Default, repeat, delay);
+      m_vxmlSession.PlayText(val, PTextToSpeech::Default, repeat, delay);
     }
 
     else if (key *= "silence") {
       unsigned msecs;
       if (val.IsEmpty() && (val[0] == '$'))
-        msecs = vars(val.Mid(1)).AsUnsigned();
+        msecs = m_vxmlSession.GetVar(val.Mid(1)).AsUnsigned();
       else
         msecs = val.AsUnsigned();
       PTRACE(3, "IVR\tSpeaking silence for " << msecs << " msecs");
-      vxmlSession.PlaySilence(msecs);
+      m_vxmlSession.PlaySilence(msecs);
+    }
+    else {
+      PTRACE(2, "IVR\tInvalid command in \"" << script << '"');
+      return false;
     }
   }
 
-  vxmlSession.PlayStop();
-  vxmlSession.Trigger();
+  m_vxmlSession.PlayStop();
+  m_vxmlSession.Trigger();
 
-  return PTrue;
+  return true;
+}
+
+
+void OpalIVRConnection::OnEndDialog()
+{
+  endpoint.OnEndDialog(*this);
 }
 
 
 OpalMediaFormatList OpalIVRConnection::GetMediaFormats() const
 {
-  return vxmlMediaFormats;
+  return m_vxmlMediaFormats;
 }
 
 
@@ -320,7 +367,9 @@ OpalMediaStream * OpalIVRConnection::CreateMediaStream(const OpalMediaFormat & m
                                                        unsigned sessionID,
                                                        PBoolean isSource)
 {
-  return new OpalIVRMediaStream(*this, mediaFormat, sessionID, isSource, vxmlSession);
+  return mediaFormat.GetMediaType() != OpalMediaType::Audio()
+            ? OpalConnection::CreateMediaStream(mediaFormat, sessionID, isSource)
+            : new OpalIVRMediaStream(*this, mediaFormat, sessionID, isSource, m_vxmlSession);
 }
 
 
@@ -329,21 +378,11 @@ PBoolean OpalIVRConnection::SendUserInputString(const PString & value)
   PTRACE(3, "IVR\tSendUserInputString(" << value << ')');
 
   for (PINDEX i = 0; i < value.GetLength(); i++)
-    vxmlSession.OnUserInput(value[i]);
+    m_vxmlSession.OnUserInput(value[i]);
 
-  return PTrue;
+  return true;
 }
 
-void OpalIVRConnection::OnStopMediaPatch(OpalMediaPatch & patch)
-{
-  // lose the audio, then lose the call
-  if (patch.GetSource().GetMediaFormat().GetMediaType() == OpalMediaType::Audio()) {
-    synchronousOnRelease = false; // Deadlocks if try to do it synchronously ...
-    Release();
-  }
-}
-
-
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -363,7 +402,7 @@ PBoolean OpalIVRMediaStream::Open()
 {
   PTRACE(3, "IVR\tOpen");
   if (isOpen)
-    return PTrue;
+    return true;
 
   if (vxmlSession.IsOpen()) {
     PVXMLChannel * vxmlChannel = vxmlSession.GetAndLockVXMLChannel();
@@ -371,7 +410,7 @@ PBoolean OpalIVRMediaStream::Open()
     
     if (vxmlChannel == NULL) {
       PTRACE(1, "IVR\tVXML engine not really open");
-      return PFalse;
+      return false;
     }
 
     vxmlChannelMediaFormat = vxmlChannel->GetMediaFormat();
@@ -379,7 +418,7 @@ PBoolean OpalIVRMediaStream::Open()
     
     if (mediaFormat.GetName() != vxmlChannelMediaFormat) {
       PTRACE(1, "IVR\tCannot use VXML engine: asymmetrical media formats: " << mediaFormat << " <-> " << vxmlChannelMediaFormat);
-      return PFalse;
+      return false;
     }
 
     return OpalMediaStream::Open();
@@ -389,9 +428,20 @@ PBoolean OpalIVRMediaStream::Open()
     return OpalMediaStream::Open();
 
   PTRACE(1, "IVR\tCannot open VXML engine: incompatible media format");
-  return PFalse;
+  return false;
 }
 
+
+PBoolean OpalIVRMediaStream::Close()
+{
+  if (connection.IsReleased())
+    return OpalRawMediaStream::Close();
+
+  // Pause the vxmlSession here ...
+  return OpalMediaStream::Close();
+}
+
+
 PBoolean OpalIVRMediaStream::IsSynchronous() const
 {
   return true;
