@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24917 $
+ * $Revision: 25928 $
  * $Author: rjongbloed $
- * $Date: 2010-12-02 20:26:58 -0600 (Thu, 02 Dec 2010) $
+ * $Date: 2011-05-30 22:13:33 -0500 (Mon, 30 May 2011) $
  */
 
 #include <ptlib.h>
@@ -45,6 +45,7 @@
 #include <codec/g711codec.h>
 #include <codec/vidcodec.h>
 #include <codec/rfc4175.h>
+#include <codec/rfc2435.h>
 #include <codec/opalpluginmgr.h>
 
 #if OPAL_HAS_H224
@@ -55,6 +56,7 @@
 #include <ptclib/url.h>
 
 #include "../../version.h"
+#include "../../revision.h"
 
 
 static const char * const DefaultMediaFormatOrder[] = {
@@ -86,6 +88,11 @@ OPAL_REGISTER_G711();
 OPAL_REGISTER_RFC4175();
 #endif
 
+// Same deal for RC2435 video
+#if OPAL_RFC2435
+OPAL_REGISTER_RFC2435_JPEG();
+#endif
+
 
 #define new PNEW
 
@@ -98,7 +105,7 @@ PString OpalGetVersion()
 #define BetaCode    "beta"
 #define ReleaseCode "."
 
-  return psprintf("%u.%u%s%u", MAJOR_VERSION, MINOR_VERSION, BUILD_TYPE, BUILD_NUMBER);
+  return psprintf("%u.%u%s%u (svn:%u)", MAJOR_VERSION, MINOR_VERSION, BUILD_TYPE, BUILD_NUMBER, SVN_REVISION);
 }
 
 
@@ -191,7 +198,7 @@ OpalManager::OpalManager()
   : productInfo(OpalProductInfo::Default())
   , defaultUserName(PProcess::Current().GetUserName())
   , defaultDisplayName(defaultUserName)
-  , rtpIpTypeofService(0xb8)  // New DiffServ value for Expidited Forwarding as per RFC3246
+  , m_defaultMediaTypeOfService(0xb8)  // New DiffServ value for Expidited Forwarding as per RFC3246
   , rtpPayloadSizeMax(576-20-16-12) // Max safe MTU size (576 bytes as per RFC879) minus IP, UDP an RTP headers
   , rtpPacketSizeMax(2048)
   , minAudioJitterDelay(50)  // milliseconds
@@ -206,6 +213,9 @@ OpalManager::OpalManager()
 #ifdef OPAL_ZRTP
   , zrtpEnabled(false)
 #endif
+#ifdef OPAL_HAS_IM
+  , m_imManager(NULL)
+#endif
 {
   rtpIpPorts.current = rtpIpPorts.base = 5000;
   rtpIpPorts.max = 5999;
@@ -236,6 +246,8 @@ OpalManager::OpalManager()
   SetAutoStartReceiveVideo(!videoOutputDevice.deviceName.IsEmpty());
 #endif
 
+  m_imManager = new OpalIMManager(*this);
+
   garbageCollector = PThread::Create(PCREATE_NOTIFIER(GarbageMain), "Opal Garbage");
 
   PTRACE(4, "OpalMan\tCreated manager.");
@@ -261,6 +273,7 @@ OpalManager::~OpalManager()
 
   delete stun;
   delete interfaceMonitor;
+  delete m_imManager;
 
   PTRACE(4, "OpalMan\tDeleted manager.");
 }
@@ -516,7 +529,7 @@ void OpalManager::InternalClearAllCalls(OpalConnection::CallEndReason reason, bo
        released from the PSyncPoint wait. */
     m_clearingAllCallsMutex.Wait();
     if (firstThread)
-      m_allCallsCleared.Wait();
+      PAssert(m_allCallsCleared.Wait(120000), "All calls not cleared in a timely manner");
     m_clearingAllCallsMutex.Signal();
   }
 
@@ -728,6 +741,13 @@ PBoolean OpalManager::OnForwarded(OpalConnection & PTRACE_PARAM(connection),
 }
 
 
+bool OpalManager::OnTransferNotify(OpalConnection & PTRACE_PARAM(connection), const PStringToString & info)
+{
+  PTRACE(4, "OpalManager\tOnTransferNotify for " << connection << '\n' << info);
+  return info["result"] != "success";
+}
+
+
 OpalMediaFormatList OpalManager::GetCommonMediaFormats(bool transportable, bool pcmAudio) const
 {
   OpalMediaFormatList formats;
@@ -743,6 +763,9 @@ OpalMediaFormatList OpalManager::GetCommonMediaFormats(bool transportable, bool
   if (pcmAudio) {
     // Sound cards can only do 16 bit PCM, but at various sample rates
     // The following will be in order of preference, so lets do wideband first
+    formats += OpalPCM16S_48KHZ;
+    formats += OpalPCM16S_32KHZ;
+    formats += OpalPCM16S_16KHZ;
     formats += OpalPCM16_48KHZ;
     formats += OpalPCM16_32KHZ;
     formats += OpalPCM16_16KHZ;
@@ -780,12 +803,13 @@ OpalMediaFormatList OpalManager::GetCommonMediaFormats(bool transportable, bool
 
 
 void OpalManager::AdjustMediaFormats(bool local,
-                                     const OpalConnection & /*connection*/,
+                                     const OpalConnection & connection,
                                      OpalMediaFormatList & mediaFormats) const
 {
   mediaFormats.Remove(mediaFormatMask);
   if (local)
     mediaFormats.Reorder(mediaFormatOrder);
+  connection.GetCall().AdjustMediaFormats(local, connection, mediaFormats);
 }
 
 
@@ -830,19 +854,34 @@ bool OpalManager::OnLocalRTP(OpalConnection & PTRACE_PARAM(connection1),
 }
 
 
-static bool BypassOne(OpalMediaStreamPtr source,
-                      OpalMediaStreamPtr sink,
-                      bool bypass)
+static bool PassOneThrough(OpalMediaStreamPtr source,
+                           OpalMediaStreamPtr sink,
+                           bool bypass)
 {
-  OpalMediaPatch * sourcePatch = source != NULL ? source->GetPatch() : NULL;
-  OpalMediaPatch * sinkPatch = sink != NULL ? sink->GetPatch() : NULL;
-  if (sourcePatch == NULL || sinkPatch == NULL) {
-    PTRACE(2, "OpalMan\tSetMediaBypass could not complete as one patch does not exist");
+  if (source == NULL) {
+    PTRACE(2, "OpalMan\tSetMediaPassThrough could not complete as source stream does not exist");
+    return false;
+  }
+
+  if (sink == NULL) {
+    PTRACE(2, "OpalMan\tSetMediaPassThrough could not complete as sink stream does not exist");
+    return false;
+  }
+
+  OpalMediaPatch * sourcePatch = source->GetPatch();
+  if (sourcePatch == NULL) {
+    PTRACE(2, "OpalMan\tSetMediaPassThrough could not complete as source patch does not exist");
+    return false;
+  }
+
+  OpalMediaPatch * sinkPatch = sink->GetPatch();
+  if (sinkPatch == NULL) {
+    PTRACE(2, "OpalMan\tSetMediaPassThrough could not complete as sink patch does not exist");
     return false;
   }
 
   if (source->GetMediaFormat() != sink->GetMediaFormat()) {
-    PTRACE(3, "OpalMan\tSetMediaBypass could not complete as different formats: "
+    PTRACE(3, "OpalMan\tSetMediaPassThrough could not complete as different formats: "
            << source->GetMediaFormat() << "!=" << sink->GetMediaFormat());
     return false;
   }
@@ -852,28 +891,28 @@ static bool BypassOne(OpalMediaStreamPtr source,
 }
 
 
-bool OpalManager::SetMediaBypass(OpalConnection & connection1,
-                                 OpalConnection & connection2,
-                                 bool bypass,
-                                 unsigned sessionID)
+bool OpalManager::SetMediaPassThrough(OpalConnection & connection1,
+                                      OpalConnection & connection2,
+                                      bool bypass,
+                                      unsigned sessionID)
 {
   bool gotOne = false;
 
   if (sessionID != 0) {
     // Do not use || as McCarthy will not execute the second bypass
-    if (BypassOne(connection1.GetMediaStream(sessionID, true), connection2.GetMediaStream(sessionID, false), bypass))
+    if (PassOneThrough(connection1.GetMediaStream(sessionID, true), connection2.GetMediaStream(sessionID, false), bypass))
       gotOne = true;
-    if (BypassOne(connection2.GetMediaStream(sessionID, true), connection1.GetMediaStream(sessionID, false), bypass))
+    if (PassOneThrough(connection2.GetMediaStream(sessionID, true), connection1.GetMediaStream(sessionID, false), bypass))
       gotOne = true;
   }
   else {
     OpalMediaStreamPtr stream;
     while ((stream = connection1.GetMediaStream(OpalMediaType(), true , stream)) != NULL) {
-      if (BypassOne(stream, connection2.GetMediaStream(stream->GetSessionID(), false), bypass))
+      if (PassOneThrough(stream, connection2.GetMediaStream(stream->GetSessionID(), false), bypass))
         gotOne = true;
     }
     while ((stream = connection2.GetMediaStream(OpalMediaType(), true, stream)) != NULL) {
-      if (BypassOne(stream, connection1.GetMediaStream(stream->GetSessionID(), false), bypass))
+      if (PassOneThrough(stream, connection1.GetMediaStream(stream->GetSessionID(), false), bypass))
         gotOne = true;
     }
   }
@@ -882,17 +921,17 @@ bool OpalManager::SetMediaBypass(OpalConnection & connection1,
 }
 
 
-bool OpalManager::SetMediaBypass(const PString & token1,
-                                 const PString & token2,
-                                 bool bypass,
-                                 unsigned sessionID,
-                                 bool network)
+bool OpalManager::SetMediaPassThrough(const PString & token1,
+                                      const PString & token2,
+                                      bool bypass,
+                                      unsigned sessionID,
+                                      bool network)
 {
   PSafePtr<OpalCall> call1 = FindCallWithLock(token1);
   PSafePtr<OpalCall> call2 = FindCallWithLock(token2);
 
   if (call1 == NULL || call2 == NULL) {
-    PTRACE(2, "OpalMan\tSetMediaBypass could not complete as one call does not exist");
+    PTRACE(2, "OpalMan\tSetMediaPassThrough could not complete as one call does not exist");
     return false;
   }
 
@@ -905,11 +944,11 @@ bool OpalManager::SetMediaBypass(const PString & token1,
     ++connection2;
 
   if (connection1 == NULL || connection2 == NULL) {
-    PTRACE(2, "OpalMan\tSetMediaBypass could not complete as network connection not present in calls");
+    PTRACE(2, "OpalMan\tSetMediaPassThrough could not complete as network connection not present in calls");
     return false;
   }
 
-  return OpalManager::SetMediaBypass(*connection1, *connection2, sessionID, bypass);
+  return OpalManager::SetMediaPassThrough(*connection1, *connection2, sessionID, bypass);
 }
 
 
@@ -1217,15 +1256,16 @@ PString OpalManager::ApplyRouteTable(const PString & a_party, const PString & b_
   PINDEX colon = b_party.Find(':');
   if (colon == P_MAX_INDEX)
     colon = 0;
-  else if (b_party.NumCompare("tel", colon) == EqualTo) // Small cheat for tel: URI (RFC3966)
-    colon++;
-  else if (FindEndPoint(b_party.Left(colon)) == NULL)
-    colon = 0;
-  else {
+  else if (FindEndPoint(b_party.Left(colon)) != NULL) {
+    // Hack to make some modes work
     if (destination.Find("<da>") != P_MAX_INDEX)
       return b_party;
     colon++;
   }
+  else if (b_party.NumCompare("tel", colon) == EqualTo) // Small cheat for tel: URI (RFC3966)
+    colon++;
+  else
+    colon = 0;
 
   PINDEX nonDigitPos = b_party.FindSpan("0123456789*#-.()", colon + (b_party[colon] == '+'));
   PString digits = b_party(colon, nonDigitPos-1);
@@ -1611,6 +1651,19 @@ WORD OpalManager::GetRtpIpPortPair()
 }
 
 
+BYTE OpalManager::GetMediaTypeOfService(const OpalMediaType & type) const
+{
+  map<OpalMediaType, BYTE>::const_iterator it = m_mediaTypeOfService.find(type);
+  return it != m_mediaTypeOfService.end() ? it->second : m_defaultMediaTypeOfService;
+}
+
+
+void OpalManager::SetMediaTypeOfService(const OpalMediaType & type, unsigned tos)
+{
+  m_mediaTypeOfService[type] = (BYTE)tos;
+}
+
+
 void OpalManager::SetAudioJitterDelay(unsigned minDelay, unsigned maxDelay)
 {
   if (minDelay == 0) {
@@ -1706,7 +1759,10 @@ PBoolean OpalManager::SetNoMediaTimeout(const PTimeInterval & newInterval)
 
 void OpalManager::GarbageCollection()
 {
-  PBoolean allCleared = activeCalls.DeleteObjectsToBeRemoved();
+  m_presentities.DeleteObjectsToBeRemoved();
+  m_imManager->GarbageCollection();
+
+  bool allCleared = activeCalls.DeleteObjectsToBeRemoved();
 
   endpointsMutex.StartRead();
 
@@ -1772,18 +1828,25 @@ bool OpalManager::StopRecording(const PString & callToken)
 #endif
 
 
-void OpalManager::OnApplyStringOptions(OpalConnection & conn, OpalConnection::StringOptions & stringOptions)
+void OpalManager::OnApplyStringOptions(OpalConnection &, OpalConnection::StringOptions &)
 {
-  conn.ApplyStringOptions(stringOptions);
 }
 
 
 PSafePtr<OpalPresentity> OpalManager::AddPresentity(const PString & presentity)
 {
+  if (presentity.IsEmpty())
+    return NULL;
+
+  PSafePtr<OpalPresentity> oldPresentity = m_presentities.FindWithLock(presentity, PSafeReadWrite);
+  if (oldPresentity != NULL)
+    return oldPresentity;
+
   OpalPresentity * newPresentity = OpalPresentity::Create(*this, presentity);
   if (newPresentity == NULL)
     return NULL;
 
+  PTRACE(4, "OpalMan\tAdded presentity for " << *newPresentity);
   m_presentities.SetAt(presentity, newPresentity);
   return PSafePtr<OpalPresentity>(newPresentity, PSafeReadWrite);
 }
@@ -1808,78 +1871,63 @@ PStringList OpalManager::GetPresentities() const
 
 bool OpalManager::RemovePresentity(const PString & presentity)
 {
+  PTRACE(4, "OpalMan\tRemoving presentity for " << presentity);
   return m_presentities.RemoveAt(presentity);
 }
 
 
 PBoolean OpalManager::Message(const PString & to, const PString & body)
 {
-  PURL url(to);
-
-  for (PList<OpalEndPoint>::iterator it = endpointList.begin(); it != endpointList.end(); ++it) {
-    if (url.GetScheme() *= it->GetPrefixName())
-      return it->Message(to, body);
-  }
-
-  return false;
+  OpalIM message;
+  message.m_to   = to;
+  message.m_body = body;
+  return Message(message);
 }
 
 
 PBoolean OpalManager::Message(const PURL & to, const PString & type, const PString & body, PURL & from, PString & conversationId)
 {
-  for (PList<OpalEndPoint>::iterator it = endpointList.begin(); it != endpointList.end(); ++it) {
-    if (to.GetScheme() *= it->GetPrefixName())
-      return it->Message(to, type, body, from, conversationId);
-  }
+  OpalIM message;
+  message.m_to             = to;
+  message.m_mimeType       = type;
+  message.m_body           = body;
+  message.m_from           = from;
+  message.m_conversationId = conversationId;
 
-  return false;
+  bool stat = Message(message);
+
+  from           = message.m_from;
+  conversationId = message.m_conversationId;
+
+  return stat;
 }
 
-void OpalManager::OnMessageReceived(
-  const PURL & /*from*/, 
-  const PString & /*fromName*/,
-  const PURL & /*to*/, 
-  const PString & /*type*/,
-  const PString & /*body*/,
-  const PString & /*conversationId*/
-)
+
+bool OpalManager::Message(OpalIM & message)
 {
+  PSafePtr<OpalIMContext> context = m_imManager->FindContextForMessageWithLock(message);
+  if (context == NULL)
+    context = OpalIMContext::Create(*this, message.m_from, message.m_to);
+  if (context == NULL)
+    return false;
+
+  OpalIMContext::SentStatus stat = context->Send(new OpalIM(message));
+
+  return (stat == OpalIMContext::SentOK) || (stat == OpalIMContext::SentPending);
 }
 
-#if OPAL_HAS_IM
 
-bool OpalManager::TransmitExternalIM(OpalConnection & conn, const OpalMediaFormat & /*format*/, RTP_IMFrame & frame)
+void OpalManager::OnMessageReceived(const OpalIM & message)
 {
-  PURL remotePartyURL, localPartyURL;
-  PString remotePartyName;
-
-  // get information about sender
-  PSafePtr<OpalConnection> otherParty = conn.GetOtherPartyConnectionAs<OpalConnection>();
-  if (otherParty != NULL) {
-    remotePartyURL  = otherParty->GetRemotePartyCallbackURL();
-    remotePartyName = otherParty->GetRemotePartyName();
-    localPartyURL   = otherParty->GetLocalPartyURL();
+  // find a presentity to give the message to
+  for (PSafePtr<OpalPresentity> presentity(m_presentities, PSafeReference); presentity != NULL; ++presentity) {
+    if (message.m_to == presentity->GetAOR()) {
+      presentity->OnReceivedMessage(message);
+      break;
+    }
   }
-
-  // get ID
-  PString id = conn.GetToken(); //format.GetOptionString("Path", GetIdentifier());
-
-  // pass information up the chain
-  T140String t140;
-  PString str;
-  frame.GetContent(t140);
-  t140.AsString(str);
-  OnMessageReceived(remotePartyURL, 
-                    remotePartyName, 
-                    localPartyURL, 
-                    frame.GetContentType(),
-                    str,
-                    id);
-
-  return true;
 }
 
-#endif
 
 /////////////////////////////////////////////////////////////////////////////
 
