@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22989 $
+ * $Revision: 24951 $
  * $Author: rjongbloed $
- * $Date: 2009-06-25 19:04:24 -0500 (Thu, 25 Jun 2009) $
+ * $Date: 2010-12-17 01:47:00 -0600 (Fri, 17 Dec 2010) $
  */
 
 #ifdef P_USE_PRAGMA
@@ -36,7 +36,7 @@
 
 #include <opal/buildopts.h>
 
-#include <opal/opalvxml.h>
+#include <opal/ivr.h>
 
 #include <opal/call.h>
 #include <codec/opalwavfile.h>
@@ -46,51 +46,41 @@
 
 #if OPAL_PTLIB_VXML
 
-OpalVXMLSession::OpalVXMLSession(OpalConnection * _conn, PTextToSpeech * tts, PBoolean autoDelete)
+OpalVXMLSession::OpalVXMLSession(OpalIVRConnection & conn, PTextToSpeech * tts, PBoolean autoDelete)
   : PVXMLSession(tts, autoDelete),
-    conn(_conn)
+    m_connection(conn)
 {
-  if (tts == NULL) {
-    PFactory<PTextToSpeech>::KeyList_T engines = PFactory<PTextToSpeech>::GetKeyList();
-    if (engines.size() != 0) {
-      PFactory<PTextToSpeech>::Key_T name;
-#ifdef _WIN32
-      name = "Microsoft SAPI";
-      if (std::find(engines.begin(), engines.end(), name) == engines.end())
-#endif
-        name = engines[0];
-      SetTextToSpeech(name);
-    }
-  }
+  if (tts == NULL)
+    SetTextToSpeech(PString::Empty());
 }
 
 
-PBoolean OpalVXMLSession::Close()
+void OpalVXMLSession::OnEndDialog()
 {
-  if (!IsOpen())
-    return PTrue;
-
-  PBoolean ok = PVXMLSession::Close();
-  conn->Release();
-  return ok;
+  m_connection.OnEndDialog();
+  PVXMLSession::OnEndDialog();
 }
 
 
 void OpalVXMLSession::OnEndSession()
 {
-  conn->Release();
+  PTRACE(3, "IVR\tEnd of session, releasing connection.");
+  m_connection.Release();
 }
 
 
 void OpalVXMLSession::OnTransfer(const PString & destination, bool bridged)
 {
-  conn->GetCall().Transfer(destination, bridged ? &*conn->GetOtherPartyConnection() : &*conn);
-}
-
-
-PWAVFile * OpalVXMLSession::CreateWAVFile(const PFilePath & fn, PFile::OpenMode mode, int opts, unsigned fmt)
-{ 
-  return new OpalWAVFile(fn, mode, opts, fmt); 
+  if (!bridged)
+    m_connection.GetCall().Transfer(destination, &m_connection);
+  else {
+    PSafePtr<OpalConnection> otherConnection = m_connection.GetOtherPartyConnection();
+    if (otherConnection != NULL)
+      m_connection.GetCall().Transfer(destination, &*otherConnection);
+    else {
+      PTRACE(1, "IVR\tAttempt to make transfer when no second connecion in call");
+    }
+  }
 }
 
 
