@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24924 $
+ * $Revision: 25734 $
  * $Author: rjongbloed $
- * $Date: 2010-12-05 17:52:45 -0600 (Sun, 05 Dec 2010) $
+ * $Date: 2011-05-10 21:41:47 -0500 (Tue, 10 May 2011) $
  */
 
 #include <ptlib.h>
@@ -183,7 +183,6 @@ SDPMediaFormat::SDPMediaFormat(SDPMediaDescription & parent, const OpalMediaForm
 void SDPMediaFormat::SetFMTP(const PString & str)
 {
   m_fmtp = str;
-  InitialiseMediaFormat(m_mediaFormat); // Initialise all the OpalMediaOptions from m_fmtp
 }
 
 
@@ -199,10 +198,12 @@ PString SDPMediaFormat::GetFMTP() const
     return fmtp;
 
   PStringStream strm;
+  PStringSet used;
   for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); i++) {
     const OpalMediaOption & option = mediaFormat.GetOption(i);
     const PString & name = option.GetFMTPName();
-    if (!name.IsEmpty()) {
+    if (!name.IsEmpty() && !used.Contains(name)) {
+      used.Include(name);
       PString value = option.AsString();
       if (value.IsEmpty() && value != option.GetFMTPDefault())
         strm << name;
@@ -256,65 +257,12 @@ void SDPMediaFormat::PrintOn(ostream & strm) const
 }
 
 
-const OpalMediaFormat & SDPMediaFormat::GetMediaFormat() const
-{
-  if (m_mediaFormat.IsEmpty())
-    InitialiseMediaFormat(const_cast<SDPMediaFormat *>(this)->m_mediaFormat);
-  return m_mediaFormat;
-}
-
-
-OpalMediaFormat & SDPMediaFormat::GetWritableMediaFormat()
-{
-  if (m_mediaFormat.IsEmpty())
-    InitialiseMediaFormat(m_mediaFormat);
-  return m_mediaFormat;
-}
-
-
-void SDPMediaFormat::InitialiseMediaFormat(OpalMediaFormat & mediaFormat) const
-{
-  if (mediaFormat.IsEmpty()) {
-    OpalMediaFormatList masterFormats = OpalMediaFormat::GetMatchingRegisteredMediaFormats(payloadType, clockRate, encodingName, "sip");
-    for (OpalMediaFormatList::iterator iterFormat = masterFormats.begin(); iterFormat != masterFormats.end(); ++iterFormat) {
-      OpalMediaFormat adjustedFormat = *iterFormat;
-      SetMediaFormatOptions(adjustedFormat);
-      // skip formats whose fmtp don't match options
-      if (iterFormat->ValidateMerge(adjustedFormat)) {
-        PTRACE(3, "SIP\tRTP payload type " << encodingName << " matched to codec " << *iterFormat);
-        mediaFormat = adjustedFormat;
-        break;
-      }
-
-      PTRACE(4, "SIP\tRTP payload type " << encodingName << " not matched to codec " << *iterFormat);
-    }
-  }
-
-  if (mediaFormat.IsEmpty())
-    mediaFormat = OpalMediaFormat(encodingName);
-
-  if (mediaFormat.IsEmpty()) {
-    PTRACE(2, "SDP\tCould not find media format for \""
-           << encodingName << "\", pt=" << payloadType << ", clock=" << clockRate);
-    return;
-  }
-
-  SetMediaFormatOptions(mediaFormat);
-}
-
-
 void SDPMediaFormat::SetMediaFormatOptions(OpalMediaFormat & mediaFormat) const
 {
   mediaFormat.MakeUnique();
   mediaFormat.SetPayloadType(payloadType);
   mediaFormat.SetOptionInteger(OpalAudioFormat::ChannelsOption(), parameters.IsEmpty() ? 1 : parameters.AsUnsigned());
 
-  // Set bandwidth limits, if present
-  for (SDPBandwidth::const_iterator r = m_parent.GetBandwidth().begin(); r != m_parent.GetBandwidth().end(); ++r) {
-    if (r->second > 0)
-      mediaFormat.AddOption(new OpalMediaOptionString(SDPBandwidthPrefix + r->first, false, r->second), true);
-  }
-
   // Fill in the default values for (possibly) missing FMTP options
   for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); i++) {
     OpalMediaOption & option = const_cast<OpalMediaOption &>(mediaFormat.GetOption(i));
@@ -339,6 +287,10 @@ void SDPMediaFormat::SetMediaFormatOptions(OpalMediaFormat & mediaFormat) const
   if (mediaFormat.SetOptionString("FMTP", m_fmtp))
     return;
 
+  // Save the RTCP feedback (RFC4585) capability.
+  if (!m_rtcp_fb.IsEmpty())
+    mediaFormat.AddOption(new OpalMediaOptionString("RTCP-FB", false, m_rtcp_fb), true);
+
   // No FMTP to parse, may as well stop here
   if (m_fmtp.IsEmpty())
     return;
@@ -421,32 +373,57 @@ bool SDPMediaFormat::PreEncode()
 }
 
 
-bool SDPMediaFormat::PostDecode(unsigned bandwidth)
+bool SDPMediaFormat::PostDecode(const OpalMediaFormatList & mediaFormats, unsigned bandwidth)
 {
-  // Use GetWritableMediaFormat() to force creation of member
-  OpalMediaFormat & mediaFormat = GetWritableMediaFormat();
-  if (mediaFormat.IsEmpty())
-    return false;
-
   // try to init encodingName from global list, to avoid PAssert when media has no rtpmap
   if (encodingName.IsEmpty())
-    encodingName = mediaFormat.GetEncodingName();
+    encodingName = m_mediaFormat.GetEncodingName();
 
-  if (bandwidth > 0) {
-    PTRACE(4, "SDP\tAdjusting format \"" << mediaFormat << "\" bandwidth to " << bandwidth);
-    mediaFormat.SetOptionInteger(OpalMediaFormat::MaxBitRateOption(), bandwidth);
+  if (m_mediaFormat.IsEmpty()) {
+    for (OpalMediaFormatList::const_iterator iterFormat = mediaFormats.FindFormat(payloadType, clockRate, encodingName, "sip");
+         iterFormat != mediaFormats.end();
+         iterFormat = mediaFormats.FindFormat(payloadType, clockRate, encodingName, "sip", iterFormat)) {
+      OpalMediaFormat adjustedFormat = *iterFormat;
+      SetMediaFormatOptions(adjustedFormat);
+      // skip formats whose fmtp don't match options
+      if (iterFormat->ValidateMerge(adjustedFormat)) {
+        PTRACE(3, "SIP\tRTP payload type " << encodingName << " matched to codec " << *iterFormat);
+        m_mediaFormat = adjustedFormat;
+        break;
+      }
 
-    /* Some codecs (e.g. MPEG4, H.264) have difficulties with controlling max
-       bit rate. This is due to the very poor support of TIAS by SIP clients.
-       So, if we HAVE got a TIAS we indicate it with a special media option
-       so the codec can then trust the "MaxBitRate" option. If this is not
-       present then the codec has to play games with downgrading profile
-       levels to assure that a max bit rate is not exceeded. */
-    mediaFormat.AddOption(new OpalMediaOptionBoolean("SDP-Bandwidth-TIAS", false, OpalMediaOption::AndMerge, true), true);
+      PTRACE(4, "SIP\tRTP payload type " << encodingName << " not matched to codec " << *iterFormat);
+    }
+
+    if (m_mediaFormat.IsEmpty()) {
+      PTRACE(2, "SDP\tCould not find media format for \""
+             << encodingName << "\", pt=" << payloadType << ", clock=" << clockRate);
+      return false;
+    }
   }
 
-  mediaFormat.SetOptionString(OpalMediaFormat::ProtocolOption(), "SIP");
-  if (mediaFormat.ToNormalisedOptions())
+  SetMediaFormatOptions(m_mediaFormat);
+
+  /* Set bandwidth limits, if present, e.g. "SDP-Bandwidth-AS" and "SDP-Bandwidth-TIAS".
+
+     Some codecs (e.g. MPEG4, H.264) have difficulties with controlling max
+     bit rate. This is due to the very poor support of TIAS by SIP clients.
+     So, if we HAVE got a TIAS we indicate it with a special media option
+     so the codec can then trust the "MaxBitRate" option. If this is not
+     present then the codec has to play games with downgrading profile
+     levels to assure that a max bit rate is not exceeded. */
+  for (SDPBandwidth::const_iterator r = m_parent.GetBandwidth().begin(); r != m_parent.GetBandwidth().end(); ++r) {
+    if (r->second > 0)
+      m_mediaFormat.AddOption(new OpalMediaOptionString(SDPBandwidthPrefix + r->first, false, r->second), true);
+  }
+
+  if (bandwidth > 0) {
+    PTRACE(4, "SDP\tAdjusting format \"" << m_mediaFormat << "\" bandwidth to " << bandwidth);
+    m_mediaFormat.SetOptionInteger(OpalMediaFormat::MaxBitRateOption(), bandwidth);
+  }
+
+  m_mediaFormat.SetOptionString(OpalMediaFormat::ProtocolOption(), "SIP");
+  if (m_mediaFormat.ToNormalisedOptions())
     return true;
 
   PTRACE(2, "SDP\tCould not normalise format \"" << encodingName << "\", pt=" << payloadType << ", removing.");
@@ -456,13 +433,13 @@ bool SDPMediaFormat::PostDecode(unsigned bandwidth)
 
 //////////////////////////////////////////////////////////////////////////////
 
-unsigned & SDPBandwidth::operator[](const PString & type)
+unsigned & SDPBandwidth::operator[](const PCaselessString & type)
 {
-  return std::map<PString, unsigned>::operator[](type);
+  return std::map<PCaselessString, unsigned>::operator[](type);
 }
 
 
-unsigned SDPBandwidth::operator[](const PString & type) const
+unsigned SDPBandwidth::operator[](const PCaselessString & type) const
 {
   const_iterator it = find(type);
   return it != end() ? it->second : UINT_MAX;
@@ -490,12 +467,12 @@ bool SDPBandwidth::Parse(const PString & param)
 }
 
 
-void SDPBandwidth::SetMin(const PString & type, unsigned value)
+void SDPBandwidth::SetMax(const PCaselessString & type, unsigned value)
 {
   iterator it = find(type);
   if (it == end())
     (*this)[type] = value;
-  else if (it->second > value)
+  else if (it->second < value)
     it->second = value;
 }
 
@@ -513,6 +490,7 @@ SDPMediaDescription::SDPMediaDescription(const OpalTransportAddress & address, c
   portCount = 0;
 }
 
+
 PBoolean SDPMediaDescription::SetTransportAddress(const OpalTransportAddress &t)
 {
   PIPSocket::Address ip;
@@ -524,6 +502,7 @@ PBoolean SDPMediaDescription::SetTransportAddress(const OpalTransportAddress &t)
   return PFalse;
 }
 
+
 bool SDPMediaDescription::Decode(const PStringArray & tokens)
 {
   if (tokens.GetSize() < 3) {
@@ -587,6 +566,7 @@ bool SDPMediaDescription::Decode(const PStringArray & tokens)
   return PTrue;
 }
 
+
 void SDPMediaDescription::CreateSDPMediaFormats(const PStringArray & tokens)
 {
   // create the format list
@@ -603,6 +583,9 @@ void SDPMediaDescription::CreateSDPMediaFormats(const PStringArray & tokens)
 
 bool SDPMediaDescription::Decode(char key, const PString & value)
 {
+  /* NOTE: must make sure anything passed through to a SDPFormat isntance does
+           not require it to have an OpalMediaFormat yet, as that is not created
+           until PostDecode() */
   PINDEX pos;
 
   switch (key) {
@@ -637,7 +620,7 @@ bool SDPMediaDescription::Decode(char key, const PString & value)
 }
 
 
-bool SDPMediaDescription::PostDecode()
+bool SDPMediaDescription::PostDecode(const OpalMediaFormatList & mediaFormats)
 {
   unsigned bw = bandwidth[SDPSessionDescription::TransportIndependentBandwidthType()];
   if (bw == 0)
@@ -645,7 +628,7 @@ bool SDPMediaDescription::PostDecode()
 
   SDPMediaFormatList::iterator format = formats.begin();
   while (format != formats.end()) {
-    if (format->PostDecode(bw))
+    if (format->PostDecode(mediaFormats, bw))
       ++format;
     else
       formats.erase(format++);
@@ -657,6 +640,10 @@ bool SDPMediaDescription::PostDecode()
 
 void SDPMediaDescription::SetAttribute(const PString & attr, const PString & value)
 {
+  /* NOTE: must make sure anything passed through to a SDPFormat isntance does
+           not require it to have an OpalMediaFormat yet, as that is not created
+           until PostDecode() */
+
   // get the attribute type
   if (attr *= "sendonly") {
     direction = SendOnly;
@@ -903,6 +890,12 @@ SDPDummyMediaDescription::SDPDummyMediaDescription(const OpalTransportAddress &
 }
 
 
+SDPMediaDescription * SDPDummyMediaDescription::CreateEmpty() const
+{
+  return new SDPDummyMediaDescription(*this);
+}
+
+
 PString SDPDummyMediaDescription::GetSDPMediaType() const
 {
   return m_tokens[0];
@@ -949,6 +942,9 @@ SDPMediaFormat * SDPRTPAVPMediaDescription::CreateSDPMediaFormat(const PString &
 
 PString SDPRTPAVPMediaDescription::GetSDPPortList() const
 {
+  if (formats.IsEmpty())
+    return "127"; // Have to have SOMETHING
+
   PStringStream str;
 
   SDPMediaFormatList::const_iterator format;
@@ -974,28 +970,50 @@ bool SDPRTPAVPMediaDescription::PrintOn(ostream & str, const PString & connectSt
   return true;
 }
 
+
 void SDPRTPAVPMediaDescription::SetAttribute(const PString & attr, const PString & value)
 {
-  // handle rtpmap attribute
+  /* NOTE: must make sure anything passed through to a SDPFormat isntance does
+           not require it to have an OpalMediaFormat yet, as that is not created
+           until PostDecode() */
+
   if (attr *= "rtpmap") {
     PString params = value;
     SDPMediaFormat * format = FindFormat(params);
-    if (format != NULL) {
-      PStringArray tokens = params.Tokenise('/');
-      if (tokens.GetSize() < 2) {
-        PTRACE(2, "SDP\tMalformed rtpmap attribute for " << format->GetEncodingName());
-        return;
-      }
+    if (format == NULL)
+      return;
 
-      format->SetEncodingName(tokens[0]);
-      format->SetClockRate(tokens[1].AsUnsigned());
-      if (tokens.GetSize() > 2)
-        format->SetParameters(tokens[2]);
+    PStringArray tokens = params.Tokenise('/');
+    if (tokens.GetSize() < 2) {
+      PTRACE(2, "SDP\tMalformed rtpmap attribute for " << format->GetEncodingName());
+      return;
+    }
+
+    format->SetEncodingName(tokens[0]);
+    format->SetClockRate(tokens[1].AsUnsigned());
+    if (tokens.GetSize() > 2)
+      format->SetParameters(tokens[2]);
+
+    return;
+  }
+
+  if (attr *= "rtcp-fb") {
+    if (value[0] == '*') {
+      PString params = value.Mid(1).Trim();
+      SDPMediaFormatList::iterator format;
+      for (format = formats.begin(); format != formats.end(); ++format)
+        format->SetRTCP_FB(params);
+    }
+    else {
+      PString params = value;
+      SDPMediaFormat * format = FindFormat(params);
+      if (format != NULL)
+        format->SetRTCP_FB(params);
     }
     return;
   }
 
-  return SDPMediaDescription::SetAttribute(attr, value);
+  SDPMediaDescription::SetAttribute(attr, value);
 }
 
 
@@ -1003,10 +1021,17 @@ void SDPRTPAVPMediaDescription::SetAttribute(const PString & attr, const PString
 
 SDPAudioMediaDescription::SDPAudioMediaDescription(const OpalTransportAddress & address)
   : SDPRTPAVPMediaDescription(address, OpalMediaType::Audio())
+  , m_offerPTime(false)
 {
 }
 
 
+SDPMediaDescription * SDPAudioMediaDescription::CreateEmpty() const
+{
+  return new SDPAudioMediaDescription(OpalTransportAddress());
+}
+
+
 PString SDPAudioMediaDescription::GetSDPMediaType() const 
 { 
   return "audio"; 
@@ -1020,21 +1045,26 @@ bool SDPAudioMediaDescription::PrintOn(ostream & str, const PString & connectStr
     return false;
 
   /* The ptime parameter is a recommendation to the remote that we want them
-     to send that number of milliseconds on audio in each RTP packet. OPAL
-     does not have an equivalent parameter anywhere, so we do not provide it
-     on outgoing SDP. We do try to honour it on incoing SDP, however.
+     to send that number of milliseconds of audio in each RTP packet. The 
+     maxptime parameter specifies the absolute maximum packet size that can
+     be handled. There purpose is to limit the size of the packets sent by the
+     remote to prevent buffer overflows at this end.
+     
+     OPAL does not have a parameter equivalent to ptime. This code uses the 
+     TxFramesPerPacketOption. We go through all of the codecs offered and 
+     calculate the largest of them. The value can be controlled by 
+     manipulating the registered formats set or the set of formats given to 
+     AddMediaFormat. It would make most sense here if they were all the same.
 
      The maxptime parameter can be represented by the RxFramesPerPacketOption,
      so we go through all the codecs offered and calculate a maxptime based on
-     the smallest maximum rx packets of the codecs. Allowance must be made for
-     maxptime to be at least big enough for 1 frame per packet for the largest
-     frame size of those codecs.
-     
-     In practice this generally means if we mix GSM and G.723.1 then the
-     maxptime cannot be smaller than 30ms even if GSM wants one frame per
-     packet. That should still work as teh remote cannot send 2fpp as it would
-     exceed the 30ms.
+     the smallest maximum rx packets of the codecs.
 
+     Allowance must be made for either value to be at least big enough for 1 
+     frame per packet for the largest frame size of those codecs. In practice 
+     this generally means if we mix GSM and G.723.1 then the value cannot be 
+     smaller than 30ms even if GSM wants one frame per packet. That should 
+     still work as the remote cannot send 2fpp as it would exceed the 30ms.
      However, certain combinations cannot be represented, e.g. if you want 2fpp
      of G.729 (20ms) and 1fpp of G.723.1 (30ms) then the G.729 codec COULD
      receive 3fpp. This is really a failing in SIP/SDP and the techniques for
@@ -1042,6 +1072,20 @@ bool SDPAudioMediaDescription::PrintOn(ostream & str, const PString & connectStr
      what should be rare cases.
     */
 
+  if (m_offerPTime) {
+    unsigned ptime = 0;
+    for (SDPMediaFormatList::const_iterator format = formats.begin(); format != formats.end(); ++format) {
+      const OpalMediaFormat & mediaFormat = format->GetMediaFormat();
+      if (mediaFormat.HasOption(OpalAudioFormat::TxFramesPerPacketOption())) {
+        unsigned ptime1 = mediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption()) * mediaFormat.GetFrameTime() / mediaFormat.GetTimeUnits();
+        if (ptime < ptime1)
+          ptime = ptime1;
+      }
+    }
+    if (ptime > 0)
+      str << "a=ptime:" << ptime << "\r\n";
+  }
+
   unsigned largestFrameTime = 0;
   unsigned maxptime = UINT_MAX;
 
@@ -1071,6 +1115,10 @@ bool SDPAudioMediaDescription::PrintOn(ostream & str, const PString & connectStr
 
 void SDPAudioMediaDescription::SetAttribute(const PString & attr, const PString & value)
 {
+  /* NOTE: must make sure anything passed through to a SDPFormat isntance does
+           not require it to have an OpalMediaFormat yet, as that is not created
+           until PostDecode() */
+
   if (attr *= "ptime") {
     unsigned newTime = value.AsUnsigned();
     if (newTime < SDP_MIN_PTIME) {
@@ -1103,6 +1151,12 @@ SDPVideoMediaDescription::SDPVideoMediaDescription(const OpalTransportAddress &
 }
 
 
+SDPMediaDescription * SDPVideoMediaDescription::CreateEmpty() const
+{
+  return new SDPVideoMediaDescription(OpalTransportAddress());
+}
+
+
 PString SDPVideoMediaDescription::GetSDPMediaType() const 
 { 
   return "video"; 
@@ -1134,6 +1188,10 @@ bool SDPVideoMediaDescription::PrintOn(ostream & str, const PString & connectStr
 
 void SDPVideoMediaDescription::SetAttribute(const PString & attr, const PString & value)
 {
+  /* NOTE: must make sure anything passed through to a SDPFormat isntance does
+           not require it to have an OpalMediaFormat yet, as that is not created
+           until PostDecode() */
+
   if (attr *= "content") {
     PINDEX role = 0;
     PStringArray tokens = value.Tokenise(',');
@@ -1179,12 +1237,16 @@ bool SDPVideoMediaDescription::PreEncode()
       const OpalMediaOption & option = mediaFormat.GetOption(i);
       PCaselessString name = option.GetName();
       if (name.NumCompare(SDPBandwidthPrefix, sizeof(SDPBandwidthPrefix)-1) == EqualTo)
-        bandwidth.SetMin(name.Mid(sizeof(SDPBandwidthPrefix)-1), option.AsString().AsUnsigned());
+        bandwidth.SetMax(name.Mid(sizeof(SDPBandwidthPrefix)-1), option.AsString().AsUnsigned());
     }
 
+    /**We set the bandwidth parameter to the largest of all the formats offerred.
+       And individual format may be able to further retrict the bandwidth in it's
+       FMTP line, e.g. H.264 can use a max-br=XXX option.
+      */
     unsigned bw = mediaFormat.GetBandwidth();
-    bandwidth.SetMin(SDPSessionDescription::TransportIndependentBandwidthType(), bw);
-    bandwidth.SetMin(SDPSessionDescription::ApplicationSpecificBandwidthType(), (bw+999)/1000);
+    bandwidth.SetMax(SDPSessionDescription::TransportIndependentBandwidthType(), bw);
+    bandwidth.SetMax(SDPSessionDescription::ApplicationSpecificBandwidthType(), (bw+999)/1000);
   }
 
   return true;
@@ -1205,6 +1267,12 @@ PCaselessString SDPApplicationMediaDescription::GetSDPTransportType() const
 }
 
 
+SDPMediaDescription * SDPApplicationMediaDescription::CreateEmpty() const
+{
+  return new SDPApplicationMediaDescription(OpalTransportAddress());
+}
+
+
 PString SDPApplicationMediaDescription::GetSDPMediaType() const 
 { 
   return "application"; 
@@ -1219,6 +1287,9 @@ SDPMediaFormat * SDPApplicationMediaDescription::CreateSDPMediaFormat(const PStr
 
 PString SDPApplicationMediaDescription::GetSDPPortList() const
 {
+  if (formats.IsEmpty())
+    return "na"; // Have to have SOMETHING
+
   PStringStream str;
 
   // output encoding names for non RTP
@@ -1231,9 +1302,9 @@ PString SDPApplicationMediaDescription::GetSDPPortList() const
 
 //////////////////////////////////////////////////////////////////////////////
 
-const PString & SDPSessionDescription::ConferenceTotalBandwidthType()      { static PString s = "CT";   return s; }
-const PString & SDPSessionDescription::ApplicationSpecificBandwidthType()  { static PString s = "AS";   return s; }
-const PString & SDPSessionDescription::TransportIndependentBandwidthType() { static PString s = "TIAS"; return s; }
+const PCaselessString & SDPSessionDescription::ConferenceTotalBandwidthType()      { static const PConstCaselessString s("CT");   return s; }
+const PCaselessString & SDPSessionDescription::ApplicationSpecificBandwidthType()  { static const PConstCaselessString s("AS");   return s; }
+const PCaselessString & SDPSessionDescription::TransportIndependentBandwidthType() { static const PConstCaselessString s("TIAS"); return s; }
 
 SDPSessionDescription::SDPSessionDescription(time_t sessionId, unsigned version, const OpalTransportAddress & address)
   : sessionName(SIP_DEFAULT_SESSION_NAME)
@@ -1299,8 +1370,10 @@ PString SDPSessionDescription::Encode() const
 }
 
 
-PBoolean SDPSessionDescription::Decode(const PString & str)
+bool SDPSessionDescription::Decode(const PString & str, const OpalMediaFormatList & mediaFormats)
 {
+  PTRACE(5, "SDP\tDecode using media formats: " << setfill(',') << mediaFormats);
+
   bool atLeastOneValidMedia = false;
   bool ok = true;
 
@@ -1311,128 +1384,104 @@ PBoolean SDPSessionDescription::Decode(const PString & str)
   SDPMediaDescription * currentMedia = NULL;
   PINDEX i;
   for (i = 0; i < lines.GetSize(); i++) {
-    PString & line = lines[i];
-    PINDEX pos = line.Find('=');
-    if (pos == 1) {
-      PString value = line.Mid(pos+1).Trim();
+    const PString & line = lines[i];
+    if (line.GetLength() < 3 || line[1] != '=')
+      continue; // Ignore illegal lines
 
-      /////////////////////////////////
-      //
-      // Session description
-      //
-      /////////////////////////////////
+    PString value = line.Mid(2).Trim();
+
+    /////////////////////////////////
+    //
+    // Session description
+    //
+    /////////////////////////////////
   
-      if (currentMedia != NULL && line[0] != 'm') {
+    if (currentMedia != NULL && line[0] != 'm')
+      currentMedia->Decode(line[0], value);
+    else {
+      switch (line[0]) {
+        case 'v' : // protocol version (mandatory)
+          protocolVersion = value.AsInteger();
+          break;
 
-        // process all of the "a=rtpmap" lines first so that the media formats are 
-        // created before any media description paramaters are processed
-        int y = i;
-        for (int pass = 0; pass < 2; ++pass) {
-          y = i;
-          for (y = i; y < lines.GetSize(); ++y) {
-            PString & line2 = lines[y];
-            PINDEX pos = line2.Find('=');
-            if (pos == 1) {
-              if (line2[0] == 'm') {
-                --y;
-                break;
-              }
-              PString value = line2.Mid(pos+1).Trim();
-              bool isRtpMap = value.Left(6) *= "rtpmap";
-              if (pass == 0 && isRtpMap)
-                currentMedia->Decode(line2[0], value);
-              else if (pass == 1 && !isRtpMap)
-                currentMedia->Decode(line2[0], value);
+        case 'o' : // owner/creator and session identifier (mandatory)
+          ParseOwner(value);
+          break;
+
+        case 's' : // session name (mandatory)
+          sessionName = value;
+          break;
+
+        case 'c' : // connection information - not required if included in all media
+          defaultConnectAddress = ParseConnectAddress(value);
+          break;
+
+        case 't' : // time the session is active (mandatory)
+        case 'i' : // session information
+        case 'u' : // URI of description
+        case 'e' : // email address
+        case 'p' : // phone number
+          break;
+        case 'b' : // bandwidth information
+          bandwidth.Parse(value);
+          break;
+        case 'z' : // time zone adjustments
+        case 'k' : // encryption key
+        case 'r' : // zero or more repeat times
+          break;
+        case 'a' : // zero or more session attribute lines
+          if (value *= "sendonly")
+            SetDirection (SDPMediaDescription::SendOnly);
+          else if (value *= "recvonly")
+            SetDirection (SDPMediaDescription::RecvOnly);
+          else if (value *= "sendrecv")
+            SetDirection (SDPMediaDescription::SendRecv);
+          else if (value *= "inactive")
+            SetDirection (SDPMediaDescription::Inactive);
+          break;
+
+        case 'm' : // media name and transport address (mandatory)
+          {
+            if (currentMedia != NULL) {
+              PTRACE(3, "SDP\tParsed media session with " << currentMedia->GetSDPMediaFormats().GetSize()
+                                                          << " '" << currentMedia->GetSDPMediaType() << "' formats");
+              if (!currentMedia->PostDecode(mediaFormats))
+                ok = false;
             }
-          }
-        }
-        i = y;
-      }
-      else {
-        switch (line[0]) {
-          case 'v' : // protocol version (mandatory)
-            protocolVersion = value.AsInteger();
-            break;
 
-          case 'o' : // owner/creator and session identifier (mandatory)
-            ParseOwner(value);
-            break;
-
-          case 's' : // session name (mandatory)
-            sessionName = value;
-            break;
-
-          case 'c' : // connection information - not required if included in all media
-            defaultConnectAddress = ParseConnectAddress(value);
-            break;
-
-          case 't' : // time the session is active (mandatory)
-          case 'i' : // session information
-          case 'u' : // URI of description
-          case 'e' : // email address
-          case 'p' : // phone number
-            break;
-          case 'b' : // bandwidth information
-            bandwidth.Parse(value);
-            break;
-          case 'z' : // time zone adjustments
-          case 'k' : // encryption key
-          case 'r' : // zero or more repeat times
-            break;
-          case 'a' : // zero or more session attribute lines
-            if (value *= "sendonly")
-              SetDirection (SDPMediaDescription::SendOnly);
-            else if (value *= "recvonly")
-              SetDirection (SDPMediaDescription::RecvOnly);
-            else if (value *= "sendrecv")
-              SetDirection (SDPMediaDescription::SendRecv);
-            else if (value *= "inactive")
-              SetDirection (SDPMediaDescription::Inactive);
-            break;
-
-          case 'm' : // media name and transport address (mandatory)
-            {
-              if (currentMedia != NULL) {
-                PTRACE(3, "SDP\tParsed media session with " << currentMedia->GetSDPMediaFormats().GetSize()
-                                                            << " '" << currentMedia->GetSDPMediaType() << "' formats");
-                if (!currentMedia->PostDecode())
-                  ok = false;
-              }
+            currentMedia = NULL;
 
+            OpalMediaType mediaType;
+            OpalMediaTypeDefinition * defn;
+            PStringArray tokens = value.Tokenise(" ");
+            if (tokens.GetSize() < 4) {
+              PTRACE(1, "SDP\tMedia session has only " << tokens.GetSize() << " elements");
+            }
+            else if ((mediaType = OpalMediaType::GetMediaTypeFromSDP(tokens[0], tokens[2])).empty()) {
+              PTRACE(1, "SDP\tUnknown SDP media type " << tokens[0]);
+            }
+            else if ((defn = mediaType.GetDefinition()) == NULL) {
+              PTRACE(1, "SDP\tNo definition for SDP media type " << tokens[0]);
+            }
+            else if ((currentMedia = defn->CreateSDPMediaDescription(defaultConnectAddress)) == NULL) {
+              PTRACE(1, "SDP\tCould not create SDP media description for SDP media type " << tokens[0]);
+            }
+            else if (currentMedia->Decode(tokens))
+              atLeastOneValidMedia = true;
+            else {
+              delete currentMedia;
               currentMedia = NULL;
-
-              OpalMediaType mediaType;
-              OpalMediaTypeDefinition * defn;
-              PStringArray tokens = value.Tokenise(" ");
-              if (tokens.GetSize() < 4) {
-                PTRACE(1, "SDP\tMedia session has only " << tokens.GetSize() << " elements");
-              }
-              else if ((mediaType = OpalMediaType::GetMediaTypeFromSDP(tokens[0], tokens[2])).empty()) {
-                PTRACE(1, "SDP\tUnknown SDP media type " << tokens[0]);
-              }
-              else if ((defn = mediaType.GetDefinition()) == NULL) {
-                PTRACE(1, "SDP\tNo definition for SDP media type " << tokens[0]);
-              }
-              else if ((currentMedia = defn->CreateSDPMediaDescription(defaultConnectAddress)) == NULL) {
-                PTRACE(1, "SDP\tCould not create SDP media description for SDP media type " << tokens[0]);
-              }
-              else if (currentMedia->Decode(tokens))
-                atLeastOneValidMedia = true;
-              else {
-                delete currentMedia;
-                currentMedia = NULL;
-              }
-
-              if (currentMedia == NULL)
-                currentMedia = new SDPDummyMediaDescription(defaultConnectAddress, tokens);
-
-              mediaDescriptions.Append(currentMedia);
             }
-            break;
 
-          default:
-            PTRACE(1, "SDP\tUnknown session information key " << line[0]);
-        }
+            if (currentMedia == NULL)
+              currentMedia = new SDPDummyMediaDescription(defaultConnectAddress, tokens);
+
+            mediaDescriptions.Append(currentMedia);
+          }
+          break;
+
+        default:
+          PTRACE(1, "SDP\tUnknown session information key " << line[0]);
       }
     }
   }
@@ -1440,7 +1489,7 @@ PBoolean SDPSessionDescription::Decode(const PString & str)
   if (currentMedia != NULL) {
     PTRACE(3, "SDP\tParsed media session with " << currentMedia->GetSDPMediaFormats().GetSize()
                                                 << " '" << currentMedia->GetSDPMediaType() << "' formats");
-    if (!currentMedia->PostDecode())
+    if (!currentMedia->PostDecode(mediaFormats))
       ok = false;
   }
 
@@ -1550,6 +1599,14 @@ OpalMediaFormatList SDPSessionDescription::GetMediaFormats() const
 #endif
   }
 
+#if OPAL_T38_CAPABILITY
+  /* We default to having T.38 included as most UAs do not actually
+     tell you that they support it or not. For the re-INVITE mechanism
+     to work correctly, the rest ofthe system has to assume that the
+     UA is capable of it, even it it isn't. */
+  formatList += OpalT38;
+#endif
+
   return formatList;
 }
 
