@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24940 $
+ * $Revision: 25934 $
  * $Author: rjongbloed $
- * $Date: 2010-12-15 18:50:14 -0600 (Wed, 15 Dec 2010) $
+ * $Date: 2011-05-31 03:02:45 -0500 (Tue, 31 May 2011) $
  */
 
 #include <ptlib.h>
@@ -52,8 +52,11 @@
 
 ////////////////////////////////////////////////////////////////////////////
 
-SIPEndPoint::SIPEndPoint(OpalManager & mgr)
-  : OpalRTPEndPoint(mgr, "sip", CanTerminateCall)
+SIPEndPoint::SIPEndPoint(OpalManager & mgr,
+                         unsigned maxConnectionThreads,
+                         unsigned maxHandlerThreads)
+  : OpalRTPEndPoint(mgr, "sip", CanTerminateCall|SupportsE164)
+  , m_defaultPrackMode(SIPConnection::e_prackSupported)
   , retryTimeoutMin(500)             // 0.5 seconds
   , retryTimeoutMax(0, 4)            // 4 seconds
   , nonInviteTimeout(0, 16)          // 16 seconds
@@ -66,6 +69,8 @@ SIPEndPoint::SIPEndPoint(OpalManager & mgr)
   , natBindingTimeout(0, 0, 1)       // 1 minute
   , m_shuttingDown(false)
   , m_defaultAppearanceCode(-1)
+  , m_connectionThreadPool(maxConnectionThreads)
+  , m_handlerThreadPool(maxHandlerThreads)
 
 #ifdef _MSC_VER
 #pragma warning(disable:4355)
@@ -73,10 +78,6 @@ SIPEndPoint::SIPEndPoint(OpalManager & mgr)
 
   , m_highPriorityMonitor(*this, HighPriority)
   , m_lowPriorityMonitor(*this, LowPriority)
-
-#if OPAL_HAS_SIPIM
-  , m_sipIMManager(*this)
-#endif
   , m_disableTrying(true)
 
 #ifdef _MSC_VER
@@ -137,11 +138,13 @@ void SIPEndPoint::ShutDown()
     PThread::Sleep(100);
   }
 
-  // Clean up transactions still in progress, waiting for them to complete.
+  // Clean up transactions still in progress, waiting for them to terminate.
   PSafePtr<SIPTransaction> transaction;
   while ((transaction = transactions.GetAt(0, PSafeReference)) != NULL) {
-    transaction->WaitForTermination();
-    transactions.RemoveAt(transaction->GetTransactionID());
+    if (transaction->IsTerminated())
+      transactions.RemoveAt(transaction->GetTransactionID());
+    else
+      PThread::Sleep(100);
   }
 
   // Now shut down listeners and aggregators
@@ -150,7 +153,7 @@ void SIPEndPoint::ShutDown()
 
 
 PString SIPEndPoint::GetDefaultTransport() const 
-{  
+{
   return "udp$,tcp$"
 #if OPAL_PTLIB_SSL
          ",tcps$:5061"
@@ -180,7 +183,7 @@ PBoolean SIPEndPoint::NewIncomingConnection(OpalTransport * transport)
     PTRACE(2, "SIP\tListening thread finished.");
   }
   else {
-    transport->SetBufferSize(SIP_PDU::MaxSize);
+    transport->SetBufferSize(m_maxSizeUDP);
     HandlePDU(*transport); // Always just one PDU
   }
 
@@ -219,10 +222,11 @@ void SIPEndPoint::NATBindingRefresh(PTimer &, INT)
 
       switch (natMethod) {
 
-        case Options: 
+        case Options:
           {
-            SIPOptions options(*this, *transport, SIPURL(transport->GetRemoteAddress()).GetHostName());
-            options.Write(*transport);
+            SIPOptions::Params params;
+            params.m_remoteAddress = transport->GetRemoteAddress().GetHostName();
+            SendOPTIONS(params);
           }
           break;
 
@@ -243,6 +247,13 @@ void SIPEndPoint::NATBindingRefresh(PTimer &, INT)
 OpalTransport * SIPEndPoint::CreateTransport(const SIPURL & remoteURL, const PString & localInterface)
 {
   OpalTransportAddress remoteAddress = remoteURL.GetHostAddress();
+  if (remoteAddress.IsEmpty()) {
+    if (GetRegistrationsCount() == 0) {
+      PTRACE(1, "SIP\tCannot use tel URI with phone-context or existing registration.");
+      return false;
+    }
+    remoteAddress = SIPURL(GetRegistrations()[0]).GetHostAddress();
+  }
 
   OpalTransportAddress localAddress;
   if (!localInterface.IsEmpty()) {
@@ -282,7 +293,7 @@ OpalTransport * SIPEndPoint::CreateTransport(const SIPURL & remoteURL, const PSt
 
   PTRACE(4, "SIP\tCreated transport " << *transport);
 
-  transport->SetBufferSize(SIP_PDU::MaxSize);
+  transport->SetBufferSize(m_maxSizeUDP);
   if (!transport->Connect()) {
     PTRACE(1, "SIP\tCould not connect to " << remoteAddress << " - " << transport->GetErrorText());
     transport->CloseWait();
@@ -308,16 +319,19 @@ void SIPEndPoint::HandlePDU(OpalTransport & transport)
   SIP_PDU * pdu = new SIP_PDU;
 
   PTRACE(4, "SIP\tWaiting for PDU on " << transport);
-  if (pdu->Read(transport)) {
+  SIP_PDU::StatusCodes status = pdu->Read(transport);
+  if (status == SIP_PDU::Successful_OK) {
     if (OnReceivedPDU(transport, pdu)) 
       return;
   }
   else {
-    PTRACE_IF(1, transport.GetErrorCode(PChannel::LastReadError) != PChannel::NoError,
-              "SIP\tPDU Read failed: " << transport.GetErrorText(PChannel::LastReadError));
-    if (transport.good()) {
-      PTRACE(2, "SIP\tMalformed request received on " << transport);
-      pdu->SendResponse(transport, SIP_PDU::Failure_BadRequest, this);
+    if (status < SIP_PDU::Successful_OK) {
+      PTRACE(1, "SIP\tPDU Read failed: " << transport.GetErrorText(PChannel::LastReadError));
+    }
+    else {
+      PTRACE_IF(2, status == SIP_PDU::Failure_BadRequest,
+                "SIP\tMalformed request received on " << transport);
+      pdu->SendResponse(transport, status, this);
     }
   }
 
@@ -414,6 +428,7 @@ PBoolean SIPEndPoint::IsAcceptedAddress(const SIPURL & /*toAddr*/)
 }
 
 
+
 SIPConnection * SIPEndPoint::CreateConnection(OpalCall & call,
                                               const PString & token,
                                               void * /*userData*/,
@@ -444,13 +459,17 @@ PBoolean SIPEndPoint::SetupTransfer(const PString & token,
   OpalConnection::StringOptions options;
   if (!callId.IsEmpty())
     options.SetAt(SIP_HEADER_REPLACES, callId);
+  options.SetAt(SIP_HEADER_REFERRED_BY, otherConnection->GetRedirectingParty());
   options.SetAt(OPAL_OPT_CALLING_PARTY_URL, otherConnection->GetLocalPartyURL());
 
   SIPConnection * connection = CreateConnection(call, SIPURL::GenerateTag(), userData, TranslateENUM(remoteParty), NULL, NULL, 0, &options);
   if (!AddConnection(connection))
     return false;
 
-  otherConnection->Release(OpalConnection::EndedByCallForwarded);
+  if (remoteParty.Find(";OPAL-"OPAL_SIP_REFERRED_CONNECTION) == P_MAX_INDEX)
+    otherConnection->Release(OpalConnection::EndedByCallForwarded);
+  else
+    otherConnection->SetPhase(OpalConnection::ForwardingPhase);
   otherConnection->CloseMediaStreams();
 
   return connection->SetUpConnection();
@@ -495,9 +514,8 @@ bool SIPEndPoint::ClearDialogContext(SIPDialogContext & context)
   if (transport.get() == NULL)
     return true; // Can't create transport, so remote host uncontactable, assume dialog cleared.
 
-  PSafePtr<SIPTransaction> byeTransaction = new SIPTransaction(SIP_PDU::Method_BYE, *this, *transport);
-  byeTransaction->InitialiseHeaders(context);
-  byeTransaction->WaitForTermination();
+  PSafePtr<SIPTransaction> byeTransaction = new SIPBye(*this, *transport, context);
+  byeTransaction->WaitForCompletion();
   return !byeTransaction->IsFailed();
 }
 
@@ -613,6 +631,17 @@ bool SIPEndPoint::OnReceivedConnectionlessPDU(OpalTransport & transport, SIP_PDU
     return false;
   }
 
+  // Check if we have already received this command (have a transaction)
+  {
+    PString id = pdu->GetTransactionID();
+    PSafePtr<SIPResponse> transaction = PSafePtrCast<SIPTransaction, SIPResponse>(GetTransaction(id, PSafeReadOnly));
+    if (transaction != NULL) {
+      PTRACE(4, "SIP\tRetransmitting previous response for transaction id=" << id);
+      transaction->Send(transport, *pdu);
+      return false;
+    }
+  }
+
   switch (pdu->GetMethod()) {
     case SIP_PDU::Method_INVITE :
       pdu->AdjustVia(transport);   // // Adjust the Via list
@@ -648,12 +677,15 @@ bool SIPEndPoint::OnReceivedConnectionlessPDU(OpalTransport & transport, SIP_PDU
         return false;
       break;
 
-    case SIP_PDU::Method_ACK :
     case SIP_PDU::Method_BYE :
-      // If we receive an ACK or BYE outside of the context of a connection, ignore it.
+      // If we receive a BYE outside of the context of a connection, tell them.
       pdu->SendResponse(transport, SIP_PDU::Failure_TransactionDoesNotExist, this);
       return false;
 
+      // If we receive an ACK outside of the context of a connection, ignore it.
+    case SIP_PDU::Method_ACK :
+      return false;
+
     default :
       break;
   }
@@ -682,12 +714,7 @@ PBoolean SIPEndPoint::OnReceivedSUBSCRIBE(OpalTransport & transport, SIP_PDU & p
   // speaking we should check the from-tag and to-tags as well due to it being a dialog.
   PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(mime.GetCallID(), PSafeReadWrite);
   if (handler == NULL) {
-    SIPDialogContext dialog;
-    dialog.SetRequestURI(mime.GetContact());
-    dialog.SetLocalURI(mime.GetTo());
-    dialog.SetRemoteURI(mime.GetFrom());
-    dialog.SetCallID(mime.GetCallID());
-    dialog.SetRouteSet(mime.GetRecordRoute(true));
+    SIPDialogContext dialog(mime);
 
     handler = new SIPNotifyHandler(*this, dialog.GetRemoteURI().AsString(), eventPackage, dialog);
     handler.SetSafetyMode(PSafeReadWrite);
@@ -727,7 +754,8 @@ void SIPEndPoint::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & res
   if (handler != NULL)
     handler->OnReceivedResponse(transaction, response);
   else {
-    PTRACE(2, "SIP\tResponse received for unknown handler ID: " << response.GetMIME().GetCallID());
+    PTRACE(2, "SIP\tResponse for " << transaction << " received,"
+              " but unknown handler, ID: " << transaction.GetMIME().GetCallID());
   }
 }
 
@@ -881,7 +909,8 @@ void SIPEndPoint::OnTransactionFailed(SIPTransaction & transaction)
   if (handler != NULL) 
     handler->OnTransactionFailed(transaction);
   else {
-    PTRACE(2, "SIP\tResponse received for unknown handler ID: " << transaction.GetMIME().GetCallID());
+    PTRACE(2, "SIP\tTransaction " << transaction << " failed,"
+              " unknown handler, ID: " << transaction.GetMIME().GetCallID());
   }
 }
 
@@ -920,20 +949,76 @@ bool SIPEndPoint::OnReceivedMESSAGE(OpalTransport & transport, SIP_PDU & pdu)
   // handle a MESSAGE received outside the context of a call
   PTRACE(3, "SIP\tReceived MESSAGE outside the context of a call");
 
-  if (m_onConnectionlessMessage.IsNULL()) {
-    pdu.SendResponse(transport, SIP_PDU::Successful_OK, this);
-    return true;
+  // if there is a callback, assume that the application knows what it is doing
+  if (!m_onConnectionlessMessage.IsNULL()) {
+    ConnectionlessMessageInfo info(transport, pdu);
+    m_onConnectionlessMessage(*this, info);
+    return info.m_status;
   }
 
-  ConnectionlessMessageInfo info(transport, pdu);
-  m_onConnectionlessMessage(*this, info);
+  SIPMIMEInfo & mime  = pdu.GetMIME();
 
-  return info.m_status;
+  SIPURL from(mime.GetFrom());
+  from.Sanitise(SIPURL::FromURI);
+
+  SIPURL to(mime.GetTo());
+  to.Sanitise(SIPURL::ToURI);
+
+  OpalIMManager & imManager = manager.GetIMManager();
+  OpalIMContext::SentStatus status;
+  PString conversationId;
+  {
+    OpalIM * message    = new OpalIM();
+    message->m_to       = to.AsString();
+    message->m_from     = from.AsString();
+    message->m_mimeType = mime.GetContentType();
+    message->m_toAddr   = transport.GetLastReceivedAddress();
+    message->m_fromAddr = transport.GetRemoteAddress();
+    message->m_body     = pdu.GetEntityBody();
+    status = imManager.OnIncomingMessage(message, conversationId);
+  }
+
+  SIPResponse * response = new SIPResponse(*this, SIP_PDU::Failure_BadRequest);
+
+  switch (status ) {
+    case OpalIMContext::SentOK:
+    case OpalIMContext::SentPending:
+      response->SetStatusCode(SIP_PDU::Successful_Accepted);
+      break;
+
+    case OpalIMContext::SentUnacceptableContent:
+      response->SetStatusCode(SIP_PDU::Failure_UnsupportedMediaType);
+
+      // urrgh - complicated process to get the content types...
+      {
+        PStringArray contentTypes;
+        {
+          PSafePtr<OpalIMContext> context = imManager.FindContextByIdWithLock(conversationId, PSafeReadOnly);
+          if (context != NULL)
+            contentTypes = context->GetContentTypes();
+        }
+        if (contentTypes.GetSize() != 0) {
+          PStringStream strm;
+          strm << setfill(',') << contentTypes;
+          response->GetMIME().SetAccept(strm);
+        }
+      }
+      break;
+
+    default:
+      response->SetStatusCode(SIP_PDU::Failure_BadRequest);
+      break;
+  }
+
+  response->Send(transport, pdu);
+  return true;
 }
 
+
 bool SIPEndPoint::OnReceivedOPTIONS(OpalTransport & transport, SIP_PDU & pdu)
 {
-  pdu.SendResponse(transport, SIP_PDU::Successful_OK, this);
+  SIPResponse * response = new SIPResponse(*this, SIP_PDU::Successful_OK);
+  response->Send(transport, pdu);
   return true;
 }
 
@@ -962,7 +1047,7 @@ bool SIPEndPoint::Register(const PString & host,
 }
 
 
-bool SIPEndPoint::Register(const SIPRegister::Params & newParams, PString & aor)
+bool SIPEndPoint::Register(const SIPRegister::Params & newParams, PString & aor, bool asynchronous)
 {
   PTRACE(4, "SIP\tStart REGISTER\n" << newParams);
 
@@ -983,7 +1068,15 @@ bool SIPEndPoint::Register(const SIPRegister::Params & newParams, PString & aor)
 
   aor = handler->GetAddressOfRecord().AsString();
 
-  return handler->ActivateState(SIPHandler::Subscribing);
+  if (!handler->ActivateState(SIPHandler::Subscribing))
+    return false;
+
+  if (asynchronous)
+    return true;
+
+  m_registrationComplete[aor].Wait();
+  m_registrationComplete.erase(aor);
+  return handler->GetState() == SIPHandler::Subscribed;
 }
 
 
@@ -1039,6 +1132,16 @@ bool SIPEndPoint::UnregisterAll()
 void SIPEndPoint::OnRegistrationStatus(const RegistrationStatus & status)
 {
   OnRegistrationStatus(status.m_addressofRecord, status.m_wasRegistering, status.m_reRegistering, status.m_reason);
+
+  if (!status.m_wasRegistering ||
+       status.m_reRegistering ||
+       status.m_reason == SIP_PDU::Information_Trying ||
+       status.m_reason == SIP_PDU::Failure_UnAuthorised)
+    return;
+
+  std::map<PString, PSyncPoint>::iterator it = m_registrationComplete.find(status.m_addressofRecord);
+  if (it != m_registrationComplete.end())
+    it->second.Signal();
 }
 
 
@@ -1132,38 +1235,46 @@ bool SIPEndPoint::IsSubscribed(const PString & eventPackage, const PString & tok
 }
 
 
-bool SIPEndPoint::Unsubscribe(const PString & token)
+bool SIPEndPoint::Unsubscribe(const PString & token, bool invalidateNotifiers)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(token, PSafeReference);
-  if (handler != NULL)
-    return handler->ActivateState(SIPHandler::Unsubscribing);
-
-  PTRACE(1, "SIP\tCould not find active SUBSCRIBE of id " << token);
-  return PFalse;
+  return Unsubscribe(SIPEventPackage(), token, invalidateNotifiers);
 }
 
 
-bool SIPEndPoint::Unsubscribe(SIPSubscribe::PredefinedPackages eventPackage, const PString & token)
+bool SIPEndPoint::Unsubscribe(SIPSubscribe::PredefinedPackages eventPackage,
+                              const PString & token,
+                              bool invalidateNotifiers)
 {
-  return Unsubscribe(SIPEventPackage(eventPackage), token);
+  return Unsubscribe(SIPEventPackage(eventPackage), token, invalidateNotifiers);
 }
 
 
-bool SIPEndPoint::Unsubscribe(const PString & eventPackage, const PString & token)
+bool SIPEndPoint::Unsubscribe(const PString & eventPackage, const PString & token, bool invalidateNotifiers)
 {
   PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(token, PSafeReference);
   if (handler == NULL)
     handler = activeSIPHandlers.FindSIPHandlerByUrl(token, SIP_PDU::Method_SUBSCRIBE, eventPackage, PSafeReference);
   else {
-    if (handler->GetEventPackage() != eventPackage)
+    if (!eventPackage.IsEmpty() && handler->GetEventPackage() != eventPackage)
       handler.SetNULL();
   }
 
-  if (handler != NULL)
-    return handler->ActivateState(SIPHandler::Unsubscribing);
+  if (handler == NULL) {
+    PTRACE(1, "SIP\tCould not find active SUBSCRIBE of " << eventPackage << " package to " << token);
+    return false;
+  }
 
-  PTRACE(1, "SIP\tCould not find active SUBSCRIBE of " << eventPackage << " package to " << token);
-  return PFalse;
+  if (invalidateNotifiers) {
+    PSafePtr<SIPSubscribeHandler> subscription = PSafePtrCast<SIPHandler, SIPSubscribeHandler>(handler);
+    if (subscription != NULL) {
+      SIPSubscribe::Params params(subscription->GetParams());
+      params.m_onNotify = NULL;
+      params.m_onSubcribeStatus = NULL;
+      subscription->UpdateParameters(params);
+    }
+  }
+
+  return handler->ActivateState(SIPHandler::Unsubscribing);
 }
 
 
@@ -1247,33 +1358,171 @@ bool SIPEndPoint::Notify(const SIPURL & aor, const PString & eventPackage, const
 
 PBoolean SIPEndPoint::Message(const PURL & to, const PString & type, const PString & body, PURL & from, PString & conversationId)
 {
-  if (conversationId.IsEmpty()) 
-    conversationId = SIPTransaction::GenerateCallID();
+  OpalIM message;
+  message.m_to.Parse(to, "sip");
+  message.m_from.Parse(from, "sip");
+  message.m_mimeType       = type;
+  message.m_body           = body;
+  message.m_conversationId = conversationId;
 
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to.AsString(), SIP_PDU::Method_MESSAGE, PSafeReadWrite);
+  bool stat = Message(message);
 
-  if (handler != NULL) {
-    handler->SetBody(body);
+  from           = message.m_from;
+  conversationId = message.m_conversationId;
+
+  return stat;
+}
+
+
+bool SIPEndPoint::Message(OpalIM & message)
+{
+  OpalIMContext::SentStatus status = SendMESSAGE(message);
+  return (status == OpalIMContext::SentOK) || (status == OpalIMContext::SentPending);
+}
+
+
+OpalIMContext::SentStatus SIPEndPoint::SendMESSAGE(OpalIM & message)
+{
+  // This function is assumed to be used by applications that don't 
+  // know about OpalIMContext, and therefore, have no knowledge of
+  // how to link individual messages into a conversation using the id
+  // 
+  // As such, we use the "from" and "to" fields to identify which
+  // existing conversation this message belongs to (if any) so
+  // that the same Call-ID can be used for each transmission.
+  // Without this, each message could appear as a seperate conversation
+  //
+  // But if the conversation ID *is* set, ignore all of this and
+  // assume the application knows what it is doing.
+  //
+
+  if (message.m_to.IsEmpty())
+    return OpalIMContext::SentDestinationUnknown;
+
+  // don't send empty MESSAGE because some clients barf (cough...X-Lite...cough)
+  if (message.m_body.IsEmpty() && (message.m_body *= "text/plain"))
+    return OpalIMContext::SentOK;
+
+  SIPMessage::Params params;
+  OpalSIPIMContext::PopulateParams(params, message);
+
+  OpalIMContext::SentStatus stat = SendMESSAGE(params);
+
+  // conversation ID may have been changed
+  message.m_conversationId = params.m_id;
+
+  // set the from address as well
+  //if (message.m_from.IsEmpty())
+  //  message.m_from = ((SIPMessageHandler *)&*handler)->GetLocalAddress();
+
+  return stat;
+}
+
+
+OpalIMContext::SentStatus SIPEndPoint::SendMESSAGE(SIPMessage::Params & params)
+{
+  //
+  // if conversation ID has been set, assume the handler with the matching call ID is correct if the destination is the same, else create a new conversation
+  // if no conversation ID has been set, see if the destination AOR exists and use that handler (and it's call ID)
+  // 
+  PSafePtr<SIPHandler> handler;
+  if (!params.m_id.IsEmpty()) {
+    handler = activeSIPHandlers.FindSIPHandlerByCallID(params.m_id, PSafeReference);
+    if ((handler != NULL) && !(handler->GetAddressOfRecord().AsString() *= params.m_localAddress))
+      handler.SetNULL();
   }
   else {
-    SIPMessage::Params params;
-    params.m_localAddress    = from.AsString();
-    params.m_addressOfRecord = params.m_localAddress;
-    params.m_remoteAddress   = to.AsString();
-    params.m_id              = conversationId;
-    params.m_contentType     = type;
+    handler = activeSIPHandlers.FindSIPHandlerByUrl(params.m_remoteAddress, SIP_PDU::Method_MESSAGE, PSafeReference);
+    if (handler == NULL)
+      params.m_id = OpalGloballyUniqueID().AsString();
+    else
+      params.m_id = handler->GetCallID();
+  }
 
-    handler = new SIPMessageHandler(*this, params, body);
+
+  // create or update the handler if required
+  if (handler == NULL) {
+    handler = new SIPMessageHandler(*this, params);
     activeSIPHandlers.Append(handler);
   }
 
+  {
+    PSafePtr<SIPMessageHandler> messageHandler = PSafePtrCast<SIPHandler, SIPMessageHandler>(handler);
+    messageHandler->SetMessageId(params.m_messageId);
+  }
+
+  SIPMIMEInfo & mime = handler->m_mime;
+  mime.SetContentType(params.m_contentType);
+  handler->SetBody(params.m_body);
+
   if (!handler->ActivateState(SIPHandler::Subscribing))
-    return false;
+    return OpalIMContext::SentNoTransport;
 
-  from           = ((SIPMessageHandler *)&*handler)->m_localAddress;
-  conversationId = ((SIPMessageHandler *)&*handler)->m_id;
+  return OpalIMContext::SentPending;
+}
 
-  return true;
+
+void SIPEndPoint::OnMESSAGECompleted(const SIPMessage::Params & params, SIP_PDU::StatusCodes reason)
+{
+  // not possible, but let's be paranoid
+  if (params.m_id.IsEmpty()) {
+    PTRACE(2, "SIP\tHow did a MESSAGE get sent without an ID?");
+    return;
+  }
+
+  PTRACE(4, "SIP\tFinal status of message in conversation '" << params.m_id << "' received - " << reason);
+
+  OpalIMContext::SentStatus status;
+  int reasonClass = ((int)reason)/100;
+  switch (reason) {
+    case SIP_PDU::Successful_OK:
+      status = OpalIMContext::SentOK;
+      break;
+    case SIP_PDU::Successful_Accepted:
+      status = OpalIMContext::SentAccepted;
+      break;
+    case SIP_PDU::Failure_RequestTimeout:
+      status = OpalIMContext::SentNoAnswer;
+      break;
+    default:
+      switch (reasonClass) {
+        case 2:
+          status = OpalIMContext::SentOK;
+          break;
+        case 3:
+          status = OpalIMContext::SentFailedGeneric;
+          break;
+        default:
+          status = OpalIMContext::SentFailedGeneric;
+          break;
+      }
+  }
+
+  OpalIMManager & mgr = manager.GetIMManager();
+
+  OpalIMContext::MessageSentInfo info;
+
+  info.messageId = params.m_messageId;
+  info.status    = status;
+  mgr.AddWork(new OpalIMManager::MessageSent_Work(mgr, params.m_id, info));
+}
+
+
+bool SIPEndPoint::SendOPTIONS(const SIPOptions::Params & newParams)
+{
+  SIPOptions::Params params(newParams);
+  params.Normalise(GetDefaultLocalPartyName(), GetNotifierTimeToLive());
+
+  PSafePtr<SIPHandler> handler = new SIPOptionsHandler(*this, params);
+  activeSIPHandlers.Append(handler);
+  return handler->ActivateState(SIPHandler::Subscribing);
+}
+
+
+void SIPEndPoint::OnOptionsCompleted(const SIPOptions::Params & PTRACE_PARAM(params),
+                                                const SIP_PDU & PTRACE_PARAM(response))
+{
+  PTRACE(3, "SIP\tCompleted OPTIONS command to " << params.m_remoteAddress << ", status=" << response.GetStatusCode());
 }
 
 
@@ -1295,7 +1544,7 @@ bool SIPEndPoint::Publish(const SIPSubscribe::Params & newParams, const PString
 
   SIPSubscribe::Params params(newParams);
   params.Normalise(GetDefaultLocalPartyName(), GetNotifierTimeToLive());
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(params.m_addressOfRecord, SIP_PDU::Method_PUBLISH, PSafeReadWrite);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(params.m_addressOfRecord, SIP_PDU::Method_PUBLISH, params.m_eventPackage, PSafeReadWrite);
   if (handler != NULL)
     handler->SetBody(body);
   else {
@@ -1323,7 +1572,7 @@ bool SIPEndPoint::Publish(const PString & to, const PString & body, unsigned exp
 bool SIPEndPoint::PublishPresence(const SIPPresenceInfo & info, unsigned expire)
 {
   SIPSubscribe::Params params(SIPSubscribe::Presence);
-  params.m_addressOfRecord = info.m_entity; // As required by OMA-TS-Presence_SIMPLE-V2_0-20090917-C
+  params.m_addressOfRecord = info.m_contact.IsEmpty() ? info.m_entity.AsString() : info.m_contact;
   params.m_expire          = expire;
   params.m_agentAddress    = info.m_presenceAgent;
   params.m_contentType     = "application/pidf+xml";
@@ -1335,6 +1584,8 @@ bool SIPEndPoint::PublishPresence(const SIPPresenceInfo & info, unsigned expire)
 
 void SIPEndPoint::OnPresenceInfoReceived(const SIPPresenceInfo & info)
 {
+  PTRACE(4, "SIP\tReceived presence for entity '" << info.m_entity << "' using old API");
+
   // For backward compatibility
   switch (info.m_state) {
     case OpalPresenceInfo::Available :
@@ -1367,13 +1618,6 @@ void SIPEndPoint::SendNotifyDialogInfo(const SIPDialogNotification & info)
   Notify(info.m_entity, SIPEventPackage(SIPSubscribe::Dialog), info);
 }
 
-
-void SIPEndPoint::OnMessageFailed(const SIPURL & /* messageUrl */,
-          SIP_PDU::StatusCodes /* reason */)
-{
-}
-
-
 void SIPEndPoint::SetProxy(const PString & hostname,
                            const PString & username,
                            const PString & password)
@@ -1396,6 +1640,7 @@ void SIPEndPoint::SetProxy(const PString & hostname,
 void SIPEndPoint::SetProxy(const SIPURL & url) 
 { 
   proxy = url; 
+  PTRACE_IF(3, !proxy.IsEmpty(), "SIP\tOutbound proxy for endpoint set to " << proxy);
 }
 
 
@@ -1420,6 +1665,7 @@ unsigned SIPEndPoint::GetAllowedMethods() const
          (1<<SIP_PDU::Method_MESSAGE  )|
          (1<<SIP_PDU::Method_INFO     )|
          (1<<SIP_PDU::Method_PING     )|
+         (1<<SIP_PDU::Method_PRACK    )|
          (1<<SIP_PDU::Method_SUBSCRIBE);
 }
 
@@ -1440,16 +1686,34 @@ bool SIPEndPoint::GetAuthentication(const PString & realm, PString & user, PStri
 
 SIPURL SIPEndPoint::GetRegisteredProxy(const SIPURL & url)
 {
-  // Look up by the full URL first in case of multiple registrations to the same domain.
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(url.AsString(), SIP_PDU::Method_REGISTER, PSafeReadOnly);
-  if (handler == NULL) {
-    // Precise AOR not found, locate the name used for the domain.
-    handler = activeSIPHandlers.FindSIPHandlerByDomain(url.GetHostName(), SIP_PDU::Method_REGISTER, PSafeReadOnly);
-    if (handler == NULL) 
-      return SIPURL();
+  PSafePtr<SIPHandler> handler;
+
+  if (url.GetScheme() != "tel") {
+    // Look up by the full URL first in case of multiple registrations to the same domain.
+    if ((handler = activeSIPHandlers.FindSIPHandlerByUrl(url.AsString(), SIP_PDU::Method_REGISTER, PSafeReadOnly)) == NULL)
+      // Precise AOR not found, locate the name used for the domain.
+      handler = activeSIPHandlers.FindSIPHandlerByDomain(url.GetHostName(), SIP_PDU::Method_REGISTER, PSafeReadOnly);
+    return handler != NULL ? handler->GetProxy() : SIPURL();
   }
 
-  return handler->GetProxy();
+
+  PString domain = url.GetHostName();
+  if (domain.IsEmpty() || OpalIsE164(domain)) {
+    // No context, just get first registration
+    handler = activeSIPHandlers.GetFirstHandler();
+    while (handler != NULL && handler->GetMethod() != SIP_PDU::Method_REGISTER)
+      ++handler;
+  }
+  else
+    handler = activeSIPHandlers.FindSIPHandlerByDomain(domain, SIP_PDU::Method_REGISTER, PSafeReadOnly);
+
+  if (handler == NULL)
+    return SIPURL();
+
+  SIPURL proxy = handler->GetProxy();
+  if (proxy.IsEmpty())
+    return handler->GetAddressOfRecord();
+  return proxy;
 }
 
 
@@ -1464,6 +1728,7 @@ SIPURL SIPEndPoint::GetRegisteredPartyName(const SIPURL & url, const OpalTranspo
       return GetDefaultRegisteredPartyName(transport);
   }
 
+  PTRACE(4, "SIP\tGetting local URI from registeration: " << handler->GetAddressOfRecord());
   return handler->GetAddressOfRecord();
 }
 
@@ -1512,22 +1777,58 @@ SIPURL SIPEndPoint::GetDefaultRegisteredPartyName(const OpalTransport & transpor
   }
 
   rpn.SetDisplayName(GetDefaultDisplayName());
+  PTRACE(4, "SIP\tGenerated default local URI: " << rpn);
   return rpn;
 }
 
 
-SIPURL SIPEndPoint::GetContactURL(const OpalTransport &transport, const SIPURL & localURI)
+void SIPEndPoint::AdjustToRegistration(const OpalTransport &transport, SIP_PDU & pdu)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(localURI, SIP_PDU::Method_REGISTER, PSafeReadOnly);
-  if (handler == NULL) {
-    // Precise AOR not found, locate the name used for the domain.
-    handler = activeSIPHandlers.FindSIPHandlerByDomain(localURI.GetHostName(), SIP_PDU::Method_REGISTER, PSafeReadOnly);
-    if (handler == NULL)
-      return GetLocalURL(transport, localURI.GetUserName());
+  // When we start adding things like P-Asserted-Identity, this function will change
+  // Right now it just does the Contact field
+
+  SIPMIMEInfo & mime = pdu.GetMIME();
+
+  PString user = SIPURL(mime.GetFrom()).GetUserName();
+  PString domain = SIPURL(mime.GetTo()).GetHostName();
+
+  const SIPRegisterHandler * registrar = NULL;
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl("sip:"+user+'@'+domain, SIP_PDU::Method_REGISTER, PSafeReadOnly);
+  // If precise AOR not found, locate the name used for the domain.
+  if (handler == NULL)
+    handler = activeSIPHandlers.FindSIPHandlerByDomain(domain, SIP_PDU::Method_REGISTER, PSafeReadOnly);
+  if (handler != NULL) {
+    registrar = dynamic_cast<const SIPRegisterHandler *>(&*handler);
+    PAssertNULL(registrar);
   }
 
-  OpalTransport * handlerTransport = handler->GetTransport();
-  return GetLocalURL(handlerTransport != NULL ? *handlerTransport : transport, handler->GetAddressOfRecord().GetUserName());
+  if (!mime.Has("Contact")) {
+    SIPURL contact;
+    if (registrar != NULL) {
+      PIPSocket::Address ip;
+      bool transportLocal = transport.GetRemoteAddress().GetIpAddress(ip) && manager.IsLocalAddress(ip);
+
+      const SIPURLList & contacts = registrar->GetContacts();
+      for (SIPURLList::const_iterator it = contacts.begin(); it != contacts.end(); ++it) {
+        OpalTransportAddress contactAddress = it->GetHostAddress();
+        if (contactAddress.GetProto(true) == transport.GetProtoPrefix() &&
+            contactAddress.GetIpAddress(ip) &&
+            manager.IsLocalAddress(ip) == transportLocal) {
+          contact = *it;
+          break;
+        }
+      }
+    }
+
+    if (contact.IsEmpty())
+      contact = GetLocalURL(transport, user);
+
+    contact.Sanitise(SIPURL::ContactURI);
+    pdu.GetMIME().SetContact(contact.AsQuotedString());
+  }
+
+  if (!mime.Has("Route") && registrar != NULL)
+    mime.SetRoute(registrar->GetServiceRoute());
 }
 
 
@@ -1575,77 +1876,12 @@ void SIPEndPoint::OnRTPStatistics(const SIPConnection & connection,
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
-SIPEndPoint::WorkThreadPool::WorkerThreadBase * SIPEndPoint::WorkThreadPool::CreateWorkerThread()
+PThreadPoolBase::WorkerThreadBase * SIPEndPoint::WorkThreadPool::CreateWorkerThread()
 { 
-  return new SIP_Work_Thread(*this); 
+  return new QueuedWorkerThread(*this, PThread::HighPriority); 
 }
 
 
-SIPEndPoint::SIP_Work_Thread::SIP_Work_Thread(WorkThreadPool & pool_)
-  : WorkThreadPool::WorkerThread(pool_)
-{
-  SetPriority(HighPriority);
-}
-
-
-unsigned SIPEndPoint::SIP_Work_Thread::GetWorkSize() const 
-{ 
-  return m_pduQueue.size(); 
-}
-
-
-void SIPEndPoint::SIP_Work_Thread::AddWork(SIP_Work * work)
-{
-  PWaitAndSignal m(m_workerMutex);
-  m_pduQueue.push(work);
-  if (m_pduQueue.size() == 1)
-    m_sync.Signal();
-}
-
-
-void SIPEndPoint::SIP_Work_Thread::RemoveWork(SIP_Work * work)
-{
-  m_workerMutex.Wait();
-  m_pduQueue.pop();
-  m_workerMutex.Signal();
-
-  delete work;
-}
-
-
-void SIPEndPoint::SIP_Work_Thread::Shutdown()
-{
-  m_shutdown = true;
-  m_sync.Signal();
-}
-
-
-void SIPEndPoint::SIP_Work_Thread::Main()
-{
-  while (!m_shutdown) {
-
-    // get the work
-    m_workerMutex.Wait();
-    SIP_Work * work = m_pduQueue.size() > 0 ? m_pduQueue.front() : NULL;
-    m_workerMutex.Signal();
-
-    // wait for work to become available
-    if (work == NULL) {
-      m_sync.Wait();
-      continue;
-    }
-
-    // process the work
-    work->Process();
-
-    // indicate work is now free
-    m_pool.RemoveWork(work);
-  }
-}
-
-
-///////////////////////////////////////////////////////////////////////////////////////////////
-
 SIPEndPoint::SIP_Work::SIP_Work(SIPEndPoint & ep, SIP_PDU * pdu, const PString & token)
   : m_endpoint(ep)
   , m_pdu(pdu)
@@ -1662,7 +1898,7 @@ SIPEndPoint::SIP_Work::~SIP_Work()
 }
 
 
-void SIPEndPoint::SIP_Work::Process()
+void SIPEndPoint::SIP_Work::Work()
 {
   if (PAssertNULL(m_pdu) == NULL)
     return;
